@@ -47,6 +47,7 @@
 #import "java/lang/NoSuchMethodException.h"
 #import "java/lang/NullPointerException.h"
 #import "java/lang/Package.h"
+#import "java/lang/Record.h"
 #import "java/lang/StringBuilder.h"
 #import "java/lang/annotation/Annotation.h"
 #import "java/lang/annotation/Inherited.h"
@@ -54,6 +55,7 @@
 #import "java/lang/reflect/Field.h"
 #import "java/lang/reflect/Method.h"
 #import "java/lang/reflect/Modifier.h"
+#import "java/lang/reflect/RecordComponent.h"
 #import "java/lang/reflect/TypeVariable.h"
 #import "java/util/ArrayList.h"
 #import "java/util/Iterator.h"
@@ -225,7 +227,7 @@ static LibcoreReflectGenericSignatureParser *NewParsedClassSignature(IOSClass *c
 }
 
 // Returns true if an object is an instance of this class.
-- (jboolean)isInstance:(id)object {
+- (bool)isInstance:(id)object {
   return false;
 }
 
@@ -378,7 +380,7 @@ static NSString *Capitalize(NSString *s) {
   @throw create_JavaLangNoSuchMethodException_init();
 }
 
-- (jboolean)isAssignableFrom:(IOSClass *)cls {
+- (bool)isAssignableFrom:(IOSClass *)cls {
   @throw create_JavaLangAssertionError_initWithId_(@"abstract method not overridden");
 }
 
@@ -424,7 +426,7 @@ static NSString *Capitalize(NSString *s) {
     [sb appendWithNSString:[self getName]];
     IOSObjectArray *typeparms = [self getTypeParameters];
     if (((IOSObjectArray *) nil_chk(typeparms))->size_ > 0) {
-      jboolean first = true;
+      bool first = true;
       [sb appendWithChar:'<'];
       for (id<JavaLangReflectTypeVariable> typeparm in typeparms) {
         if (!first) {
@@ -681,14 +683,14 @@ IOSClass *IOSClass_forName_(NSString *className) {
   return IOSClass_forName_(className);
 }
 
-IOSClass *IOSClass_forName_initialize_classLoader_(
-    NSString *className, jboolean load, JavaLangClassLoader *loader) {
+IOSClass *IOSClass_forName_initialize_classLoader_(NSString *className, bool load,
+                                                   JavaLangClassLoader *loader) {
   IOSClass_initialize();
   return IOSClass_forName_(className);
 }
 
 + (IOSClass *)forName:(NSString *)className
-           initialize:(jboolean)load
+           initialize:(bool)load
           classLoader:(JavaLangClassLoader *)loader {
   return IOSClass_forName_initialize_classLoader_(className, load, loader);
 }
@@ -708,40 +710,44 @@ IOSClass *IOSClass_forName_initialize_classLoader_(
   return enclosingClass ? JreClassForString(enclosingClass) : nil;
 }
 
-- (jboolean)isArray {
+- (bool)isArray {
   return false;
 }
 
-- (jboolean)isEnum {
+- (bool)isEnum {
   return false;
 }
 
-- (jboolean)isInterface {
+- (bool)isRecord {
+  return [self getSuperclass] == JavaLangRecord_class_();
+}
+
+- (bool)isInterface {
   return false;
 }
 
-- (jboolean)isPrimitive {
+- (bool)isPrimitive {
   return false;  // Overridden by IOSPrimitiveClass.
 }
 
-static jboolean hasModifier(IOSClass *cls, int flag) {
+static bool hasModifier(IOSClass *cls, int flag) {
   return cls->metadata_ ? (cls->metadata_->modifiers & flag) > 0 : false;
 }
 
-- (jboolean)isAnnotation {
+- (bool)isAnnotation {
   return hasModifier(self, JavaLangReflectModifier_ANNOTATION);
 }
 
-- (jboolean)isMemberClass {
+- (bool)isMemberClass {
   return metadata_ && JrePtrAtIndex(metadata_->ptrTable, metadata_->enclosingClassIdx)
       && ![self isAnonymousClass];
 }
 
-- (jboolean)isLocalClass {
+- (bool)isLocalClass {
   return [self getEnclosingMethod] && ![self isAnonymousClass];
 }
 
-- (jboolean)isSynthetic {
+- (bool)isSynthetic {
   return hasModifier(self, JavaLangReflectModifier_SYNTHETIC);
 }
 
@@ -829,7 +835,7 @@ IOSObjectArray *IOSClass_NewInterfacesFromProtocolList(
   return nil;
 }
 
-- (jboolean)isAnnotationPresentWithIOSClass:(IOSClass *)annotationClass {
+- (bool)isAnnotationPresentWithIOSClass:(IOSClass *)annotationClass {
   return [self getAnnotationWithIOSClass:annotationClass] != nil;
 }
 
@@ -931,8 +937,7 @@ IOSObjectArray *IOSClass_NewInterfacesFromProtocolList(
 }
 
 // Adds all the fields for a specified class to a specified dictionary.
-static void GetFieldsFromClass(IOSClass *iosClass, NSMutableDictionary *fields,
-    jboolean publicOnly) {
+static void GetFieldsFromClass(IOSClass *iosClass, NSMutableDictionary *fields, bool publicOnly) {
   const J2ObjcClassInfo *metadata = IOSClass_GetMetadataOrFail(iosClass);
   for (int i = 0; i < metadata->fieldCount; i++) {
     const J2ObjcFieldInfo *fieldInfo = &metadata->fields[i];
@@ -1032,8 +1037,8 @@ static void getAllFields(IOSClass *cls, NSMutableDictionary *fieldMap) {
 
 
 // Adds all the inner classes for a specified class to a specified dictionary.
-static void GetInnerClasses(IOSClass *iosClass, NSMutableArray *classes,
-    jboolean publicOnly, jboolean includeInterfaces) {
+static void GetInnerClasses(IOSClass *iosClass, NSMutableArray *classes, bool publicOnly,
+                            bool includeInterfaces) {
   const J2ObjcClassInfo *metadata = iosClass->metadata_;
   if (metadata) {
     IOSObjectArray *innerClasses = JreParseClassList(
@@ -1076,11 +1081,11 @@ static void GetInnerClasses(IOSClass *iosClass, NSMutableArray *classes,
   return result;
 }
 
-- (jboolean)isAnonymousClass {
+- (bool)isAnonymousClass {
   return false;
 }
 
-- (jboolean)desiredAssertionStatus {
+- (bool)desiredAssertionStatus {
   return false;
 }
 
@@ -1092,6 +1097,40 @@ static IOSObjectArray *GetEnumConstants(IOSClass *cls) {
   return [GetEnumConstants(self) java_clone];
 }
 
+- (IOSObjectArray *)getRecordComponents {
+  if (![self isRecord]) {
+    return nil;
+  }
+  // Return record components in declaration order, which is how they're stored in metadata.
+  const J2ObjcClassInfo *metadata = IOSClass_GetMetadataOrFail(self);
+  uint16_t fieldCount = metadata->fieldCount;
+  NSMutableArray *componentFields = [[NSMutableArray alloc] initWithCapacity:fieldCount];
+  for (uint16_t i = 0; i < fieldCount; i++) {
+    const J2ObjcFieldInfo *fieldInfo = &metadata->fields[i];
+    if (fieldInfo->modifiers == (JavaLangReflectModifier_PRIVATE | JavaLangReflectModifier_FINAL)) {
+      [componentFields addObject:[self getDeclaredField:
+          [NSString stringWithUTF8String:fieldInfo->name]]];
+    }
+  }
+
+  uint16_t count = (jint)[componentFields count];
+  IOSObjectArray *result =
+      [IOSObjectArray arrayWithLength:count type:JavaLangReflectRecordComponent_class_()];
+  for (uint16_t i = 0; i < count; i++) {
+    JavaLangReflectField *field = [componentFields objectAtIndex:i];
+    NSString *name = [field getName];
+    JavaLangReflectMethod *accessor = [self getMethod:name parameterTypes:nil];
+    JavaLangReflectRecordComponent *comp = 
+          [[JavaLangReflectRecordComponent alloc] initWithIOSClass:self
+                                          withJavaLangReflectField:field
+                                         withJavaLangReflectMethod:accessor];
+    [result replaceObjectAtIndex:i withObject:comp];
+    [comp release];
+  }
+  [componentFields release];
+  return result;
+}
+
 // Package private method. In OpenJDK it differentiated from the above because
 // a single constants array is cached and then cloned by getEnumConstants().
 // That's not necessary here, since the Enum.getSharedConstants() function
@@ -1142,7 +1181,7 @@ NSString *resolveResourceName(IOSClass *cls, NSString *resourceName) {
   return (id)rawValue->asId;
 }
 
-- (jboolean)__unboxValue:(id)value toRawValue:(J2ObjcRawValue *)rawValue {
+- (bool)__unboxValue:(id)value toRawValue:(J2ObjcRawValue *)rawValue {
   rawValue->asId = value;
   return true;
 }
@@ -1155,7 +1194,7 @@ NSString *resolveResourceName(IOSClass *cls, NSString *resourceName) {
   *(id *)addr = (id)rawValue->asId;
 }
 
-- (jboolean)__convertRawValue:(J2ObjcRawValue *)rawValue toType:(IOSClass *)type {
+- (bool)__convertRawValue:(J2ObjcRawValue *)rawValue toType:(IOSClass *)type {
   // No conversion necessary if both types are ids.
   return ![type isPrimitive];
 }
@@ -1170,7 +1209,7 @@ NSString *resolveResourceName(IOSClass *cls, NSString *resourceName) {
   return IOSClass_class_();
 }
 
-static jboolean IsStringType(Class cls) {
+static bool IsStringType(Class cls) {
   // We can't trigger class initialization because that might recursively enter
   // FetchClass and result in deadlock within the FastPointerLookup. Therefore,
   // we can't use [cls isSubclassOfClass:[NSString class]].
@@ -1304,6 +1343,7 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     { NULL, "[LJavaLangReflectConstructor;", 0x1, -1, -1, 20, 21, -1, -1 },
     { NULL, "LJavaLangReflectField;", 0x1, 23, 1, 24, -1, -1, -1 },
     { NULL, "[LJavaLangReflectField;", 0x1, -1, -1, -1, -1, -1, -1 },
+    { NULL, "[LJavaLangReflectRecordComponent;", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "LJavaLangReflectMethod;", 0x81, 25, 26, 18, 27, -1, -1 },
     { NULL, "[LJavaLangReflectMethod;", 0x1, -1, -1, 20, -1, -1, -1 },
     { NULL, "LIOSClass;", 0x1, -1, -1, -1, 15, -1, -1 },
@@ -1336,6 +1376,7 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     { NULL, "Z", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "Z", 0x1, 38, 7, -1, 39, -1, -1 },
     { NULL, "Z", 0x1, -1, -1, -1, -1, -1, -1 },
+    { NULL, "Z", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "Z", 0x1, 40, 10, -1, -1, -1, -1 },
     { NULL, "Z", 0x1, -1, -1, -1, -1, -1, -1 },
     { NULL, "Z", 0x1, -1, -1, -1, -1, -1, -1 },
@@ -1373,51 +1414,53 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
   methods[17].selector = @selector(getDeclaredConstructors);
   methods[18].selector = @selector(getDeclaredField:);
   methods[19].selector = @selector(getDeclaredFields);
-  methods[20].selector = @selector(getDeclaredMethod:parameterTypes:);
-  methods[21].selector = @selector(getDeclaredMethods);
-  methods[22].selector = @selector(getDeclaringClass);
-  methods[23].selector = @selector(getEnclosingClass);
-  methods[24].selector = @selector(getEnclosingConstructor);
-  methods[25].selector = @selector(getEnclosingMethod);
-  methods[26].selector = @selector(getEnumConstants);
-  methods[27].selector = @selector(getEnumConstantsShared);
-  methods[28].selector = @selector(getField:);
-  methods[29].selector = @selector(getFields);
-  methods[30].selector = @selector(getGenericInterfaces);
-  methods[31].selector = @selector(getGenericSuperclass);
-  methods[32].selector = @selector(getInterfaces);
-  methods[33].selector = @selector(getMethod:parameterTypes:);
-  methods[34].selector = @selector(getMethods);
-  methods[35].selector = @selector(getModifiers);
-  methods[36].selector = @selector(getName);
-  methods[37].selector = @selector(getPackage);
-  methods[38].selector = @selector(getPackageName);
-  methods[39].selector = @selector(getProtectionDomain);
-  methods[40].selector = @selector(getResource:);
-  methods[41].selector = @selector(getResourceAsStream:);
-  methods[42].selector = @selector(getSigners);
-  methods[43].selector = @selector(getSimpleName);
-  methods[44].selector = @selector(getSuperclass);
-  methods[45].selector = @selector(getTypeParameters);
-  methods[46].selector = @selector(isAnnotation);
-  methods[47].selector = @selector(isAnnotationPresentWithIOSClass:);
-  methods[48].selector = @selector(isAnonymousClass);
-  methods[49].selector = @selector(isArray);
-  methods[50].selector = @selector(isAssignableFrom:);
-  methods[51].selector = @selector(isEnum);
-  methods[52].selector = @selector(isInstance:);
-  methods[53].selector = @selector(isInterface);
-  methods[54].selector = @selector(isLocalClass);
-  methods[55].selector = @selector(isMemberClass);
-  methods[56].selector = @selector(isPrimitive);
-  methods[57].selector = @selector(isSynthetic);
-  methods[58].selector = @selector(newInstance);
-  methods[59].selector = @selector(description);
-  methods[60].selector = @selector(getDeclaredAnnotationsByTypeWithIOSClass:);
-  methods[61].selector = @selector(getAnnotationsByTypeWithIOSClass:);
-  methods[62].selector = @selector(getDeclaredAnnotationWithIOSClass:);
-  methods[63].selector = @selector(getTypeName);
-  methods[64].selector = @selector(toGenericString);
+  methods[20].selector = @selector(getRecordComponents);
+  methods[21].selector = @selector(getDeclaredMethod:parameterTypes:);
+  methods[22].selector = @selector(getDeclaredMethods);
+  methods[23].selector = @selector(getDeclaringClass);
+  methods[24].selector = @selector(getEnclosingClass);
+  methods[25].selector = @selector(getEnclosingConstructor);
+  methods[26].selector = @selector(getEnclosingMethod);
+  methods[27].selector = @selector(getEnumConstants);
+  methods[28].selector = @selector(getEnumConstantsShared);
+  methods[29].selector = @selector(getField:);
+  methods[30].selector = @selector(getFields);
+  methods[31].selector = @selector(getGenericInterfaces);
+  methods[32].selector = @selector(getGenericSuperclass);
+  methods[33].selector = @selector(getInterfaces);
+  methods[34].selector = @selector(getMethod:parameterTypes:);
+  methods[35].selector = @selector(getMethods);
+  methods[36].selector = @selector(getModifiers);
+  methods[37].selector = @selector(getName);
+  methods[38].selector = @selector(getPackage);
+  methods[39].selector = @selector(getPackageName);
+  methods[40].selector = @selector(getProtectionDomain);
+  methods[41].selector = @selector(getResource:);
+  methods[42].selector = @selector(getResourceAsStream:);
+  methods[43].selector = @selector(getSigners);
+  methods[44].selector = @selector(getSimpleName);
+  methods[45].selector = @selector(getSuperclass);
+  methods[46].selector = @selector(getTypeParameters);
+  methods[47].selector = @selector(isAnnotation);
+  methods[48].selector = @selector(isAnnotationPresentWithIOSClass:);
+  methods[49].selector = @selector(isAnonymousClass);
+  methods[50].selector = @selector(isArray);
+  methods[51].selector = @selector(isAssignableFrom:);
+  methods[52].selector = @selector(isEnum);
+  methods[53].selector = @selector(isRecord);
+  methods[54].selector = @selector(isInstance:);
+  methods[55].selector = @selector(isInterface);
+  methods[56].selector = @selector(isLocalClass);
+  methods[57].selector = @selector(isMemberClass);
+  methods[58].selector = @selector(isPrimitive);
+  methods[59].selector = @selector(isSynthetic);
+  methods[60].selector = @selector(newInstance);
+  methods[61].selector = @selector(description);
+  methods[62].selector = @selector(getDeclaredAnnotationsByTypeWithIOSClass:);
+  methods[63].selector = @selector(getAnnotationsByTypeWithIOSClass:);
+  methods[64].selector = @selector(getDeclaredAnnotationWithIOSClass:);
+  methods[65].selector = @selector(getTypeName);
+  methods[66].selector = @selector(toGenericString);
   #pragma clang diagnostic pop
   static const J2ObjcFieldInfo fields[] = {
     { "serialVersionUID", "J", .constantValue.asLong = IOSClass_serialVersionUID, 0x1a, -1, -1, -1,
@@ -1449,7 +1492,7 @@ IOSClass *IOSClass_arrayType(IOSClass *componentType, jint dimensions) {
     "Ljava/lang/reflect/GenericDeclaration;Ljava/lang/reflect/Type;"
     "Ljava/lang/reflect/AnnotatedElement;" };
   static const J2ObjcClassInfo _IOSClass = {
-    "Class", "java.lang", ptrTable, methods, fields, 7, 0x11, 65, 1, -1, -1, -1, 48, -1 };
+    "Class", "java.lang", ptrTable, methods, fields, 7, 0x11, 67, 1, -1, -1, -1, 48, -1 };
   return &_IOSClass;
 }
 
@@ -1499,8 +1542,9 @@ J2OBJC_NAME_MAPPING(IOSClass, "java.lang.Class", "IOSClass")
 @implementation PackagePrefixLoader
 
 - (void)load__WithNSString:(NSString *)key withNSString:(NSString *)value {
-  [nil_chk(prefixMapping) addWithId:[[PackagePrefixEntry alloc] initWithNSString:key
-                                                                    withNSString:value]];
+  PackagePrefixEntry *entry = [[PackagePrefixEntry alloc] initWithNSString:key withNSString:value];
+  [nil_chk(prefixMapping) addWithId:entry];
+  [entry release];
 }
 
 @end
