@@ -56,7 +56,7 @@
   /**
    * The elements of this array.
    */
-  jboolean buffer_[0];
+  bool buffer_[0];
 }
 
 /**
@@ -72,43 +72,43 @@
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithBooleans:(const jboolean *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithBooleans:(const bool *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithBooleans:(const jboolean *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithBooleans:(const bool *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of booleans.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of booleans.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the boolean at the specified index.
  */
-- (jboolean)booleanAtIndex:(NSUInteger)index;
+- (bool)booleanAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the boolean at the specified index.
  */
-- (jboolean *)booleanRefAtIndex:(NSUInteger)index;
+- (bool *)booleanRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the boolean at the specified index. Return the new value.
  */
-- (jboolean)replaceBooleanAtIndex:(NSUInteger)index withBoolean:(jboolean)value;
+- (bool)replaceBooleanAtIndex:(NSUInteger)index withBoolean:(bool)value;
 
 /**
  * Copy this array's booleans to a buffer.
  */
-- (void)getBooleans:(jboolean *)buffer length:(NSUInteger)length;
+- (void)getBooleans:(bool *)buffer length:(NSUInteger)length;
 
 @end
 
@@ -116,8 +116,8 @@
  * @brief Return the boolean at the specified index.
  * Equivalent to booleanAtIndex:.
  */
-__attribute__((always_inline)) inline jboolean IOSBooleanArray_Get(
-    __unsafe_unretained IOSBooleanArray *array, jint index) {
+__attribute__((always_inline)) inline bool IOSBooleanArray_Get(
+    __unsafe_unretained IOSBooleanArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -126,13 +126,12 @@ __attribute__((always_inline)) inline jboolean IOSBooleanArray_Get(
  * @brief Return a pointer to the boolean at the specified index.
  * Equivalent to booleanRefAtIndex:.
  */
-__attribute__((always_inline)) inline jboolean *IOSBooleanArray_GetRef(
-    __unsafe_unretained IOSBooleanArray *array, jint index) {
+__attribute__((always_inline)) inline bool *IOSBooleanArray_GetRef(
+    __unsafe_unretained IOSBooleanArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
 
-
 // ********** IOSCharArray **********
 
 /**
@@ -144,7 +143,7 @@ __attribute__((always_inline)) inline jboolean *IOSBooleanArray_GetRef(
   /**
    * The elements of this array.
    */
-  jchar buffer_[0];
+  uint16_t buffer_[0];
 }
 
 /**
@@ -160,43 +159,43 @@ __attribute__((always_inline)) inline jboolean *IOSBooleanArray_GetRef(
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithChars:(const jchar *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithChars:(const uint16_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithChars:(const jchar *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithChars:(const uint16_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of chars.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of chars.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the L_NAME at the specified index.
  */
-- (jchar)charAtIndex:(NSUInteger)index;
+- (uint16_t)charAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the char at the specified index.
  */
-- (jchar *)charRefAtIndex:(NSUInteger)index;
+- (uint16_t *)charRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the char at the specified index. Return the new value.
  */
-- (jchar)replaceCharAtIndex:(NSUInteger)index withChar:(jchar)value;
+- (uint16_t)replaceCharAtIndex:(NSUInteger)index withChar:(uint16_t)value;
 
 /**
  * Copy this array's chars to a buffer.
  */
-- (void)getChars:(jchar *)buffer length:(NSUInteger)length;
+- (void)getChars:(uint16_t *)buffer length:(NSUInteger)length;
 
 /**
  * Create a char array from an NSString.
@@ -209,8 +208,8 @@ __attribute__((always_inline)) inline jboolean *IOSBooleanArray_GetRef(
  * @brief Return the char at the specified index.
  * Equivalent to charAtIndex:.
  */
-__attribute__((always_inline)) inline jchar IOSCharArray_Get(
-    __unsafe_unretained IOSCharArray *array, jint index) {
+__attribute__((always_inline)) inline uint16_t IOSCharArray_Get(
+    __unsafe_unretained IOSCharArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -219,8 +218,8 @@ __attribute__((always_inline)) inline jchar IOSCharArray_Get(
  * @brief Return a pointer to the char at the specified index.
  * Equivalent to charRefAtIndex:.
  */
-__attribute__((always_inline)) inline jchar *IOSCharArray_GetRef(
-    __unsafe_unretained IOSCharArray *array, jint index) {
+__attribute__((always_inline)) inline uint16_t *IOSCharArray_GetRef(
+    __unsafe_unretained IOSCharArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
@@ -237,7 +236,7 @@ __attribute__((always_inline)) inline jchar *IOSCharArray_GetRef(
   /**
    * The elements of this array.
    */
-  jbyte buffer_[0];
+  int8_t buffer_[0];
 }
 
 /**
@@ -253,43 +252,43 @@ __attribute__((always_inline)) inline jchar *IOSCharArray_GetRef(
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithBytes:(const jbyte *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithBytes:(const int8_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithBytes:(const jbyte *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithBytes:(const int8_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of bytes.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of bytes.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the byte at the specified index.
  */
-- (jbyte)byteAtIndex:(NSUInteger)index;
+- (int8_t)byteAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the byte at the specified index.
  */
-- (jbyte *)byteRefAtIndex:(NSUInteger)index;
+- (int8_t *)byteRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the byte at the specified index. Return the new value.
  */
-- (jbyte)replaceByteAtIndex:(NSUInteger)index withByte:(jbyte)value;
+- (int8_t)replaceByteAtIndex:(NSUInteger)index withByte:(int8_t)value;
 
 /**
  * Copy this array's bytes to a buffer.
  */
-- (void)getBytes:(jbyte *)buffer length:(NSUInteger)length;
+- (void)getBytes:(int8_t *)buffer length:(NSUInteger)length;
 
 // Create an array from an NSData object.
 + (instancetype)arrayWithNSData:(NSData *)data;
@@ -297,14 +296,14 @@ __attribute__((always_inline)) inline jchar *IOSCharArray_GetRef(
 // Copies the array contents into a specified buffer, up to the specified
 // length.  An IndexOutOfBoundsException is thrown if the specified length
 // is greater than the array size.
-- (void)getBytes:(jbyte *)buffer
-          offset:(jint)offset
-          length:(jint)length;
+- (void)getBytes:(int8_t *)buffer
+          offset:(int32_t)offset
+          length:(int32_t)length;
 
 // Copies the specified native buffer into this array at the specified offset.
-- (void)replaceBytes:(const jbyte *)source
-              length:(jint)length
-              offset:(jint)destOffset;
+- (void)replaceBytes:(const int8_t *)source
+              length:(int32_t)length
+              offset:(int32_t)destOffset;
 
 // Returns the bytes of the array encapsulated in an NSData *. Copies the
 // underlying data.
@@ -316,8 +315,8 @@ __attribute__((always_inline)) inline jchar *IOSCharArray_GetRef(
  * @brief Return the byte at the specified index.
  * Equivalent to byteAtIndex:.
  */
-__attribute__((always_inline)) inline jbyte IOSByteArray_Get(
-    __unsafe_unretained IOSByteArray *array, jint index) {
+__attribute__((always_inline)) inline int8_t IOSByteArray_Get(
+    __unsafe_unretained IOSByteArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -326,8 +325,8 @@ __attribute__((always_inline)) inline jbyte IOSByteArray_Get(
  * @brief Return a pointer to the byte at the specified index.
  * Equivalent to byteRefAtIndex:.
  */
-__attribute__((always_inline)) inline jbyte *IOSByteArray_GetRef(
-    __unsafe_unretained IOSByteArray *array, jint index) {
+__attribute__((always_inline)) inline int8_t *IOSByteArray_GetRef(
+    __unsafe_unretained IOSByteArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
@@ -344,7 +343,7 @@ __attribute__((always_inline)) inline jbyte *IOSByteArray_GetRef(
   /**
    * The elements of this array.
    */
-  jshort buffer_[0];
+  int16_t buffer_[0];
 }
 
 /**
@@ -360,43 +359,43 @@ __attribute__((always_inline)) inline jbyte *IOSByteArray_GetRef(
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithShorts:(const jshort *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithShorts:(const int16_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithShorts:(const jshort *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithShorts:(const int16_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of shorts.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of shorts.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the short at the specified index.
  */
-- (jshort)shortAtIndex:(NSUInteger)index;
+- (int16_t)shortAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the short at the specified index.
  */
-- (jshort *)shortRefAtIndex:(NSUInteger)index;
+- (int16_t *)shortRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the short at the specified index. Return the new value.
  */
-- (jshort)replaceShortAtIndex:(NSUInteger)index withShort:(jshort)value;
+- (int16_t)replaceShortAtIndex:(NSUInteger)index withShort:(int16_t)value;
 
 /**
  * Copy this array's shorts to a buffer.
  */
-- (void)getShorts:(jshort *)buffer length:(NSUInteger)length;
+- (void)getShorts:(int16_t *)buffer length:(NSUInteger)length;
 
 @end
 
@@ -404,8 +403,8 @@ __attribute__((always_inline)) inline jbyte *IOSByteArray_GetRef(
  * @brief Return the short at the specified index.
  * Equivalent to shortAtIndex:.
  */
-__attribute__((always_inline)) inline jshort IOSShortArray_Get(
-    __unsafe_unretained IOSShortArray *array, jint index) {
+__attribute__((always_inline)) inline int16_t IOSShortArray_Get(
+    __unsafe_unretained IOSShortArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -414,8 +413,8 @@ __attribute__((always_inline)) inline jshort IOSShortArray_Get(
  * @brief Return a pointer to the short at the specified index.
  * Equivalent to shortRefAtIndex:.
  */
-__attribute__((always_inline)) inline jshort *IOSShortArray_GetRef(
-    __unsafe_unretained IOSShortArray *array, jint index) {
+__attribute__((always_inline)) inline int16_t *IOSShortArray_GetRef(
+    __unsafe_unretained IOSShortArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
@@ -433,7 +432,7 @@ __attribute__((always_inline)) inline jshort *IOSShortArray_GetRef(
    * The elements of this array.
    */
   // Ensure alignment for java.util.concurrent.atomic.AtomicIntegerArray.
-  jint buffer_[0] __attribute__((aligned(__alignof__(volatile_jint))));
+  int32_t buffer_[0] __attribute__((aligned(__alignof__(volatile_int32_t))));
 }
 
 /**
@@ -449,43 +448,43 @@ __attribute__((always_inline)) inline jshort *IOSShortArray_GetRef(
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithInts:(const jint *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithInts:(const int32_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithInts:(const jint *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithInts:(const int32_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of ints.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of ints.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the int at the specified index.
  */
-- (jint)intAtIndex:(NSUInteger)index;
+- (int32_t)intAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the int at the specified index.
  */
-- (jint *)intRefAtIndex:(NSUInteger)index;
+- (int32_t *)intRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the int at the specified index. Return the new value.
  */
-- (jint)replaceIntAtIndex:(NSUInteger)index withInt:(jint)value;
+- (int32_t)replaceIntAtIndex:(NSUInteger)index withInt:(int32_t)value;
 
 /**
  * Copy this array's ints to a buffer.
  */
-- (void)getInts:(jint *)buffer length:(NSUInteger)length;
+- (void)getInts:(int32_t *)buffer length:(NSUInteger)length;
 
 @end
 
@@ -493,8 +492,8 @@ __attribute__((always_inline)) inline jshort *IOSShortArray_GetRef(
  * @brief Return the int at the specified index.
  * Equivalent to shortAtIndex:.
  */
-__attribute__((always_inline)) inline jint IOSIntArray_Get(
-    __unsafe_unretained IOSIntArray *array, jint index) {
+__attribute__((always_inline)) inline int32_t IOSIntArray_Get(
+    __unsafe_unretained IOSIntArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -503,8 +502,8 @@ __attribute__((always_inline)) inline jint IOSIntArray_Get(
  * @brief Return a pointer to the int at the specified index.
  * Equivalent to shortRefAtIndex:.
  */
-__attribute__((always_inline)) inline jint *IOSIntArray_GetRef(
-    __unsafe_unretained IOSIntArray *array, jint index) {
+__attribute__((always_inline)) inline int32_t *IOSIntArray_GetRef(
+    __unsafe_unretained IOSIntArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
@@ -522,7 +521,7 @@ __attribute__((always_inline)) inline jint *IOSIntArray_GetRef(
    * The elements of this array.
    */
   // Ensure alignment for java.util.concurrent.atomic.AtomicLongArray.
-  jlong buffer_[0] __attribute__((aligned(__alignof__(volatile_jlong))));
+  int64_t buffer_[0] __attribute__((aligned(__alignof__(volatile_int64_t))));
 }
 
 /**
@@ -538,43 +537,43 @@ __attribute__((always_inline)) inline jint *IOSIntArray_GetRef(
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithLongs:(const jlong *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithLongs:(const int64_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithLongs:(const jlong *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithLongs:(const int64_t *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of longs.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of longs.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the long at the specified index.
  */
-- (jlong)longAtIndex:(NSUInteger)index;
+- (int64_t)longAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the long at the specified index.
  */
-- (jlong *)longRefAtIndex:(NSUInteger)index;
+- (int64_t *)longRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the long at the specified index. Return the new value.
  */
-- (jlong)replaceLongAtIndex:(NSUInteger)index withLong:(jlong)value;
+- (int64_t)replaceLongAtIndex:(NSUInteger)index withLong:(int64_t)value;
 
 /**
  * Copy this array's longs to a buffer.
  */
-- (void)getLongs:(jlong *)buffer length:(NSUInteger)length;
+- (void)getLongs:(int64_t *)buffer length:(NSUInteger)length;
 
 @end
 
@@ -582,8 +581,8 @@ __attribute__((always_inline)) inline jint *IOSIntArray_GetRef(
  * @brief Return the long at the specified index.
  * Equivalent to longAtIndex:.
  */
-__attribute__((always_inline)) inline jlong IOSLongArray_Get(
-    __unsafe_unretained IOSLongArray *array, jint index) {
+__attribute__((always_inline)) inline int64_t IOSLongArray_Get(
+    __unsafe_unretained IOSLongArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -592,8 +591,8 @@ __attribute__((always_inline)) inline jlong IOSLongArray_Get(
  * @brief Return a pointer to the long at the specified index.
  * Equivalent to longRefAtIndex:.
  */
-__attribute__((always_inline)) inline jlong *IOSLongArray_GetRef(
-    __unsafe_unretained IOSLongArray *array, jint index) {
+__attribute__((always_inline)) inline int64_t *IOSLongArray_GetRef(
+    __unsafe_unretained IOSLongArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
@@ -610,7 +609,7 @@ __attribute__((always_inline)) inline jlong *IOSLongArray_GetRef(
   /**
    * The elements of this array.
    */
-  jfloat buffer_[0];
+  float buffer_[0];
 }
 
 /**
@@ -626,43 +625,43 @@ __attribute__((always_inline)) inline jlong *IOSLongArray_GetRef(
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithFloats:(const jfloat *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithFloats:(const float *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithFloats:(const jfloat *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithFloats:(const float *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of floats.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of floats.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the float at the specified index.
  */
-- (jfloat)floatAtIndex:(NSUInteger)index;
+- (float)floatAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the float at the specified index.
  */
-- (jfloat *)floatRefAtIndex:(NSUInteger)index;
+- (float *)floatRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the float at the specified index. Return the new value.
  */
-- (jfloat)replaceFloatAtIndex:(NSUInteger)index withFloat:(jfloat)value;
+- (float)replaceFloatAtIndex:(NSUInteger)index withFloat:(float)value;
 
 /**
  * Copy this array's floats to a buffer.
  */
-- (void)getFloats:(jfloat *)buffer length:(NSUInteger)length;
+- (void)getFloats:(float *)buffer length:(NSUInteger)length;
 
 @end
 
@@ -670,8 +669,8 @@ __attribute__((always_inline)) inline jlong *IOSLongArray_GetRef(
  * @brief Return the float at the specified index.
  * Equivalent to floatAtIndex:.
  */
-__attribute__((always_inline)) inline jfloat IOSFloatArray_Get(
-    __unsafe_unretained IOSFloatArray *array, jint index) {
+__attribute__((always_inline)) inline float IOSFloatArray_Get(
+    __unsafe_unretained IOSFloatArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -680,8 +679,8 @@ __attribute__((always_inline)) inline jfloat IOSFloatArray_Get(
  * @brief Return a pointer to the float at the specified index.
  * Equivalent to floatRefAtIndex:.
  */
-__attribute__((always_inline)) inline jfloat *IOSFloatArray_GetRef(
-    __unsafe_unretained IOSFloatArray *array, jint index) {
+__attribute__((always_inline)) inline float *IOSFloatArray_GetRef(
+    __unsafe_unretained IOSFloatArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
@@ -698,7 +697,7 @@ __attribute__((always_inline)) inline jfloat *IOSFloatArray_GetRef(
   /**
    * The elements of this array.
    */
-  jdouble buffer_[0];
+  double buffer_[0];
 }
 
 /**
@@ -714,43 +713,43 @@ __attribute__((always_inline)) inline jfloat *IOSFloatArray_GetRef(
 /**
  * Create a new array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)newArrayWithDoubles:(const jdouble *)buf count:(NSUInteger)count;
++ (instancetype)newArrayWithDoubles:(const double *)buf count:(NSUInteger)count;
 
 /**
  * Create a new autoreleased array of a specified length, setting the elements to the values in buf.
  */
-+ (instancetype)arrayWithDoubles:(const jdouble *)buf count:(NSUInteger)count;
++ (instancetype)arrayWithDoubles:(const double *)buf count:(NSUInteger)count;
 
 /**
  * Create a new multi-dimensional array of doubles.
  */
-+ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths;
++ (id)arrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths;
 
 /**
  * Create a new autoreleased multi-dimensional array of doubles.
  */
-+ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const jint *)dimensionLengths
++ (id)newArrayWithDimensions:(NSUInteger)dimensionCount lengths:(const int32_t *)dimensionLengths
     __attribute__((objc_method_family(none), ns_returns_retained));
 
 /**
  * Return the double at the specified index.
  */
-- (jdouble)doubleAtIndex:(NSUInteger)index;
+- (double)doubleAtIndex:(NSUInteger)index;
 
 /**
  * Return a pointer to the double at the specified index.
  */
-- (jdouble *)doubleRefAtIndex:(NSUInteger)index;
+- (double *)doubleRefAtIndex:(NSUInteger)index;
 
 /**
  * Replace the double at the specified index. Return the new value.
  */
-- (jdouble)replaceDoubleAtIndex:(NSUInteger)index withDouble:(jdouble)value;
+- (double)replaceDoubleAtIndex:(NSUInteger)index withDouble:(double)value;
 
 /**
  * Copy this array's doubles to a buffer.
  */
-- (void)getDoubles:(jdouble *)buffer length:(NSUInteger)length;
+- (void)getDoubles:(double *)buffer length:(NSUInteger)length;
 
 @end
 
@@ -758,8 +757,8 @@ __attribute__((always_inline)) inline jfloat *IOSFloatArray_GetRef(
  * @brief Return the double at the specified index.
  * Equivalent to doubleAtIndex:.
  */
-__attribute__((always_inline)) inline jdouble IOSDoubleArray_Get(
-    __unsafe_unretained IOSDoubleArray *array, jint index) {
+__attribute__((always_inline)) inline double IOSDoubleArray_Get(
+    __unsafe_unretained IOSDoubleArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return array->buffer_[index];
 }
@@ -768,8 +767,8 @@ __attribute__((always_inline)) inline jdouble IOSDoubleArray_Get(
  * @brief Return a pointer to the double at the specified index.
  * Equivalent to doubleRefAtIndex:.
  */
-__attribute__((always_inline)) inline jdouble *IOSDoubleArray_GetRef(
-    __unsafe_unretained IOSDoubleArray *array, jint index) {
+__attribute__((always_inline)) inline double *IOSDoubleArray_GetRef(
+    __unsafe_unretained IOSDoubleArray *array, int32_t index) {
   IOSArray_checkIndex(array->size_, index);
   return &array->buffer_[index];
 }
