@@ -56,11 +56,9 @@ NSString *JreClassPackageName(const J2ObjcClassInfo *metadata);
 // Field and method lookup functions.
 const J2ObjcFieldInfo *JreFindFieldInfo(const J2ObjcClassInfo *metadata, const char *fieldName);
 // Find a field declared in the given class
-JavaLangReflectField *FindDeclaredField(
-    IOSClass *iosClass, NSString *name, jboolean publicOnly);
+JavaLangReflectField *FindDeclaredField(IOSClass *iosClass, NSString *name, bool publicOnly);
 // Find a field declared in the given class or its hierarchy
-JavaLangReflectField *FindField(
-    IOSClass *iosClass, NSString *name, jboolean publicOnly);
+JavaLangReflectField *FindField(IOSClass *iosClass, NSString *name, bool publicOnly);
 // Find a method or constructor declared in the given class.
 JavaLangReflectMethod *JreMethodWithNameAndParamTypes(
     IOSClass *iosClass, NSString *name, IOSObjectArray *paramTypes);
