@@ -130,8 +130,8 @@ __attribute__((always_inline)) inline id JreRetainedAutoreleasedReturnValue(id v
 CF_EXTERN_C_BEGIN
 
 id JreThrowNullPointerException(void) __attribute__((noreturn));
-void JreThrowClassCastException(id p, Class cls) __attribute__((noreturn));
-void JreThrowClassCastExceptionWithIOSClass(id p, IOSClass *cls) __attribute__((noreturn));
+void JreThrowClassCastException(id obj, Class cls) __attribute__((noreturn));
+void JreThrowClassCastExceptionWithIOSClass(id obj, IOSClass *cls) __attribute__((noreturn));
 void JreThrowArithmeticExceptionWithNSString(NSString *msg) __attribute__((noreturn));
 
 id JreStrongAssign(__strong id *pIvar, id value);
@@ -144,7 +144,7 @@ void JreStrictFieldStrongRelease(__strong id *pIvar);
 id JreLoadVolatileId(volatile_id *pVar);
 id JreAssignVolatileId(volatile_id *pVar, id value);
 id JreVolatileStrongAssign(volatile_id *pIvar, id value);
-jboolean JreCompareAndSwapVolatileStrongId(volatile_id *pVar, id expected, id newValue);
+bool JreCompareAndSwapVolatileStrongId(volatile_id *pVar, id expected, id newValue);
 id JreExchangeVolatileStrongId(volatile_id *pVar, id newValue);
 void JreCloneVolatile(volatile_id *pVar, volatile_id *pOther);
 void JreCloneVolatileStrong(volatile_id *pVar, volatile_id *pOther);
@@ -153,14 +153,14 @@ id JreRetainedLocalValue(id value);
 
 id JreRetainedWithAssign(id parent, __strong id *pIvar, id value);
 id JreVolatileRetainedWithAssign(id parent, volatile_id *pIvar, id value);
-void JreRetainedWithRelease(id parent, id child);
+void JreRetainedWithRelease(id parent, id value);
 void JreVolatileRetainedWithRelease(id parent, volatile_id *pVar);
 void JreStrictFieldRetainedWithRelease(id parent, id *pVar);
 
 NSString *JreStrcat(const char *types, ...);
 
-jboolean JreAnnotationEquals(id a1, id a2);
-jint JreAnnotationHashCode(id a);
+bool JreAnnotationEquals(id a1, id a2);
+int32_t JreAnnotationHashCode(id a);
 
 NSUInteger JreDefaultFastEnumeration(
     id<JavaLangIterable> obj, NSFastEnumerationState *state, id __unsafe_unretained *stackbuf);
@@ -199,14 +199,14 @@ __attribute__((always_inline)) inline id JreAutoreleasedAssign(
     return value; \
   }
 
-J2OBJC_VOLATILE_ACCESS_DEFN(Boolean, jboolean)
-J2OBJC_VOLATILE_ACCESS_DEFN(Char, jchar)
-J2OBJC_VOLATILE_ACCESS_DEFN(Byte, jbyte)
-J2OBJC_VOLATILE_ACCESS_DEFN(Short, jshort)
-J2OBJC_VOLATILE_ACCESS_DEFN(Int, jint)
-J2OBJC_VOLATILE_ACCESS_DEFN(Long, jlong)
-J2OBJC_VOLATILE_ACCESS_DEFN(Float, jfloat)
-J2OBJC_VOLATILE_ACCESS_DEFN(Double, jdouble)
+J2OBJC_VOLATILE_ACCESS_DEFN(Boolean, bool)
+J2OBJC_VOLATILE_ACCESS_DEFN(Char, uint16_t)
+J2OBJC_VOLATILE_ACCESS_DEFN(Byte, int8_t)
+J2OBJC_VOLATILE_ACCESS_DEFN(Short, int16_t)
+J2OBJC_VOLATILE_ACCESS_DEFN(Int, int32_t)
+J2OBJC_VOLATILE_ACCESS_DEFN(Long, int64_t)
+J2OBJC_VOLATILE_ACCESS_DEFN(Float, float)
+J2OBJC_VOLATILE_ACCESS_DEFN(Double, double)
 #undef J2OBJC_VOLATILE_ACCESS_DEFN
 
 /*!
@@ -215,8 +215,7 @@ J2OBJC_VOLATILE_ACCESS_DEFN(Double, jdouble)
  * @define J2OBJC_INITIALIZED_DEFN
  * @param CLASS The class for which the initialized flag is defined.
  */
-#define J2OBJC_INITIALIZED_DEFN(CLASS) \
-  _Atomic(jboolean) CLASS##__initialized = false;
+#define J2OBJC_INITIALIZED_DEFN(CLASS) _Atomic(bool) CLASS##__initialized = false;
 
 /*!
  * Defines the code to set a class's initialized flag. This should be used at
@@ -237,7 +236,7 @@ J2OBJC_VOLATILE_ACCESS_DEFN(Double, jdouble)
  * @param CLASS The class to declare the init function for.
  */
 #define J2OBJC_STATIC_INIT(CLASS)                                                           \
-  FOUNDATION_EXPORT _Atomic(jboolean) CLASS##__initialized;                                 \
+  FOUNDATION_EXPORT _Atomic(bool) CLASS##__initialized;                                     \
   __attribute__((always_inline)) inline void CLASS##_initialize(void) {                     \
     if (__builtin_expect(!__c11_atomic_load(&CLASS##__initialized, __ATOMIC_ACQUIRE), 0)) { \
       [CLASS class];                                                                        \
