@@ -134,16 +134,19 @@ id JreVolatileStrongAssign(volatile_id *pIvar, id value) {
   return value;
 }
 
-jboolean JreCompareAndSwapVolatileStrongId(volatile_id *pVar, id expected, id newValue) {
+bool JreCompareAndSwapVolatileStrongId(volatile_id *pVar, id expected, id newValue) {
   volatile_lock_t lock = VOLATILE_GETLOCK(pVar);
   VOLATILE_LOCK(lock);
-  jboolean result = *(id *)pVar == expected;
+  [newValue retain];
+  bool result = *(id *)pVar == expected;
   if (result) {
-    *(id *)pVar = [newValue retain];
+    *(id *)pVar = newValue;
+  } else {
+    [newValue release];
   }
   VOLATILE_UNLOCK(lock);
   if (result) {
-    [expected autorelease];
+    [expected release];
   }
   return result;
 }
@@ -191,7 +194,7 @@ id JreStrictFieldStrongAssign(__strong id *pIvar, id value) {
   id oldValue = *(id *)pIvar;
   *(id *)pIvar = value;
   VOLATILE_UNLOCK(lock);
-  [oldValue autorelease];
+  [oldValue release];
   return value;
 }
 
@@ -267,7 +270,8 @@ void JreStrictFieldRetainedWithRelease(id parent, id *pVar) {
 
 jint JreIndexOfStr(NSString *str, NSString **values, jint size) {
   for (int i = 0; i < size; i++) {
-    if ([str isEqualToString:values[i]]) {
+    if ([str isEqualToString:values[i]]
+        || (!str && !values[i])) {  // Check for null case, new in Java 21.
       return i;
     }
   }
@@ -317,7 +321,7 @@ static jint ComputeCapacity(const char *types, va_list va, NSString **objDescrip
         va_arg(va, jlong);
         break;
       case 'Z':
-        capacity += (jboolean)va_arg(va, jint) ? 4 : 5;
+        capacity += (bool)va_arg(va, jint) ? 4 : 5;
         break;
       case '$':
         {
@@ -365,7 +369,7 @@ static void AppendArgs(
         JreStringBuilder_appendLong(sb, va_arg(va, jlong));
         break;
       case 'Z':
-        JreStringBuilder_appendString(sb, (jboolean)va_arg(va, jint) ? @"true" : @"false");
+        JreStringBuilder_appendString(sb, (bool)va_arg(va, jint) ? @"true" : @"false");
         break;
       case '$':
         JreStringBuilder_appendString(sb, va_arg(va, NSString *));
@@ -497,7 +501,7 @@ NSUInteger JreDefaultFastEnumeration(
     state->extra[1] = (unsigned long) [iter methodForSelector:hasNextSel];
     state->extra[2] = (unsigned long) [iter methodForSelector:nextSel];
   }
-  jboolean (*hasNextImpl)(id, SEL) = (jboolean (*)(id, SEL)) state->extra[1];
+  bool (*hasNextImpl)(id, SEL) = (bool (*)(id, SEL))state->extra[1];
   id (*nextImpl)(id, SEL) = (id (*)(id, SEL)) state->extra[2];
   NSUInteger objCount = 0;
   state->itemsPtr = stackbuf;
