@@ -39,6 +39,8 @@
 @class JavaUtilLocale;
 @protocol JavaLangIterable;
 @protocol JavaUtilComparator;
+@protocol JavaUtilFunctionFunction;
+@protocol JavaUtilStreamStream;
 
 // A category that adds java.lang.String-like methods to NSString.  The method
 // list is not exhaustive, since methods that can be directly substituted are
@@ -49,48 +51,48 @@
 + (nonnull NSString *)java_valueOf:(id<NSObject>)obj;
 
 // String.valueOf(boolean)
-+ (nonnull NSString *)java_valueOfBool:(jboolean)value;
++ (nonnull NSString *)java_valueOfBool:(bool)value;
 
 // String.valueOf(char)
-+ (nonnull NSString *)java_valueOfChar:(jchar)value;
++ (nonnull NSString *)java_valueOfChar:(uint16_t)value;
 
 // String.valueOf(char[])
 + (nonnull NSString *)java_valueOfChars:(IOSCharArray *)data;
 
 // String.valueOf(char[], offset, count)
 + (nonnull NSString *)java_valueOfChars:(IOSCharArray *)data
-                                 offset:(jint)offset
-                                  count:(jint)count;
+                                 offset:(int32_t)offset
+                                  count:(int32_t)count;
 
 // String.valueOf(double)
-+ (nonnull NSString *)java_valueOfDouble:(jdouble)value;
++ (nonnull NSString *)java_valueOfDouble:(double)value;
 
 // String.valueOf(float)
-+ (nonnull NSString *)java_valueOfFloat:(jfloat)value;
++ (nonnull NSString *)java_valueOfFloat:(float)value;
 
 // String.valueOf(int)
-+ (nonnull NSString *)java_valueOfInt:(jint)value;
++ (nonnull NSString *)java_valueOfInt:(int32_t)value;
 
 // String.valueOf(long)
-+ (nonnull NSString *)java_valueOfLong:(jlong)value;
++ (nonnull NSString *)java_valueOfLong:(int64_t)value;
 
 // String.getChars(int, int, char[], int)
-- (void)java_getChars:(jint)sourceBegin
-            sourceEnd:(jint)sourceEnd
+- (void)java_getChars:(int32_t)sourceBegin
+            sourceEnd:(int32_t)sourceEnd
           destination:(IOSCharArray *)dest
-     destinationBegin:(jint)dstBegin;
+     destinationBegin:(int32_t)dstBegin;
 
 // String(byte[])
 + (NSString *)java_stringWithBytes:(IOSByteArray *)value;
 
 // String(byte[], int)
 + (NSString *)java_stringWithBytes:(IOSByteArray *)value
-                            hibyte:(jint)hibyte;
+                            hibyte:(int32_t)hibyte;
 
 // String(byte[], int, int)
 + (NSString *)java_stringWithBytes:(IOSByteArray *)value
-                            offset:(jint)offset
-                            length:(jint)count;
+                            offset:(int32_t)offset
+                            length:(int32_t)count;
 
 + (NSString *)java_stringWithBytes:(IOSByteArray *)value
                             hibyte:(NSUInteger)hibyte
@@ -107,14 +109,14 @@
 
 // String(byte[], int, int, String)
 + (NSString *)java_stringWithBytes:(IOSByteArray *)value
-                            offset:(jint)offset
-                            length:(jint)count
+                            offset:(int32_t)offset
+                            length:(int32_t)count
                        charsetName:(NSString *)charsetName;
 
 // String(byte[], int, int, Charset)
 + (NSString *)java_stringWithBytes:(IOSByteArray *)value
-                            offset:(jint)offset
-                            length:(jint)count
+                            offset:(int32_t)offset
+                            length:(int32_t)count
                           charset:(JavaNioCharsetCharset *)charset;
 
 // String(char[])
@@ -122,13 +124,13 @@
 
 // String(char[], int, int)
 + (NSString *)java_stringWithCharacters:(IOSCharArray *)value
-                                 offset:(jint)offset
-                                 length:(jint)count;
+                                 offset:(int32_t)offset
+                                 length:(int32_t)count;
 
 // String(int[], int, int)
 + (NSString *)java_stringWithInts:(IOSIntArray *)codePoints
-                           offset:(jint)offset
-                           length:(jint)count;
+                           offset:(int32_t)offset
+                           length:(int32_t)count;
 
 // String(StringBuffer)
 + (NSString *)java_stringWithJavaLangStringBuffer:(JavaLangStringBuffer *)sb;
@@ -137,60 +139,63 @@
 + (NSString *)java_stringWithJavaLangStringBuilder:(JavaLangStringBuilder *)sb;
 
 // String.substring(int)
-- (nonnull NSString *)java_substring:(jint)beginIndex;
+- (nonnull NSString *)java_substring:(int32_t)beginIndex;
 
 // String.substring(int, int)
-- (nonnull NSString *)java_substring:(jint)beginIndex
-                            endIndex:(jint)endIndex;
+- (nonnull NSString *)java_substring:(int32_t)beginIndex
+                            endIndex:(int32_t)endIndex;
 
 // String.indexOf(int)
-- (jint)java_indexOf:(jint)ch;
+- (int32_t)java_indexOf:(int32_t)ch;
 
 // String.indexOf(int, int)
-- (jint)java_indexOf:(jint)ch fromIndex:(jint)index;
+- (int32_t)java_indexOf:(int32_t)ch fromIndex:(int32_t)index;
 
 // String.indexOf(String)
-- (jint)java_indexOfString:(NSString *)s;
+- (int32_t)java_indexOfString:(NSString *)s;
 
 // String.indexOf(String, int)
-- (jint)java_indexOfString:(NSString *)s fromIndex:(jint)index;
+- (int32_t)java_indexOfString:(NSString *)s fromIndex:(int32_t)index;
 
 // String.isEmpty()
-- (jboolean)java_isEmpty;
+- (bool)java_isEmpty;
 
 // String.lastIndexOf(int)
-- (jint)java_lastIndexOf:(jint)ch;
+- (int32_t)java_lastIndexOf:(int32_t)ch;
 
 // String.lastIndexOf(int, int)
-- (jint)java_lastIndexOf:(jint)ch fromIndex:(jint)index;
+- (int32_t)java_lastIndexOf:(int32_t)ch fromIndex:(int32_t)index;
 
 // String.lastIndexOf(String)
-- (jint)java_lastIndexOfString:(NSString *)s;
+- (int32_t)java_lastIndexOfString:(NSString *)s;
 
 // String.lastIndexOf(String, int)
-- (jint)java_lastIndexOfString:(NSString *)s fromIndex:(jint)index;
+- (int32_t)java_lastIndexOfString:(NSString *)s fromIndex:(int32_t)index;
 
 // String.length()
-- (jint)java_length;
+- (int32_t)java_length;
 
 // String.toCharArray()
 - (nonnull IOSCharArray *)java_toCharArray;
 
 // java.lang.Comparable implementation methods
-- (jint)compareToWithId:(id)another;
+- (int32_t)compareToWithId:(id)another;
 
 // CharSequence.charAt(int)
-- (jchar)charAtWithInt:(jint)index;
+- (uint16_t)charAtWithInt:(int32_t)index;
+
+// CharSequence.isEmpty()
+- (bool)isEmpty;
 
 // CharSequence.subSequence(int, int)
-- (nonnull id<JavaLangCharSequence>)subSequenceFrom:(jint)start
-                                                 to:(jint)end;
+- (nonnull id<JavaLangCharSequence>)subSequenceFrom:(int32_t)start
+                                                 to:(int32_t)end;
 
 // String.compareToIgnoreCase(String)
-- (jint)java_compareToIgnoreCase:(NSString *)another;
+- (int32_t)java_compareToIgnoreCase:(NSString *)another;
 
 // String.replace(char, char)
-- (nonnull NSString *)java_replace:(jchar)oldchar withChar:(jchar)newchar;
+- (nonnull NSString *)java_replace:(uint16_t)oldchar withChar:(uint16_t)newchar;
 
 // String.replace(CharSequence, CharSequence)
 - (nonnull NSString *)java_replace:(id<JavaLangCharSequence>)oldSequence
@@ -214,10 +219,10 @@
 - (nonnull IOSByteArray *)java_getBytesWithCharset:(JavaNioCharsetCharset *)charset;
 
 // String.getBytes(int, int, byte[], int)
-- (void)java_getBytesWithSrcBegin:(jint)srcBegin
-                       withSrcEnd:(jint)srcEnd
+- (void)java_getBytesWithSrcBegin:(int32_t)srcBegin
+                       withSrcEnd:(int32_t)srcEnd
                           withDst:(IOSByteArray *)dst
-                     withDstBegin:(jint)dstBegin;
+                     withDstBegin:(int32_t)dstBegin;
 
 // String.format(String, ...), String.format(Locale, String, ...)
 + (nonnull NSString *)java_formatWithNSString:(NSString *)format
@@ -227,9 +232,9 @@
                                   withNSObjectArray:(IOSObjectArray *)args;
 
 // String.startsWith(String), String.startsWith(String, int), String.endsWith(String)
-- (jboolean)java_hasPrefix:(NSString *)prefix;
-- (jboolean)java_hasPrefix:(NSString *)prefix offset:(jint)offset;
-- (jboolean)java_hasSuffix:(NSString *)suffix;
+- (bool)java_hasPrefix:(NSString *)prefix;
+- (bool)java_hasPrefix:(NSString *)prefix offset:(int32_t)offset;
+- (bool)java_hasSuffix:(NSString *)suffix;
 
 // String.trim()
 - (nonnull NSString *)java_trim;
@@ -238,23 +243,23 @@
 - (nonnull IOSObjectArray *)java_split:(NSString *)str;
 
 // String equalsIgnoreCase(String)
-- (jboolean)java_equalsIgnoreCase:(NSString *)aString;
+- (bool)java_equalsIgnoreCase:(NSString *)aString;
 
 // String.toLowerCase(Locale), toUpperCase(Locale)
 - (nonnull NSString *)java_lowercaseStringWithJRELocale:(JavaUtilLocale *)locale;
 - (nonnull NSString *)java_uppercaseStringWithJRELocale:(JavaUtilLocale *)locale;
 
 // String.regionMatches(...)
-- (jboolean)java_regionMatches:(jint)thisOffset
-                       aString:(NSString *)aString
-                   otherOffset:(jint)otherOffset
-                         count:(jint)count;
+- (bool)java_regionMatches:(int32_t)thisOffset
+                   aString:(NSString *)aString
+               otherOffset:(int32_t)otherOffset
+                     count:(int32_t)count;
 
-- (jboolean)java_regionMatches:(jboolean)caseInsensitive
-                    thisOffset:(jint)thisOffset
-                       aString:(NSString *)aString
-                   otherOffset:(jint)otherOffset
-                         count:(jint)count;
+- (bool)java_regionMatches:(bool)caseInsensitive
+                thisOffset:(int32_t)thisOffset
+                   aString:(NSString *)aString
+               otherOffset:(int32_t)otherOffset
+                     count:(int32_t)count;
 
 // String.intern()
 - (nonnull NSString *)java_intern;
@@ -263,23 +268,23 @@
 - (nonnull NSString *)java_concat:string;
 
 // String.contains(CharSequence)
-- (jboolean)java_contains:(id<JavaLangCharSequence>)sequence;
+- (bool)java_contains:(id<JavaLangCharSequence>)sequence;
 
 // String.codePointAt(int), codePointBefore(int), codePointCount(int, int)
-- (jint)java_codePointAt:(jint)index;
-- (jint)java_codePointBefore:(jint)index;
-- (jint)java_codePointCount:(jint)beginIndex endIndex:(jint)endIndex;
+- (int32_t)java_codePointAt:(int32_t)index;
+- (int32_t)java_codePointBefore:(int32_t)index;
+- (int32_t)java_codePointCount:(int32_t)beginIndex endIndex:(int32_t)endIndex;
 
 // String.matches(), split(String, int)
-- (jboolean)java_matches:(NSString *)regex;
-- (nonnull IOSObjectArray *)java_split:(NSString *)regex limit:(jint)limit;
+- (bool)java_matches:(NSString *)regex;
+- (nonnull IOSObjectArray *)java_split:(NSString *)regex limit:(int32_t)limit;
 
 // String.contentEquals(CharSequence), contentEquals(StringBuffer)
-- (jboolean)java_contentEqualsCharSequence:(id<JavaLangCharSequence>)seq;
-- (jboolean)java_contentEqualsStringBuffer:(JavaLangStringBuffer *)sb;
+- (bool)java_contentEqualsCharSequence:(id<JavaLangCharSequence>)seq;
+- (bool)java_contentEqualsStringBuffer:(JavaLangStringBuffer *)sb;
 
 // String.offsetByCodePoints(int, int)
-- (jint)java_offsetByCodePoints:(jint)index codePointOffset:(jint)offset;
+- (int32_t)java_offsetByCodePoints:(int32_t)index codePointOffset:(int32_t)offset;
 
 // String.join(CharSequence, CharSequence...)
 + (nonnull NSString *)java_joinWithJavaLangCharSequence:(id<JavaLangCharSequence>)delimiter
@@ -290,7 +295,7 @@
                                    withJavaLangIterable:(id<JavaLangIterable>)elements;
 
 // String.repeat(int)
-- (nonnull NSString *)java_repeat:(jint)count;
+- (nonnull NSString *)java_repeat:(int32_t)count;
 
 // String.strip()
 - (nonnull NSString *)java_strip;
@@ -302,7 +307,24 @@
 - (nonnull NSString *)java_stripTrailing;
 
 // String.isBlank()
-- (jboolean)java_isBlank;
+- (bool)java_isBlank;
+
+// String.lines()
+- (id<JavaUtilStreamStream>)java_lines;
+
+- (nonnull NSString *)java_indent:(int32_t)n;
+
+// String.stripIndent()
+- (nonnull NSString *)java_stripIndent;
+
+// String.translateEscapes()
+- (nonnull NSString *)java_translateEscapes;
+
+// String.formatted(Object...)
+- (NSString *)java_formattedWithNSObjectArray:(IOSObjectArray *)args;
+
+// String.transform(Function)
+- (nonnull id)transformWithJavaUtilFunctionFunction:(id<JavaUtilFunctionFunction>)f;
 
 @end
 
@@ -313,24 +335,24 @@ FOUNDATION_EXPORT NSString *NSString_java_formatWithJavaUtilLocale_withNSString_
 FOUNDATION_EXPORT NSString *NSString_java_formatWithNSString_withNSObjectArray_(
     NSString *s, IOSObjectArray *objs);
 // String.valueOf(boolean)
-FOUNDATION_EXPORT NSString *NSString_java_valueOfBool_(jboolean b);
+FOUNDATION_EXPORT NSString *NSString_java_valueOfBool_(bool b);
 // String.valueOf(char)
-FOUNDATION_EXPORT NSString *NSString_java_valueOfChar_(jchar c);
+FOUNDATION_EXPORT NSString *NSString_java_valueOfChar_(uint16_t c);
 // String.valueOf(char[])
 // String.copyValueOf(char[])
 FOUNDATION_EXPORT NSString *NSString_java_valueOfChars_(IOSCharArray *chars);
 // String.valueOf(char[], int, int)
 // String.copyValueOf(char[], int, int)
 FOUNDATION_EXPORT NSString *NSString_java_valueOfChars_offset_count_(
-    IOSCharArray *chars, jint i, jint j);
+    IOSCharArray *chars, int32_t i, int32_t j);
 // String.valueOf(double)
-FOUNDATION_EXPORT NSString *NSString_java_valueOfDouble_(jdouble d);
+FOUNDATION_EXPORT NSString *NSString_java_valueOfDouble_(double d);
 // String.valueOf(float)
-FOUNDATION_EXPORT NSString *NSString_java_valueOfFloat_(jfloat f);
+FOUNDATION_EXPORT NSString *NSString_java_valueOfFloat_(float f);
 // String.valueOf(int)
-FOUNDATION_EXPORT NSString *NSString_java_valueOfInt_(jint i);
+FOUNDATION_EXPORT NSString *NSString_java_valueOfInt_(int32_t i);
 // String.valueOf(long)
-FOUNDATION_EXPORT NSString *NSString_java_valueOfLong_(jlong l);
+FOUNDATION_EXPORT NSString *NSString_java_valueOfLong_(int64_t l);
 // String.valueOf(Object)
 FOUNDATION_EXPORT NSString *NSString_java_valueOf_(id o);
 // String.join(CharSequence, CharSequence...)
@@ -346,7 +368,7 @@ FOUNDATION_EXPORT NSString *NSString_java_joinWithJavaLangCharSequence_withJavaL
 @end
 
 // Use the category dummy to initialize static variables for the String class.
-FOUNDATION_EXPORT _Atomic(jboolean) NSString__initialized;
+FOUNDATION_EXPORT _Atomic(bool) NSString__initialized;
 __attribute__((always_inline)) inline void NSString_initialize(void) {
   if (__builtin_expect(!__c11_atomic_load(&NSString__initialized, __ATOMIC_ACQUIRE), 0)) {
     [JreStringCategoryDummy class];
