@@ -187,7 +187,10 @@ struct JNINativeInterface {
   jint          (*Throw)(JNIEnv*, jthrowable);
   jint          (*ThrowNew)(JNIEnv *, jclass, const char *);
   jint          (*EnsureLocalCapacity)(JNIEnv *, jint);
-  void          (*ExceptionClear)(JNIEnv *);
+  jthrowable    (*ExceptionOccurred)(JNIEnv*);
+  void          (*ExceptionDescribe)(JNIEnv*);
+  void          (*ExceptionClear)(JNIEnv*);
+  void          (*FatalError)(JNIEnv*, const char*);
 
   jobject       (*NewGlobalRef)(JNIEnv*, jobject);
   jobject       (*NewLocalRef)(JNIEnv*, jobject);
@@ -263,6 +266,8 @@ struct JNINativeInterface {
   const jchar*  (*GetStringCritical)(JNIEnv*, jstring, jboolean*);
   void          (*ReleaseStringCritical)(JNIEnv*, jstring, const jchar*);
 
+  jboolean      (*ExceptionCheck)(JNIEnv*);
+
   jobject       (*NewDirectByteBuffer)(JNIEnv*, void*, jlong);
   void*         (*GetDirectBufferAddress)(JNIEnv*, jobject);
   jlong         (*GetDirectBufferCapacity)(JNIEnv*, jobject);
@@ -529,9 +534,18 @@ struct _JNIEnv {
     jint EnsureLocalCapacity(jint capacity)
     { return functions->EnsureLocalCapacity(this, capacity); }
 
+    jthrowable ExceptionOccurred()
+    { return functions->ExceptionOccurred(this); }
+
+    void ExceptionDescribe()
+    { functions->ExceptionDescribe(this); }
+
     void ExceptionClear()
     { functions->ExceptionClear(this); }
 
+    void FatalError(const char* msg)
+    { functions->FatalError(this, msg); }
+
     jobject NewGlobalRef(jobject obj)
     { return functions->NewGlobalRef(this, obj); }
 
@@ -718,6 +732,9 @@ struct _JNIEnv {
     void ReleaseStringCritical(jstring string, const jchar* carray)
     { functions->ReleaseStringCritical(this, string, carray); }
 
+    jboolean ExceptionCheck()
+    { return functions->ExceptionCheck(this); }
+
     jobject NewDirectByteBuffer(void* address, jlong capacity)
     { return functions->NewDirectByteBuffer(this, address, capacity); }
 
@@ -1311,4 +1328,4 @@ extern "C" {
 #define JNI_COMMIT      1           /* copy content, do not free buffer */
 #define JNI_ABORT       2           /* free buffer w/o copying back */
 
-#endif  /* JNI_H_ */
+#endif  /* JNI_H_ */
\ No newline at end of file
