@@ -21,6 +21,7 @@
 //
 
 #include "jni.h"
+#import <Foundation/Foundation.h>
 
 #include "IOSArray.h"
 #include "IOSClass.h"
@@ -28,6 +29,7 @@
 #include "IOSPrimitiveArray.h"
 #include "IOSReflection.h"
 #include "java/lang/ClassNotFoundException.h"
+#include "java/lang/Error.h"
 #include "java/lang/InstantiationException.h"
 #include "java/lang/Throwable.h"
 #include "java/lang/reflect/Constructor.h"
@@ -39,6 +41,8 @@
 
 #define null_chk(p) (void)nil_chk(p)
 
+static NSString *JNI_EXCEPTION_KEY = @"JNI_EXCEPTION_KEY";
+
 static IOSClass *IOSClass_forName(const char *name) {
   NSString *nameString = [NSString stringWithUTF8String:name];
   nameString = [nameString stringByReplacingOccurrencesOfString:@"/" withString:@"."];
@@ -125,6 +129,16 @@ NSString *JNIFormatMethodSignature(JNIMethodSignature sig) {
   return result;
 }
 
+static jthrowable JNIGetCurrentThreadException(JNIEnv *env) {
+  NSMutableDictionary *threadDictionary = [[NSThread currentThread] threadDictionary];
+  return threadDictionary[JNI_EXCEPTION_KEY];
+}
+
+static void JNISetCurrentThreadException(JNIEnv *env, jthrowable exception) {
+  NSMutableDictionary *threadDictionary = [[NSThread currentThread] threadDictionary];
+  threadDictionary[JNI_EXCEPTION_KEY] = exception;
+}
+
 static jclass FindClass(JNIEnv *env, const char *name) {
   return IOSClass_forName(name);
 }
@@ -306,6 +320,10 @@ static jobject NewDirectByteBuffer(JNIEnv *env, void *address, jlong capacity) {
                                                         withInt:(jint)capacity]);
 }
 
+static jboolean ExceptionCheck(JNIEnv *env) {
+  return JNIGetCurrentThreadException(env) ? JNI_TRUE : JNI_FALSE;
+}
+
 static jdoubleArray NewDoubleArray(JNIEnv *env, jsize length) {
   return [IOSDoubleArray arrayWithLength:length];
 }
@@ -443,6 +461,7 @@ SET_ARRAY_REGION_IMPL(Double, jdouble)
 
 static jint Throw(JNIEnv *env, jthrowable obj) {
   (void)nil_chk(obj);
+  JNISetCurrentThreadException(env, obj);
   @throw obj;
   return 0;
 }
@@ -451,7 +470,7 @@ static jint ThrowNew(JNIEnv *env, jclass clazz, const char *message) {
   (void)nil_chk(clazz);
   NSString *msg = [NSString stringWithUTF8String:message];
   id exc = [(JavaLangThrowable *) [((IOSClass *) clazz).objcClass alloc] initWithNSString:msg];
-  @throw AUTORELEASE(exc);
+  Throw(env, AUTORELEASE(exc));
   return 0;
 }
 
@@ -460,8 +479,24 @@ static jint EnsureLocalCapacity(JNIEnv *env, jint capacity) {
   return 0;
 }
 
+static jthrowable ExceptionOccurred(JNIEnv *env) {
+  return JNIGetCurrentThreadException(env);
+}
+
+static void ExceptionDescribe(JNIEnv *env) {
+  jthrowable exception = JNIGetCurrentThreadException(env);
+  if (exception) {
+    [exception printStackTrace];
+  }
+}
+
 static void ExceptionClear(JNIEnv *env) {
-  // no-op
+  NSMutableDictionary *threadDictionary = [[NSThread currentThread] threadDictionary];
+  [threadDictionary removeObjectForKey:JNI_EXCEPTION_KEY];
+}
+
+static void FatalError(JNIEnv *env, const char *msg) {
+  ThrowNew(env, JavaLangError_class_(), msg);
 }
 
 static jfieldID GetFieldID(JNIEnv *env, jclass clazz, const char *name, const char *sig) {
@@ -492,7 +527,7 @@ static jmethodID GetStaticMethodID(JNIEnv *env, jclass clazz, const char *name,
 }
 
 #define ALLOC_JARGS(JARGS, NUM_ARGS)           \
-  const size_t _max_stack_args = 16;           \
+  enum { _max_stack_args = 16 };               \
   jvalue _stack_args[_max_stack_args];         \
   jvalue *JARGS;                               \
   jboolean _free_jargs = false;                \
@@ -570,8 +605,13 @@ static jobject NewObject(JNIEnv *env, jclass clazz, jmethodID methodID, ...) {
 }
 
 static void CallMethodA(JNIEnv *env, jobject obj, jmethodID methodID, const jvalue *args, jvalue *result) {
-  [(JavaLangReflectMethod *)methodID
-      jniInvokeWithId:obj args:(const J2ObjcRawValue *)args result:(J2ObjcRawValue *)result];
+  @try {
+    [(JavaLangReflectMethod *)methodID
+        jniInvokeWithId:obj args:(const J2ObjcRawValue *)args result:(J2ObjcRawValue *)result];
+  } @catch (JavaLangThrowable *e) {
+    // Throw() saves the exception to the current thread's exception table.
+    Throw(env, e);
+  }
 }
 
 static void CallMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args, jvalue *result) {
@@ -584,6 +624,26 @@ static void CallMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list ar
   DEALLOC_JARGS(jargs);
 }
 
+jobject CallObjectMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {
+  @autoreleasepool {
+    jvalue result;
+    CallMethodV(env, obj, methodID, args, &result);
+    return [result.l retain];
+  }
+}
+
+jobject CallObjectMethodA(JNIEnv *env, jobject obj, jmethodID methodID, const jvalue *args) {
+  @autoreleasepool {
+    jvalue result;
+    CallMethodA(env, obj, methodID, args, &result);
+    return [result.l retain];
+  }
+}
+
+jobject CallObjectMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {
+  FORWARD_VARGS(jobject, CallObjectMethodV(env, obj, methodID, args));
+}
+
 #define DEFINE_CALL_METHOD_VARIANTS(RESULT_NAME, RESULT_TYPE, RESULT_CODE) \
   RESULT_TYPE Call##RESULT_NAME##MethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) { \
     jvalue result; \
@@ -599,7 +659,6 @@ static void CallMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list ar
     FORWARD_VARGS(RESULT_TYPE, Call##RESULT_NAME##MethodV(env, obj, methodID, args)); \
   }
 
-DEFINE_CALL_METHOD_VARIANTS(Object, jobject, l)
 DEFINE_CALL_METHOD_VARIANTS(Boolean, jboolean, z)
 DEFINE_CALL_METHOD_VARIANTS(Byte, jbyte, b)
 DEFINE_CALL_METHOD_VARIANTS(Char, jchar, c)
@@ -696,6 +755,26 @@ void SetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID, jdouble value) {
   [(JavaLangReflectField *)fieldID setDoubleWithId:obj withDouble:value];
 }
 
+jobject CallStaticObjectMethodV(JNIEnv *env, jclass clazz, jmethodID methodID, va_list args) {
+  @autoreleasepool {
+    jvalue result;
+    CallMethodV(env, nil, methodID, args, &result);
+    return [result.l retain];
+  }
+}
+
+jobject CallStaticObjectMethodA(JNIEnv *env, jclass clazz, jmethodID methodID, const jvalue *args) {
+  @autoreleasepool {
+    jvalue result;
+    CallMethodA(env, nil, methodID, args, &result);
+    return [result.l retain];
+  }
+}
+
+jobject CallStaticObjectMethod(JNIEnv *env, jclass clazz, jmethodID methodID, ...) {
+  FORWARD_VARGS(jobject, CallObjectMethodV(env, nil, methodID, args));
+}
+
 #define DEFINE_CALL_STATIC_METHOD_VARIANTS(RESULT_NAME, RESULT_TYPE, RESULT_CODE) \
   RESULT_TYPE CallStatic##RESULT_NAME##MethodV(JNIEnv *env, jclass clazz, jmethodID methodID, va_list args) { \
     jvalue result; \
@@ -711,7 +790,6 @@ void SetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID, jdouble value) {
     FORWARD_VARGS(RESULT_TYPE, Call##RESULT_NAME##MethodV(env, nil, methodID, args)); \
   }
 
-DEFINE_CALL_STATIC_METHOD_VARIANTS(Object, jobject, l)
 DEFINE_CALL_STATIC_METHOD_VARIANTS(Boolean, jboolean, z)
 DEFINE_CALL_STATIC_METHOD_VARIANTS(Byte, jbyte, b)
 DEFINE_CALL_STATIC_METHOD_VARIANTS(Char, jchar, c)
@@ -818,7 +896,10 @@ static struct JNINativeInterface JNI_JNIEnvTable = {
   &Throw,
   &ThrowNew,
   &EnsureLocalCapacity,
+  &ExceptionOccurred,
+  &ExceptionDescribe,
   &ExceptionClear,
+  &FatalError,
   &NewGlobalRef,
   &NewLocalRef,
   &DeleteGlobalRef,
@@ -884,6 +965,7 @@ static struct JNINativeInterface JNI_JNIEnvTable = {
   &ReleasePrimitiveArrayCritical,
   &GetStringCritical,
   &ReleaseStringCritical,
+  &ExceptionCheck,
   &NewDirectByteBuffer,
   &GetDirectBufferAddress,
   &GetDirectBufferCapacity,
