@@ -16,7 +16,11 @@
 
 package org.json;
 
-// import dalvik.annotation.compat.UnsupportedAppUsage;
+// import static android.annotation.SystemApi.Client.MODULE_LIBRARIES;
+
+// import android.annotation.SystemApi;
+// import android.compat.annotation.UnsupportedAppUsage;
+
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
@@ -78,8 +82,8 @@ import java.util.Set;
  * <p>Instances of this class are not thread safe. Although this class is
  * nonfinal, it was not designed for inheritance and should not be subclassed.
  * In particular, self-use by overrideable methods is not specified. See
- * <i>Effective Java</i> Item 17, "Design and Document or inheritance or else
- * prohibit it" for further information.
+ * <i>Effective Java, 3rd edition</i> Item 19, "Design and Document for
+ * inheritance or else prohibit it" for further information.
  */
 public class JSONObject {
 
@@ -113,13 +117,13 @@ public class JSONObject {
     };
 
     @UnsupportedAppUsage
-    private final LinkedHashMap<String, Object> nameValuePairs;
+    private final LinkedHashMap<@NonNull String, @Nullable Object> nameValuePairs;
 
     /**
      * Creates a {@code JSONObject} with no name/value mappings.
      */
     public JSONObject() {
-        nameValuePairs = new LinkedHashMap<String, Object>();
+        nameValuePairs = new LinkedHashMap<@NonNull String, @Nullable Object>();
     }
 
     /**
@@ -292,7 +296,7 @@ public class JSONObject {
      * mapped to {@code name}. In aggregate, this allows values to be added to a
      * mapping one at a time.
      *
-     * <p> Note that {@code append(String, Object)} provides better semantics.
+     * <p> Note that {@link #append(String, Object)} provides better semantics.
      * In particular, the mapping for {@code name} will <b>always</b> be a
      * {@link JSONArray}. Using {@code accumulate} will result in either a
      * {@link JSONArray} or a mapping whose type is the type of {@code value}
@@ -302,8 +306,6 @@ public class JSONObject {
      *     Integer, Long, Double, {@link #NULL} or null. May not be {@link
      *     Double#isNaN() NaNs} or {@link Double#isInfinite() infinities}.
      */
-    // TODO: Change {@code append) to {@link #append} when append is
-    // unhidden.
     @NonNull public JSONObject accumulate(@NonNull String name, @Nullable Object value) throws JSONException {
         Object current = nameValuePairs.get(checkName(name));
         if (current == null) {
@@ -330,11 +332,8 @@ public class JSONObject {
      *
      * @throws JSONException if {@code name} is {@code null} or if the mapping for
      *         {@code name} is non-null and is not a {@link JSONArray}.
-     *
-     * @hide
      */
-    @UnsupportedAppUsage
-    public JSONObject append(String name, Object value) throws JSONException {
+    @NonNull public JSONObject append(@NonNull String name, @Nullable Object value) throws JSONException {
         Object current = nameValuePairs.get(checkName(name));
 
         final JSONArray array;
@@ -673,11 +672,13 @@ public class JSONObject {
      *
      * See {@link #keys()}.
      *
-     * @hide.
+     * @return set of keys in this object
+     *
+     * @hide
      */
     @UnsupportedAppUsage
-//    @libcore.api.CorePlatformApi
-    public Set<String> keySet() {
+//    @SystemApi(client = MODULE_LIBRARIES)
+    @NonNull public Set<@NonNull String> keySet() {
         return nameValuePairs.keySet();
     }
 
@@ -688,7 +689,7 @@ public class JSONObject {
     @Nullable public JSONArray names() {
         return nameValuePairs.isEmpty()
                 ? null
-                : new JSONArray(new ArrayList<String>(nameValuePairs.keySet()));
+                : new JSONArray(new ArrayList<@Nullable String>(nameValuePairs.keySet()));
     }
 
     /**
@@ -701,7 +702,9 @@ public class JSONObject {
             writeTo(stringer);
             return stringer.toString();
         } catch (JSONException e) {
-            return null;
+            // j2objc: do not return null from @NonNull method.
+            // return null;
+            return e.toString();
         }
     }
 
