@@ -107,40 +107,37 @@ jlong Java_sun_misc_Unsafe_allocateMemory(JNIEnv *env, jobject self, jlong bytes
  * Method:    compareAndSwapInt
  * Signature: (Ljava/lang/Object;JII)Z
  */
-jboolean Java_sun_misc_Unsafe_compareAndSwapInt(
-    JNIEnv *env, jobject self, jobject obj, jlong offset, jint expectedValue, jint newValue) {
+bool Java_sun_misc_Unsafe_compareAndSwapInt(JNIEnv *env, jobject self, jobject obj, jlong offset,
+                                            jint expectedValue, jint newValue) {
   uintptr_t ptr = PTR(obj, offset);
   CHECK_ADDR(jint, ptr)
   return __c11_atomic_compare_exchange_strong(
       (volatile_jint *)ptr, &expectedValue, newValue, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
 }
 
-
 /*
  * Method:    compareAndSwapLong
  * Signature: (Ljava/lang/Object;JJJ)Z
  */
-jboolean Java_sun_misc_Unsafe_compareAndSwapLong(
-    JNIEnv *env, jobject self, jobject obj, jlong offset, jlong expectedValue, jlong newValue) {
+bool Java_sun_misc_Unsafe_compareAndSwapLong(JNIEnv *env, jobject self, jobject obj, jlong offset,
+                                             jlong expectedValue, jlong newValue) {
   uintptr_t ptr = PTR(obj, offset);
   CHECK_ADDR(jlong, ptr)
   return __c11_atomic_compare_exchange_strong(
       (volatile_jlong *)ptr, &expectedValue, newValue, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
 }
 
-
 /*
  * Method:    compareAndSwapObject
  * Signature: (Ljava/lang/Object;JLjava/lang/Object;Ljava/lang/Object;)Z
  */
-jboolean Java_sun_misc_Unsafe_compareAndSwapObject(
-    JNIEnv *env, jobject self, jobject obj, jlong offset, jobject expectedValue, jobject newValue) {
+bool Java_sun_misc_Unsafe_compareAndSwapObject(JNIEnv *env, jobject self, jobject obj, jlong offset,
+                                               jobject expectedValue, jobject newValue) {
   uintptr_t ptr = PTR(obj, offset);
   CHECK_ADDR(id, ptr)
   return JreCompareAndSwapVolatileStrongId((volatile_id *)ptr, expectedValue, newValue);
 }
 
-
 /*
  * Method:    freeMemory
  * Signature: (J)V
@@ -254,56 +251,48 @@ void Java_sun_misc_Unsafe_fullFence(JNIEnv *env, jobject self) {
  * Method:    getBoolean
  * Signature: (Ljava/lang/Object;J)Z
  */
-jboolean Java_sun_misc_Unsafe_getBoolean
-(JNIEnv *env, jobject self, jobject obj, jlong offset) {
-  GET_IMPL_OFFSET(jboolean, RELAXED)
+bool Java_sun_misc_Unsafe_getBoolean(JNIEnv *env, jobject self, jobject obj, jlong offset) {
+  GET_IMPL_OFFSET(bool, RELAXED)
 }
 
 /*
  * Method:    getBooleanVolatile
  * Signature: (Ljava/lang/Object;J)Z
  */
-jboolean Java_sun_misc_Unsafe_getBooleanVolatile(
-    JNIEnv *env, jobject self, jobject obj, jlong offset) {
-  GET_IMPL_OFFSET(jboolean, SEQ_CST)
+bool Java_sun_misc_Unsafe_getBooleanVolatile(JNIEnv *env, jobject self, jobject obj, jlong offset) {
+  GET_IMPL_OFFSET(bool, SEQ_CST)
 }
 
-
 /*
  * Method:    putBoolean
  * Signature: (Ljava/lang/Object;JZ)V
  */
-void Java_sun_misc_Unsafe_putBoolean(
-    JNIEnv *env, jobject self, jobject obj, jlong offset, jboolean newValue) {
-  PUT_IMPL_OFFSET(jboolean, RELAXED)
+void Java_sun_misc_Unsafe_putBoolean(JNIEnv *env, jobject self, jobject obj, jlong offset,
+                                     bool newValue) {
+  PUT_IMPL_OFFSET(bool, RELAXED)
 }
 
 /*
  * Method:    putBooleanVolatile
  * Signature: (Ljava/lang/Object;JZ)V
  */
-void Java_sun_misc_Unsafe_putBooleanVolatile(
-    JNIEnv *env, jobject self, jobject obj, jlong offset, jboolean newValue) {
-  PUT_IMPL_OFFSET(jboolean, SEQ_CST)
-}
-
+void Java_sun_misc_Unsafe_putBooleanVolatile(JNIEnv *env, jobject self, jobject obj, jlong offset,
+                                             bool newValue){PUT_IMPL_OFFSET(bool, SEQ_CST)}
 
 /*
  * Method:    getByte
  * Signature: (J)B
  */
-jbyte Java_sun_misc_Unsafe_getByte__J(JNIEnv *env, jobject self, jlong address) {
-  GET_IMPL(jbyte, RELAXED)
-}
+jbyte Java_sun_misc_Unsafe_getByte__J(JNIEnv *env, jobject self,
+                                      jlong address){GET_IMPL(jbyte, RELAXED)}
 
 /*
  * Method:    getByte
  * Signature: (Ljava/lang/Object;J)B
  */
-jbyte Java_sun_misc_Unsafe_getByte__Ljava_lang_Object_2J(
-    JNIEnv *env, jobject self, jobject obj, jlong offset) {
-  GET_IMPL_OFFSET(jbyte, RELAXED)
-}
+jbyte Java_sun_misc_Unsafe_getByte__Ljava_lang_Object_2J(JNIEnv *env, jobject self, jobject obj,
+                                                         jlong offset){
+    GET_IMPL_OFFSET(jbyte, RELAXED)}
 
 /*
  * Method:    getByteVolatile
