@@ -416,7 +416,8 @@ public class SimpleDateFormatTest extends junit.framework.TestCase {
     public void test_sl_dates() throws Exception {
         DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, new Locale("sl"));
         df.setTimeZone(TimeZone.getTimeZone("UTC"));
-        assertEquals("1. 01. 70", df.format(0L));
+        String result = df.format(0L);
+        assertTrue(result.equals("1. 1. 70") || result.equals("1. 01. 70"));
     }
 
     public void testLenientParsingForZ() throws Exception {
