@@ -287,12 +287,12 @@ public class ObjectStreamClass implements Serializable {
             return null;
         }
         requireInitialized();
-        if (System.getSecurityManager() != null) {
-            Class<?> caller = Reflection.getCallerClass();
-            if (ReflectUtil.needsPackageAccessCheck(caller.getClassLoader(), cl.getClassLoader())) {
-                ReflectUtil.checkPackageAccess(cl);
-            }
-        }
+        // if (System.getSecurityManager() != null) {
+        //     Class<?> caller = Reflection.getCallerClass();
+        //     if (ReflectUtil.needsPackageAccessCheck(caller.getClassLoader(), cl.getClassLoader())) {
+        //         ReflectUtil.checkPackageAccess(cl);
+        //     }
+        // }
         return cl;
     }
 
