@@ -26,17 +26,27 @@
 
 package java.io;
 
+/* J2ObjC removed
+import android.system.Os;
+import dalvik.annotation.optimization.ReachabilitySensitive;
+import libcore.io.IoTracker;
+import libcore.io.IoUtils;
+import static android.system.OsConstants.*;
+import sun.nio.ch.FileChannelImpl;
+*/
+
 import static libcore.io.OsConstants.O_CREAT;
 import static libcore.io.OsConstants.O_RDONLY;
 import static libcore.io.OsConstants.O_RDWR;
 import static libcore.io.OsConstants.SEEK_CUR;
 import static libcore.io.OsConstants.SEEK_SET;
+import java.nio.channels.FileChannel;
 import android.system.ErrnoException;
 import dalvik.system.CloseGuard;
-import java.nio.channels.FileChannel;
 import libcore.io.IoBridge;
 import libcore.io.Libcore;
 
+
 /**
  * Instances of this class support both reading and writing to a
  * random access file. A random access file behaves like a large
