@@ -26,6 +26,10 @@
 
 package java.lang;
 
+/* J2ObjC removed
+import jdk.internal.misc.VM;
+*/
+
 import java.io.PrintStream;
 import java.util.Arrays;
 
@@ -40,7 +44,7 @@ import java.util.Arrays;
  * parent thread group or any other thread groups.
  *
  * @author  unascribed
- * @since   JDK1.0
+ * @since   1.0
  */
 /* The locking strategy for this code is to try to lock only one level of the
  * tree wherever possible, but otherwise to lock from the bottom up.
@@ -58,14 +62,13 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /* the runtime uses these directly; do not rename */
     static final ThreadGroup systemThreadGroup = new ThreadGroup();
 
-    static final ThreadGroup mainThreadGroup = new MainThreadGroup(systemThreadGroup);
+    static final ThreadGroup mainThreadGroup = new ThreadGroup(systemThreadGroup, "main");
 
     private final ThreadGroup parent;
     String name;
     int maxPriority;
     boolean destroyed;
     boolean daemon;
-    boolean vmAllowSuspension;
 
     int nUnstartedThreads = 0;
     int nthreads;
@@ -88,14 +91,14 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Constructs a new thread group. The parent of this new group is
      * the thread group of the currently running thread.
      * <p>
-     * The <code>checkAccess</code> method of the parent thread group is
+     * The {@code checkAccess} method of the parent thread group is
      * called with no arguments; this may result in a security exception.
      *
      * @param   name   the name of the new thread group.
-     * @exception  SecurityException  if the current thread cannot create a
+     * @throws  SecurityException  if the current thread cannot create a
      *               thread in the specified thread group.
      * @see     java.lang.ThreadGroup#checkAccess()
-     * @since   JDK1.0
+     * @since   1.0
      */
     public ThreadGroup(String name) {
         this(Thread.currentThread().getThreadGroup(), name);
@@ -105,18 +108,18 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Creates a new thread group. The parent of this new group is the
      * specified thread group.
      * <p>
-     * The <code>checkAccess</code> method of the parent thread group is
+     * The {@code checkAccess} method of the parent thread group is
      * called with no arguments; this may result in a security exception.
      *
      * @param     parent   the parent thread group.
      * @param     name     the name of the new thread group.
-     * @exception  NullPointerException  if the thread group argument is
-     *               <code>null</code>.
-     * @exception  SecurityException  if the current thread cannot create a
+     * @throws    NullPointerException  if the thread group argument is
+     *               {@code null}.
+     * @throws    SecurityException  if the current thread cannot create a
      *               thread in the specified thread group.
      * @see     java.lang.SecurityException
      * @see     java.lang.ThreadGroup#checkAccess()
-     * @since   JDK1.0
+     * @since   1.0
      */
     public ThreadGroup(ThreadGroup parent, String name) {
         this(checkParentAccess(parent), parent, name);
@@ -126,7 +129,6 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
         this.name = name;
         this.maxPriority = parent.maxPriority;
         this.daemon = parent.daemon;
-        this.vmAllowSuspension = parent.vmAllowSuspension;
         this.parent = parent;
         parent.add(this);
     }
@@ -145,7 +147,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Returns the name of this thread group.
      *
      * @return  the name of this thread group.
-     * @since   JDK1.0
+     * @since   1.0
      */
     public final String getName() {
         return name;
@@ -154,18 +156,18 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Returns the parent of this thread group.
      * <p>
-     * First, if the parent is not <code>null</code>, the
-     * <code>checkAccess</code> method of the parent thread group is
+     * First, if the parent is not {@code null}, the
+     * {@code checkAccess} method of the parent thread group is
      * called with no arguments; this may result in a security exception.
      *
      * @return  the parent of this thread group. The top-level thread group
-     *          is the only thread group whose parent is <code>null</code>.
-     * @exception  SecurityException  if the current thread cannot modify
+     *          is the only thread group whose parent is {@code null}.
+     * @throws  SecurityException  if the current thread cannot modify
      *               this thread group.
      * @see        java.lang.ThreadGroup#checkAccess()
      * @see        java.lang.SecurityException
      * @see        java.lang.RuntimePermission
-     * @since   JDK1.0
+     * @since   1.0
      */
     public final ThreadGroup getParent() {
         if (parent != null)
@@ -181,7 +183,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * @return  the maximum priority that a thread in this thread group
      *          can have.
      * @see     #setMaxPriority
-     * @since   JDK1.0
+     * @since   1.0
      */
     public final int getMaxPriority() {
         return maxPriority;
@@ -192,9 +194,9 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * daemon thread group is automatically destroyed when its last
      * thread is stopped or its last thread group is destroyed.
      *
-     * @return  <code>true</code> if this thread group is a daemon thread group;
-     *          <code>false</code> otherwise.
-     * @since   JDK1.0
+     * @return  {@code true} if this thread group is a daemon thread group;
+     *          {@code false} otherwise.
+     * @since   1.0
      */
     public final boolean isDaemon() {
         return daemon;
@@ -204,7 +206,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Tests if this thread group has been destroyed.
      *
      * @return  true if this object is destroyed
-     * @since   JDK1.1
+     * @since   1.1
      */
     public synchronized boolean isDestroyed() {
         return destroyed;
@@ -213,20 +215,20 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Changes the daemon status of this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
      * A daemon thread group is automatically destroyed when its last
      * thread is stopped or its last thread group is destroyed.
      *
-     * @param      daemon   if <code>true</code>, marks this thread group as
+     * @param      daemon   if {@code true}, marks this thread group as
      *                      a daemon thread group; otherwise, marks this
      *                      thread group as normal.
-     * @exception  SecurityException  if the current thread cannot modify
+     * @throws     SecurityException  if the current thread cannot modify
      *               this thread group.
      * @see        java.lang.SecurityException
      * @see        java.lang.ThreadGroup#checkAccess()
-     * @since      JDK1.0
+     * @since      1.0
      */
     public final void setDaemon(boolean daemon) {
         checkAccess();
@@ -237,37 +239,36 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Sets the maximum priority of the group. Threads in the thread
      * group that already have a higher priority are not affected.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * If the <code>pri</code> argument is less than
+     * If the {@code pri} argument is less than
      * {@link Thread#MIN_PRIORITY} or greater than
      * {@link Thread#MAX_PRIORITY}, it is clamped to those values.
      * <p>
      * Otherwise, the priority of this ThreadGroup object is set to the
-     * smaller of the specified <code>pri</code> and the maximum permitted
+     * smaller of the specified {@code pri} and the maximum permitted
      * priority of the parent of this thread group. (If this thread group
      * is the system thread group, which has no parent, then its maximum
-     * priority is simply set to <code>pri</code>.) Then this method is
-     * called recursively, with <code>pri</code> as its argument, for
+     * priority is simply set to {@code pri}.) Then this method is
+     * called recursively, with {@code pri} as its argument, for
      * every thread group that belongs to this thread group.
      *
      * @param      pri   the new priority of the thread group.
-     * @exception  SecurityException  if the current thread cannot modify
+     * @throws     SecurityException  if the current thread cannot modify
      *               this thread group.
      * @see        #getMaxPriority
      * @see        java.lang.SecurityException
      * @see        java.lang.ThreadGroup#checkAccess()
-     * @since      JDK1.0
+     * @since      1.0
      */
-    // Android-changed: We clamp the priority to the range [MIN_PRIORITY, MAX_PRIORITY]
-    // before using it.
     public final void setMaxPriority(int pri) {
         int ngroupsSnapshot;
         ThreadGroup[] groupsSnapshot;
         synchronized (this) {
             checkAccess();
-            // Android-changed: Clamp to MIN_PRIORITY, MAX_PRIORITY.
+            // BEGIN Android-changed: Clamp to the range [MIN_PRIORITY, MAX_PRIORITY].
+            // This preserves backward compatibility with previous versions of Android.
             // if (pri < Thread.MIN_PRIORITY || pri > Thread.MAX_PRIORITY) {
             //     return;
             // }
@@ -277,6 +278,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
             if (pri > Thread.MAX_PRIORITY) {
                 pri = Thread.MAX_PRIORITY;
             }
+            // END Android-changed: Clamp to the range [MIN_PRIORITY, MAX_PRIORITY].
 
             maxPriority = (parent != null) ? Math.min(pri, parent.maxPriority) : pri;
             ngroupsSnapshot = ngroups;
@@ -296,10 +298,10 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * argument or one of its ancestor thread groups.
      *
      * @param   g   a thread group.
-     * @return  <code>true</code> if this thread group is the thread group
+     * @return  {@code true} if this thread group is the thread group
      *          argument or one of its ancestor thread groups;
-     *          <code>false</code> otherwise.
-     * @since   JDK1.0
+     *          {@code false} otherwise.
+     * @since   1.0
      */
     public final boolean parentOf(ThreadGroup g) {
         for (; g != null ; g = g.parent) {
@@ -314,16 +316,21 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Determines if the currently running thread has permission to
      * modify this thread group.
      * <p>
-     * If there is a security manager, its <code>checkAccess</code> method
+     * If there is a security manager, its {@code checkAccess} method
      * is called with this thread group as its argument. This may result
-     * in throwing a <code>SecurityException</code>.
+     * in throwing a {@code SecurityException}.
      *
-     * @exception  SecurityException  if the current thread is not allowed to
+     * @throws     SecurityException  if the current thread is not allowed to
      *               access this thread group.
      * @see        java.lang.SecurityManager#checkAccess(java.lang.ThreadGroup)
-     * @since      JDK1.0
+     * @since      1.0
      */
     public final void checkAccess() {
+        // Android-removed: SecurityManager stubbed out on Android.
+        // SecurityManager security = System.getSecurityManager();
+        // if (security != null) {
+        //     security.checkAccess(this);
+        // }
     }
 
     /**
@@ -341,7 +348,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      *          group and in any other thread group that has this thread
      *          group as an ancestor
      *
-     * @since   JDK1.0
+     * @since   1.0
      */
     public int activeCount() {
         int result;
@@ -387,7 +394,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      *          if {@linkplain #checkAccess checkAccess} determines that
      *          the current thread cannot access this thread group
      *
-     * @since   JDK1.0
+     * @since   1.0
      */
     public int enumerate(Thread list[]) {
         checkAccess();
@@ -425,7 +432,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      *          if {@linkplain #checkAccess checkAccess} determines that
      *          the current thread cannot access this thread group
      *
-     * @since   JDK1.0
+     * @since   1.0
      */
     public int enumerate(Thread list[], boolean recurse) {
         checkAccess();
@@ -478,7 +485,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * @return  the number of active thread groups with this thread group as
      *          an ancestor
      *
-     * @since   JDK1.0
+     * @since   1.0
      */
     public int activeGroupCount() {
         int ngroupsSnapshot;
@@ -521,7 +528,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      *          if {@linkplain #checkAccess checkAccess} determines that
      *          the current thread cannot access this thread group
      *
-     * @since   JDK1.0
+     * @since   1.0
      */
     public int enumerate(ThreadGroup list[]) {
         checkAccess();
@@ -559,7 +566,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      *          if {@linkplain #checkAccess checkAccess} determines that
      *          the current thread cannot access this thread group
      *
-     * @since   JDK1.0
+     * @since   1.0
      */
     public int enumerate(ThreadGroup list[], boolean recurse) {
         checkAccess();
@@ -601,23 +608,23 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Stops all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>stop</code> method on all the
+     * This method then calls the {@code stop} method on all the
      * threads in this thread group and in all of its subgroups.
      *
-     * @exception  SecurityException  if the current thread is not allowed
+     * @throws     SecurityException  if the current thread is not allowed
      *               to access this thread group or any of the threads in
      *               the thread group.
      * @see        java.lang.SecurityException
      * @see        java.lang.Thread#stop()
      * @see        java.lang.ThreadGroup#checkAccess()
-     * @since      JDK1.0
+     * @since      1.0
      * @deprecated    This method is inherently unsafe.  See
      *     {@link Thread#stop} for details.
      */
-    @Deprecated
+    @Deprecated(since="1.2")
     public final void stop() {
         if (stopOrSuspend(false))
             Thread.currentThread().stop();
@@ -626,13 +633,13 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Interrupts all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>interrupt</code> method on all the
+     * This method then calls the {@code interrupt} method on all the
      * threads in this thread group and in all of its subgroups.
      *
-     * @exception  SecurityException  if the current thread is not allowed
+     * @throws     SecurityException  if the current thread is not allowed
      *               to access this thread group or any of the threads in
      *               the thread group.
      * @see        java.lang.Thread#interrupt()
@@ -663,23 +670,23 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Suspends all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>suspend</code> method on all the
+     * This method then calls the {@code suspend} method on all the
      * threads in this thread group and in all of its subgroups.
      *
-     * @exception  SecurityException  if the current thread is not allowed
+     * @throws     SecurityException  if the current thread is not allowed
      *               to access this thread group or any of the threads in
      *               the thread group.
      * @see        java.lang.Thread#suspend()
      * @see        java.lang.SecurityException
      * @see        java.lang.ThreadGroup#checkAccess()
-     * @since      JDK1.0
+     * @since      1.0
      * @deprecated    This method is inherently deadlock-prone.  See
      *     {@link Thread#suspend} for details.
      */
-    @Deprecated
+    @Deprecated(since="1.2")
     @SuppressWarnings("deprecation")
     public final void suspend() {
         if (stopOrSuspend(true))
@@ -724,25 +731,25 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Resumes all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>resume</code> method on all the
+     * This method then calls the {@code resume} method on all the
      * threads in this thread group and in all of its sub groups.
      *
-     * @exception  SecurityException  if the current thread is not allowed to
+     * @throws     SecurityException  if the current thread is not allowed to
      *               access this thread group or any of the threads in the
      *               thread group.
      * @see        java.lang.SecurityException
      * @see        java.lang.Thread#resume()
      * @see        java.lang.ThreadGroup#checkAccess()
-     * @since      JDK1.0
+     * @since      1.0
      * @deprecated    This method is used solely in conjunction with
-     *      <tt>Thread.suspend</tt> and <tt>ThreadGroup.suspend</tt>,
+     *       {@code Thread.suspend} and {@code ThreadGroup.suspend},
      *       both of which have been deprecated, as they are inherently
      *       deadlock-prone.  See {@link Thread#suspend} for details.
      */
-    @Deprecated
+    @Deprecated(since="1.2")
     @SuppressWarnings("deprecation")
     public final void resume() {
         int ngroupsSnapshot;
@@ -769,15 +776,15 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * group must be empty, indicating that all threads that had been in
      * this thread group have since stopped.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      *
-     * @exception  IllegalThreadStateException  if the thread group is not
+     * @throws     IllegalThreadStateException  if the thread group is not
      *               empty or if the thread group has already been destroyed.
-     * @exception  SecurityException  if the current thread cannot modify this
+     * @throws     SecurityException  if the current thread cannot modify this
      *               thread group.
      * @see        java.lang.ThreadGroup#checkAccess()
-     * @since      JDK1.0
+     * @since      1.0
      */
     public final void destroy() {
         int ngroupsSnapshot;
@@ -812,7 +819,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Adds the specified Thread group to this group.
      * @param g the specified Thread group to be added
-     * @exception IllegalThreadStateException If the Thread group has been destroyed.
+     * @throws  IllegalThreadStateException If the Thread group has been destroyed.
      */
     private final void add(ThreadGroup g){
         synchronized (this) {
@@ -890,7 +897,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * @param  t
      *         the Thread to be added
      *
-     * @throws  IllegalThreadStateException
+     * @throws IllegalThreadStateException
      *          if the Thread group has been destroyed
      */
     void add(Thread t) {
@@ -990,7 +997,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Prints information about this thread group to the standard
      * output. This method is useful only for debugging.
      *
-     * @since   JDK1.0
+     * @since   1.0
      */
     public void list() {
         list(System.out, 0);
@@ -1028,34 +1035,34 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * does not have a specific {@link Thread.UncaughtExceptionHandler}
      * installed.
      * <p>
-     * The <code>uncaughtException</code> method of
-     * <code>ThreadGroup</code> does the following:
+     * The {@code uncaughtException} method of
+     * {@code ThreadGroup} does the following:
      * <ul>
      * <li>If this thread group has a parent thread group, the
-     *     <code>uncaughtException</code> method of that parent is called
+     *     {@code uncaughtException} method of that parent is called
      *     with the same two arguments.
      * <li>Otherwise, this method checks to see if there is a
      *     {@linkplain Thread#getDefaultUncaughtExceptionHandler default
      *     uncaught exception handler} installed, and if so, its
-     *     <code>uncaughtException</code> method is called with the same
+     *     {@code uncaughtException} method is called with the same
      *     two arguments.
-     * <li>Otherwise, this method determines if the <code>Throwable</code>
+     * <li>Otherwise, this method determines if the {@code Throwable}
      *     argument is an instance of {@link ThreadDeath}. If so, nothing
      *     special is done. Otherwise, a message containing the
      *     thread's name, as returned from the thread's {@link
      *     Thread#getName getName} method, and a stack backtrace,
-     *     using the <code>Throwable</code>'s {@link
+     *     using the {@code Throwable}'s {@link
      *     Throwable#printStackTrace printStackTrace} method, is
      *     printed to the {@linkplain System#err standard error stream}.
      * </ul>
      * <p>
      * Applications can override this method in subclasses of
-     * <code>ThreadGroup</code> to provide alternative handling of
+     * {@code ThreadGroup} to provide alternative handling of
      * uncaught exceptions.
      *
      * @param   t   the thread that is about to exit.
      * @param   e   the uncaught exception.
-     * @since   JDK1.0
+     * @since   1.0
      */
     public void uncaughtException(Thread t, Throwable e) {
         if (parent != null) {
@@ -1073,22 +1080,28 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
         }
     }
 
+    /**
+     * Used by VM to control lowmem implicit suspension.
+     *
+     * @param b boolean to allow or disallow suspension
+     * @return true on success
+     * @since   1.1
+     * @deprecated The definition of this call depends on {@link #suspend},
+     *             which is deprecated.  Further, the behavior of this call
+     *             was never specified.
+     */
+    @Deprecated(since="1.2")
+    public boolean allowThreadSuspension(boolean b) {
+        return true;
+    }
+
     /**
      * Returns a string representation of this Thread group.
      *
      * @return  a string representation of this thread group.
-     * @since   JDK1.0
+     * @since   1.0
      */
     public String toString() {
         return getClass().getName() + "[name=" + getName() + ",maxpri=" + maxPriority + "]";
     }
-
-    // j2objc: main thread group needs to potentially handle lots of native
-    // thread additions and removals, so start with a bigger threads array.
-    private static class MainThreadGroup extends ThreadGroup {
-      MainThreadGroup(ThreadGroup parent) {
-        super(parent, "main");
-        threads = new Thread[32];
-      }
-    }
 }
