@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2017, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,6 +28,8 @@ package javax.net.ssl;
 
 import java.io.IOException;
 import java.net.*;
+import java.util.List;
+import java.util.function.BiFunction;
 
 
 /**
@@ -173,6 +175,11 @@ import java.net.*;
  *             <td>16+</td>
  *             <td>20+</td>
  *         </tr>
+ *         <tr>
+ *             <td>TLSv1.3</td>
+ *             <td>29+</td>
+ *             <td>29+</td>
+ *         </tr>
  *     </tbody>
  * </table>
  *
@@ -206,6 +213,11 @@ import java.net.*;
  *             <td>16+</td>
  *             <td>16+</td>
  *         </tr>
+ *         <tr>
+ *             <td>TLSv1.3</td>
+ *             <td>29+</td>
+ *             <td>29+</td>
+ *         </tr>
  *     </tbody>
  * </table>
  *
@@ -321,6 +333,21 @@ import java.net.*;
  *       <td>9-25</td>
  *       <td>9-23</td>
  *     </tr>
+ *     <tr>
+ *       <td>TLS_AES_128_GCM_SHA256</td>
+ *       <td>29+</td>
+ *       <td>29+</td>
+ *     </tr>
+ *     <tr>
+ *       <td>TLS_AES_256_GCM_SHA384</td>
+ *       <td>29+</td>
+ *       <td>29+</td>
+ *     </tr>
+ *     <tr>
+ *       <td>TLS_CHACHA20_POLY1305_SHA256</td>
+ *       <td>29+</td>
+ *       <td>29+</td>
+ *     </tr>
  *     <tr class="deprecated">
  *       <td>TLS_DHE_DSS_WITH_AES_128_CBC_SHA</td>
  *       <td>9-22</td>
@@ -421,9 +448,9 @@ import java.net.*;
  *       <td>11+</td>
  *       <td>11+</td>
  *     </tr>
- *     <tr>
+ *     <tr class="deprecated">
  *       <td>TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256</td>
- *       <td>20+</td>
+ *       <td>20-28</td>
  *       <td></td>
  *     </tr>
  *     <tr>
@@ -436,9 +463,9 @@ import java.net.*;
  *       <td>11+</td>
  *       <td>11+</td>
  *     </tr>
- *     <tr>
+ *     <tr class="deprecated">
  *       <td>TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384</td>
- *       <td>20+</td>
+ *       <td>20-28</td>
  *       <td></td>
  *     </tr>
  *     <tr>
@@ -486,9 +513,9 @@ import java.net.*;
  *       <td>11+</td>
  *       <td>11+</td>
  *     </tr>
- *     <tr>
+ *     <tr class="deprecated">
  *       <td>TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256</td>
- *       <td>20+</td>
+ *       <td>20-28</td>
  *       <td></td>
  *     </tr>
  *     <tr>
@@ -501,9 +528,9 @@ import java.net.*;
  *       <td>11+</td>
  *       <td>11+</td>
  *     </tr>
- *     <tr>
+ *     <tr class="deprecated">
  *       <td>TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384</td>
- *       <td>20+</td>
+ *       <td>20-28</td>
  *       <td></td>
  *     </tr>
  *     <tr>
@@ -676,9 +703,9 @@ import java.net.*;
  *       <td>9+</td>
  *       <td>9+</td>
  *     </tr>
- *     <tr>
+ *     <tr class="deprecated">
  *       <td>TLS_RSA_WITH_AES_128_CBC_SHA256</td>
- *       <td>20+</td>
+ *       <td>20-28</td>
  *       <td></td>
  *     </tr>
  *     <tr>
@@ -691,9 +718,9 @@ import java.net.*;
  *       <td>9+</td>
  *       <td>11+</td>
  *     </tr>
- *     <tr>
+ *     <tr class="deprecated">
  *       <td>TLS_RSA_WITH_AES_256_CBC_SHA256</td>
- *       <td>20+</td>
+ *       <td>20-28</td>
  *       <td></td>
  *     </tr>
  *     <tr>
@@ -992,6 +1019,7 @@ public abstract class SSLSocket extends Socket
         { super(address, port, clientAddress, clientPort); }
 
 
+    // Android-changed: Added warnings about misuse
     /**
      * Returns the names of the cipher suites which could be enabled for use
      * on this connection.  Normally, only a subset of these will actually
@@ -999,6 +1027,15 @@ public abstract class SSLSocket extends Socket
      * do not meet quality of service requirements for those defaults.  Such
      * cipher suites might be useful in specialized applications.
      *
+     * <p class="caution">Applications should not blindly enable all supported
+     * cipher suites.  The supported cipher suites can include signaling cipher suite
+     * values that can cause connection problems if enabled inappropriately.
+     *
+     * <p>The proper way to use this method is to either check if a specific cipher
+     * suite is supported via {@code Arrays.asList(getSupportedCipherSuites()).contains(...)}
+     * or to filter a desired list of cipher suites to only the supported ones via
+     * {@code desiredSuiteSet.retainAll(Arrays.asList(getSupportedCipherSuites()))}.
+     *
      * @return an array of cipher suite names
      * @see #getEnabledCipherSuites()
      * @see #setEnabledCipherSuites(String [])
@@ -1063,6 +1100,7 @@ public abstract class SSLSocket extends Socket
     public abstract String [] getEnabledProtocols();
 
 
+    // Android-added: Added paragraph about contiguous protocols.
     /**
      * Sets the protocol versions enabled for use on this connection.
      * <P>
@@ -1070,6 +1108,11 @@ public abstract class SSLSocket extends Socket
      * <code>getSupportedProtocols()</code> as being supported.
      * Following a successful call to this method, only protocols listed
      * in the <code>protocols</code> parameter are enabled for use.
+     * <p>
+     * Because of the way the protocol version is negotiated, connections
+     * will only be able to use a member of the lowest set of contiguous
+     * enabled protocol versions.  For example, enabling TLSv1.2 and TLSv1
+     * will result in connections only being able to use TLSv1.
      *
      * @param protocols Names of all the protocols to enable.
      * @throws IllegalArgumentException when one or more of
@@ -1395,6 +1438,148 @@ public abstract class SSLSocket extends Socket
         }
     }
 
+    // BEGIN Android-added: Add ALPN-related methods from OpenJDK 9.
+    // Also removed references to DTLS in documentation; Android doesn't support DTLS.
+    /**
+     * Returns the most recent application protocol value negotiated for this
+     * connection.
+     * <p>
+     * If supported by the underlying SSL/TLS implementation,
+     * application name negotiation mechanisms such as <a
+     * href="http://www.ietf.org/rfc/rfc7301.txt"> RFC 7301 </a>, the
+     * Application-Layer Protocol Negotiation (ALPN), can negotiate
+     * application-level values between peers.
+     * <p>
+     * @implSpec
+     * The implementation in this class throws
+     * {@code UnsupportedOperationException} and performs no other action.
+     *
+     * @return null if it has not yet been determined if application
+     *         protocols might be used for this connection, an empty
+     *         {@code String} if application protocols values will not
+     *         be used, or a non-empty application protocol {@code String}
+     *         if a value was successfully negotiated.
+     * @throws UnsupportedOperationException if the underlying provider
+     *         does not implement the operation.
+     * @since 9
+     */
+    public String getApplicationProtocol() {
+        throw new UnsupportedOperationException();
+    }
+
+    /**
+     * Returns the application protocol value negotiated on a SSL/TLS
+     * handshake currently in progress.
+     * <p>
+     * Like {@link #getHandshakeSession()},
+     * a connection may be in the middle of a handshake. The
+     * application protocol may or may not yet be available.
+     * <p>
+     * @implSpec
+     * The implementation in this class throws
+     * {@code UnsupportedOperationException} and performs no other action.
+     *
+     * @return null if it has not yet been determined if application
+     *         protocols might be used for this handshake, an empty
+     *         {@code String} if application protocols values will not
+     *         be used, or a non-empty application protocol {@code String}
+     *         if a value was successfully negotiated.
+     * @throws UnsupportedOperationException if the underlying provider
+     *         does not implement the operation.
+     * @since 9
+     */
+    public String getHandshakeApplicationProtocol() {
+        throw new UnsupportedOperationException();
+    }
+
+
+    /**
+     * Registers a callback function that selects an application protocol
+     * value for a SSL/TLS handshake.
+     * The function overrides any values supplied using
+     * {@link SSLParameters#setApplicationProtocols
+     * SSLParameters.setApplicationProtocols} and it supports the following
+     * type parameters:
+     * <blockquote>
+     * <dl>
+     * <dt> {@code SSLSocket}
+     * <dd> The function's first argument allows the current {@code SSLSocket}
+     *      to be inspected, including the handshake session and configuration
+     *      settings.
+     * <dt> {@code List<String>}
+     * <dd> The function's second argument lists the application protocol names
+     *      advertised by the TLS peer.
+     * <dt> {@code String}
+     * <dd> The function's result is an application protocol name, or null to
+     *      indicate that none of the advertised names are acceptable.
+     *      If the return value is an empty {@code String} then application
+     *      protocol indications will not be used.
+     *      If the return value is null (no value chosen) or is a value that
+     *      was not advertised by the peer, the underlying protocol will
+     *      determine what action to take. (For example, ALPN will send a
+     *      "no_application_protocol" alert and terminate the connection.)
+     * </dl>
+     * </blockquote>
+     *
+     * For example, the following call registers a callback function that
+     * examines the TLS handshake parameters and selects an application protocol
+     * name:
+     * <pre>{@code
+     *     serverSocket.setHandshakeApplicationProtocolSelector(
+     *         (serverSocket, clientProtocols) -> {
+     *             SSLSession session = serverSocket.getHandshakeSession();
+     *             return chooseApplicationProtocol(
+     *                 serverSocket,
+     *                 clientProtocols,
+     *                 session.getProtocol(),
+     *                 session.getCipherSuite());
+     *         });
+     * }</pre>
+     *
+     * @apiNote
+     * This method should be called by TLS server applications before the TLS
+     * handshake begins. Also, this {@code SSLSocket} should be configured with
+     * parameters that are compatible with the application protocol selected by
+     * the callback function. For example, enabling a poor choice of cipher
+     * suites could result in no suitable application protocol.
+     * See {@link SSLParameters}.
+     *
+     * @implSpec
+     * The implementation in this class throws
+     * {@code UnsupportedOperationException} and performs no other action.
+     *
+     * @param selector the callback function, or null to de-register.
+     * @throws UnsupportedOperationException if the underlying provider
+     *         does not implement the operation.
+     * @since 9
+     */
+    public void setHandshakeApplicationProtocolSelector(
+            BiFunction<SSLSocket, List<String>, String> selector) {
+        throw new UnsupportedOperationException();
+    }
+
+    /**
+     * Retrieves the callback function that selects an application protocol
+     * value during a SSL/TLS handshake.
+     * See {@link #setHandshakeApplicationProtocolSelector
+     * setHandshakeApplicationProtocolSelector}
+     * for the function's type parameters.
+     *
+     * @implSpec
+     * The implementation in this class throws
+     * {@code UnsupportedOperationException} and performs no other action.
+     *
+     * @return the callback function, or null if none has been set.
+     * @throws UnsupportedOperationException if the underlying provider
+     *         does not implement the operation.
+     * @since 9
+     */
+    public BiFunction<SSLSocket, List<String>, String>
+            getHandshakeApplicationProtocolSelector() {
+        throw new UnsupportedOperationException();
+    }
+    // END Android-added: Add ALPN-related methods from OpenJDK 9.
+
     // Android-added: Make toString explicit that this is an SSLSocket (http://b/6602228)
     @Override
     public String toString() {
