@@ -116,10 +116,8 @@ static void initGroupSourceReq(JNIEnv* env, jbyteArray group, jint index,
 }
 #endif
 
-JNIEXPORT jboolean JNICALL
-Java_sun_nio_ch_Net_isIPv6Available0(JNIEnv* env, jclass cl)
-{
-    return (ipv6_available()) ? JNI_TRUE : JNI_FALSE;
+JNIEXPORT bool JNICALL Java_sun_nio_ch_Net_isIPv6Available0(JNIEnv *env, jclass cl) {
+  return (ipv6_available()) ? JNI_TRUE : JNI_FALSE;
 }
 
 JNIEXPORT jint JNICALL
@@ -127,9 +125,7 @@ Java_sun_nio_ch_Net_isExclusiveBindAvailable(JNIEnv *env, jclass clazz) {
     return -1;
 }
 
-JNIEXPORT jboolean JNICALL
-Java_sun_nio_ch_Net_canIPv6SocketJoinIPv4Group0(JNIEnv* env, jclass cl)
-{
+JNIEXPORT bool JNICALL Java_sun_nio_ch_Net_canIPv6SocketJoinIPv4Group0(JNIEnv *env, jclass cl) {
 #ifdef MACOSX
     /* for now IPv6 sockets cannot join IPv4 multicast groups */
     return JNI_FALSE;
@@ -138,9 +134,7 @@ Java_sun_nio_ch_Net_canIPv6SocketJoinIPv4Group0(JNIEnv* env, jclass cl)
 #endif
 }
 
-JNIEXPORT jboolean JNICALL
-Java_sun_nio_ch_Net_canJoin6WithIPv4Group0(JNIEnv* env, jclass cl)
-{
+JNIEXPORT bool JNICALL Java_sun_nio_ch_Net_canJoin6WithIPv4Group0(JNIEnv *env, jclass cl) {
 #ifdef __solaris__
     return JNI_TRUE;
 #else
@@ -148,13 +142,11 @@ Java_sun_nio_ch_Net_canJoin6WithIPv4Group0(JNIEnv* env, jclass cl)
 #endif
 }
 
-JNIEXPORT int JNICALL
-Java_sun_nio_ch_Net_socket0(JNIEnv *env, jclass cl, jboolean preferIPv6,
-                            jboolean stream, jboolean reuse)
-{
-    int fd;
-    int type = (stream ? SOCK_STREAM : SOCK_DGRAM);
-    NSString *msg = @"sun.nio.ch.Net.setIntOption";
+JNIEXPORT int JNICALL Java_sun_nio_ch_Net_socket0(JNIEnv *env, jclass cl, bool preferIPv6,
+                                                  bool stream, bool reuse) {
+  int fd;
+  int type = (stream ? SOCK_STREAM : SOCK_DGRAM);
+  NSString *msg = @"sun.nio.ch.Net.setIntOption";
 #ifdef AF_INET6
     int domain = (ipv6_available() && preferIPv6) ? AF_INET6 : AF_INET;
 #else
@@ -222,22 +214,21 @@ Java_sun_nio_ch_Net_socket0(JNIEnv *env, jclass cl, jboolean preferIPv6,
     return fd;
 }
 
-JNIEXPORT void JNICALL
-Java_sun_nio_ch_Net_bind0(JNIEnv *env, jclass clazz, jobject fdo, jboolean preferIPv6,
-                          jboolean useExclBind, jobject iao, int port)
-{
-    SOCKADDR sa;
-    int sa_len = SOCKADDR_LEN;
-    int rv = 0;
+JNIEXPORT void JNICALL Java_sun_nio_ch_Net_bind0(JNIEnv *env, jclass clazz, jobject fdo,
+                                                 bool preferIPv6, bool useExclBind, jobject iao,
+                                                 int port) {
+  SOCKADDR sa;
+  int sa_len = SOCKADDR_LEN;
+  int rv = 0;
 
-    if (NET_InetAddressToSockaddr(env, iao, port, (struct sockaddr *)&sa, &sa_len, preferIPv6) != 0) {
-      return;
-    }
+  if (NET_InetAddressToSockaddr(env, iao, port, (struct sockaddr *)&sa, &sa_len, preferIPv6) != 0) {
+    return;
+  }
 
-    rv = NET_Bind(fdval(env, fdo), (struct sockaddr *)&sa, sa_len);
-    if (rv != 0) {
-        handleSocketError(env, errno);
-    }
+  rv = NET_Bind(fdval(env, fdo), (struct sockaddr *)&sa, sa_len);
+  if (rv != 0) {
+    handleSocketError(env, errno);
+  }
 }
 
 JNIEXPORT void JNICALL
@@ -247,30 +238,26 @@ Java_sun_nio_ch_Net_listen(JNIEnv *env, jclass cl, jobject fdo, jint backlog)
         handleSocketError(env, errno);
 }
 
-JNIEXPORT jint JNICALL
-Java_sun_nio_ch_Net_connect0(JNIEnv *env, jclass clazz, jboolean preferIPv6,
-                             jobject fdo, jobject iao, jint port)
-{
-    SOCKADDR sa;
-    int sa_len = SOCKADDR_LEN;
-    int rv;
+JNIEXPORT jint JNICALL Java_sun_nio_ch_Net_connect0(JNIEnv *env, jclass clazz, bool preferIPv6,
+                                                    jobject fdo, jobject iao, jint port) {
+  SOCKADDR sa;
+  int sa_len = SOCKADDR_LEN;
+  int rv;
 
-    if (NET_InetAddressToSockaddr(env, iao, port, (struct sockaddr *) &sa,
-                                  &sa_len, preferIPv6) != 0)
-    {
-      return IOS_THROWN;
-    }
+  if (NET_InetAddressToSockaddr(env, iao, port, (struct sockaddr *)&sa, &sa_len, preferIPv6) != 0) {
+    return IOS_THROWN;
+  }
 
-    rv = connect(fdval(env, fdo), (struct sockaddr *)&sa, sa_len);
-    if (rv != 0) {
-        if (errno == EINPROGRESS) {
-            return IOS_UNAVAILABLE;
-        } else if (errno == EINTR) {
-            return IOS_INTERRUPTED;
-        }
-        return handleSocketErrorWithDefault(env, errno, JNU_JAVANETPKG "ConnectException");
+  rv = connect(fdval(env, fdo), (struct sockaddr *)&sa, sa_len);
+  if (rv != 0) {
+    if (errno == EINPROGRESS) {
+      return IOS_UNAVAILABLE;
+    } else if (errno == EINTR) {
+      return IOS_INTERRUPTED;
     }
-    return 1;
+    return handleSocketErrorWithDefault(env, errno, JNU_JAVANETPKG "ConnectException");
+  }
+  return 1;
 }
 
 JNIEXPORT jint JNICALL
@@ -342,128 +329,119 @@ Java_sun_nio_ch_Net_localInetAddress(JNIEnv *env, jclass clazz, jobject fdo)
     return NET_SockaddrToInetAddress(env, (struct sockaddr *)&sa, &port);
 }
 
-JNIEXPORT jint JNICALL
-Java_sun_nio_ch_Net_getIntOption0(JNIEnv *env, jclass clazz, jobject fdo,
-                                  jboolean mayNeedConversion, jint level, jint opt)
-{
-    int result;
-    struct linger linger;
-    u_char carg;
-    void *arg;
-    socklen_t arglen;
-    int n;
+JNIEXPORT jint JNICALL Java_sun_nio_ch_Net_getIntOption0(JNIEnv *env, jclass clazz, jobject fdo,
+                                                         bool mayNeedConversion, jint level,
+                                                         jint opt) {
+  int result;
+  struct linger linger;
+  u_char carg;
+  void *arg;
+  socklen_t arglen;
+  int n;
 
-    /* Option value is an int except for a few specific cases */
+  /* Option value is an int except for a few specific cases */
 
-    arg = (void *)&result;
-    arglen = sizeof(result);
+  arg = (void *)&result;
+  arglen = sizeof(result);
 
-    if (level == IPPROTO_IP &&
-        (opt == IP_MULTICAST_TTL || opt == IP_MULTICAST_LOOP)) {
-        arg = (void*)&carg;
-        arglen = sizeof(carg);
-    }
+  if (level == IPPROTO_IP && (opt == IP_MULTICAST_TTL || opt == IP_MULTICAST_LOOP)) {
+    arg = (void *)&carg;
+    arglen = sizeof(carg);
+  }
 
-    if (level == SOL_SOCKET && opt == SO_LINGER) {
-        arg = (void *)&linger;
-        arglen = sizeof(linger);
-    }
+  if (level == SOL_SOCKET && opt == SO_LINGER) {
+    arg = (void *)&linger;
+    arglen = sizeof(linger);
+  }
 
-    if (mayNeedConversion) {
-        n = NET_GetSockOpt(fdval(env, fdo), level, opt, arg, (int*)&arglen);
-    } else {
-        n = getsockopt(fdval(env, fdo), level, opt, arg, &arglen);
-    }
-    if (n < 0) {
-        /* TODO(zgao): Revert this change after JNU_ThrowByNameWithLastError() is implemented.
-        JNU_ThrowByNameWithLastError(env,
-                                     JNU_JAVANETPKG "SocketException",
-                                     "sun.nio.ch.Net.getIntOption");
-         return -1;
-        */
-        J2ObjCThrowByName(
-            JavaNetSocketException, @"sun.nio.ch.Net.getIntOption");
-    }
+  if (mayNeedConversion) {
+    n = NET_GetSockOpt(fdval(env, fdo), level, opt, arg, (int *)&arglen);
+  } else {
+    n = getsockopt(fdval(env, fdo), level, opt, arg, &arglen);
+  }
+  if (n < 0) {
+    /* TODO(zgao): Revert this change after JNU_ThrowByNameWithLastError() is implemented.
+    JNU_ThrowByNameWithLastError(env,
+                                 JNU_JAVANETPKG "SocketException",
+                                 "sun.nio.ch.Net.getIntOption");
+     return -1;
+    */
+    J2ObjCThrowByName(JavaNetSocketException, @"sun.nio.ch.Net.getIntOption");
+  }
 
-    if (level == IPPROTO_IP &&
-        (opt == IP_MULTICAST_TTL || opt == IP_MULTICAST_LOOP))
-    {
-        return (jint)carg;
-    }
+  if (level == IPPROTO_IP && (opt == IP_MULTICAST_TTL || opt == IP_MULTICAST_LOOP)) {
+    return (jint)carg;
+  }
 
-    if (level == SOL_SOCKET && opt == SO_LINGER)
-        return linger.l_onoff ? (jint)linger.l_linger : (jint)-1;
+  if (level == SOL_SOCKET && opt == SO_LINGER)
+    return linger.l_onoff ? (jint)linger.l_linger : (jint)-1;
 
-    return (jint)result;
+  return (jint)result;
 }
 
-JNIEXPORT void JNICALL
-Java_sun_nio_ch_Net_setIntOption0(JNIEnv *env, jclass clazz, jobject fdo,
-                                  jboolean mayNeedConversion, jint level, jint opt, jint arg)
-{
-    int result;
-    struct linger linger;
-    u_char carg;
-    void *parg;
-    socklen_t arglen;
-    int n;
+JNIEXPORT void JNICALL Java_sun_nio_ch_Net_setIntOption0(JNIEnv *env, jclass clazz, jobject fdo,
+                                                         bool mayNeedConversion, jint level,
+                                                         jint opt, jint arg) {
+  int result;
+  struct linger linger;
+  u_char carg;
+  void *parg;
+  socklen_t arglen;
+  int n;
 
-    /* Option value is an int except for a few specific cases */
+  /* Option value is an int except for a few specific cases */
 
-    parg = (void*)&arg;
-    arglen = sizeof(arg);
+  parg = (void *)&arg;
+  arglen = sizeof(arg);
 
-    if (level == IPPROTO_IP &&
-        (opt == IP_MULTICAST_TTL || opt == IP_MULTICAST_LOOP)) {
-        parg = (void*)&carg;
-        arglen = sizeof(carg);
-        carg = (u_char)arg;
-    }
+  if (level == IPPROTO_IP && (opt == IP_MULTICAST_TTL || opt == IP_MULTICAST_LOOP)) {
+    parg = (void *)&carg;
+    arglen = sizeof(carg);
+    carg = (u_char)arg;
+  }
 
-    if (level == SOL_SOCKET && opt == SO_LINGER) {
-        parg = (void *)&linger;
-        arglen = sizeof(linger);
-        if (arg >= 0) {
-            linger.l_onoff = 1;
-            linger.l_linger = arg;
-        } else {
-            linger.l_onoff = 0;
-            linger.l_linger = 0;
-        }
-    }
-
-    if (mayNeedConversion) {
-        n = NET_SetSockOpt(fdval(env, fdo), level, opt, parg, arglen);
+  if (level == SOL_SOCKET && opt == SO_LINGER) {
+    parg = (void *)&linger;
+    arglen = sizeof(linger);
+    if (arg >= 0) {
+      linger.l_onoff = 1;
+      linger.l_linger = arg;
     } else {
-        n = setsockopt(fdval(env, fdo), level, opt, parg, arglen);
-    }
-    if (n < 0) {
-        /* TODO(zgao): Revert this change after JNU_ThrowByNameWithLastError() is implemented.
-        JNU_ThrowByNameWithLastError(env,
-                                     JNU_JAVANETPKG "SocketException",
-                                     "sun.nio.ch.Net.setIntOption");
-        */
-        J2ObjCThrowByName(
-            JavaNetSocketException, @"sun.nio.ch.Net.setIntOption");
+      linger.l_onoff = 0;
+      linger.l_linger = 0;
     }
+  }
+
+  if (mayNeedConversion) {
+    n = NET_SetSockOpt(fdval(env, fdo), level, opt, parg, arglen);
+  } else {
+    n = setsockopt(fdval(env, fdo), level, opt, parg, arglen);
+  }
+  if (n < 0) {
+    /* TODO(zgao): Revert this change after JNU_ThrowByNameWithLastError() is implemented.
+    JNU_ThrowByNameWithLastError(env,
+                                 JNU_JAVANETPKG "SocketException",
+                                 "sun.nio.ch.Net.setIntOption");
+    */
+    J2ObjCThrowByName(JavaNetSocketException, @"sun.nio.ch.Net.setIntOption");
+  }
 }
 
-JNIEXPORT jint JNICALL
-Java_sun_nio_ch_Net_joinOrDrop4(JNIEnv *env, jobject this, jboolean join, jobject fdo,
-                                jint group, jint interf, jint source)
-{
-    struct ip_mreq mreq;
-    struct my_ip_mreq_source mreq_source;
-    int opt, n, optlen;
-    void* optval;
+JNIEXPORT jint JNICALL Java_sun_nio_ch_Net_joinOrDrop4(JNIEnv *env, jobject this, bool join,
+                                                       jobject fdo, jint group, jint interf,
+                                                       jint source) {
+  struct ip_mreq mreq;
+  struct my_ip_mreq_source mreq_source;
+  int opt, n, optlen;
+  void *optval;
 
-    if (source == 0) {
-        mreq.imr_multiaddr.s_addr = htonl(group);
-        mreq.imr_interface.s_addr = htonl(interf);
-        opt = (join) ? IP_ADD_MEMBERSHIP : IP_DROP_MEMBERSHIP;
-        optval = (void*)&mreq;
-        optlen = sizeof(mreq);
-    } else {
+  if (source == 0) {
+    mreq.imr_multiaddr.s_addr = htonl(group);
+    mreq.imr_interface.s_addr = htonl(interf);
+    opt = (join) ? IP_ADD_MEMBERSHIP : IP_DROP_MEMBERSHIP;
+    optval = (void *)&mreq;
+    optlen = sizeof(mreq);
+  } else {
 #ifdef MACOSX
         /* no IPv4 include-mode filtering for now */
         return IOS_UNAVAILABLE;
@@ -496,10 +474,9 @@ Java_sun_nio_ch_Net_joinOrDrop4(JNIEnv *env, jobject this, jboolean join, jobjec
     return 0;
 }
 
-JNIEXPORT jint JNICALL
-Java_sun_nio_ch_Net_blockOrUnblock4(JNIEnv *env, jobject this, jboolean block, jobject fdo,
-                                    jint group, jint interf, jint source)
-{
+JNIEXPORT jint JNICALL Java_sun_nio_ch_Net_blockOrUnblock4(JNIEnv *env, jobject this, bool block,
+                                                           jobject fdo, jint group, jint interf,
+                                                           jint source) {
 #ifdef MACOSX
     /* no IPv4 exclude-mode filtering for now */
     return IOS_UNAVAILABLE;
@@ -532,10 +509,9 @@ Java_sun_nio_ch_Net_blockOrUnblock4(JNIEnv *env, jobject this, jboolean block, j
 #endif
 }
 
-JNIEXPORT jint JNICALL
-Java_sun_nio_ch_Net_joinOrDrop6(JNIEnv *env, jobject this, jboolean join, jobject fdo,
-                                jbyteArray group, jint index, jbyteArray source)
-{
+JNIEXPORT jint JNICALL Java_sun_nio_ch_Net_joinOrDrop6(JNIEnv *env, jobject this, bool join,
+                                                       jobject fdo, jbyteArray group, jint index,
+                                                       jbyteArray source) {
 #ifdef AF_INET6
     struct ipv6_mreq mreq6;
     struct my_group_source_req req;
@@ -573,10 +549,9 @@ Java_sun_nio_ch_Net_joinOrDrop6(JNIEnv *env, jobject this, jboolean join, jobjec
 #endif  /* AF_INET6 */
 }
 
-JNIEXPORT jint JNICALL
-Java_sun_nio_ch_Net_blockOrUnblock6(JNIEnv *env, jobject this, jboolean block, jobject fdo,
-                                    jbyteArray group, jint index, jbyteArray source)
-{
+JNIEXPORT jint JNICALL Java_sun_nio_ch_Net_blockOrUnblock6(JNIEnv *env, jobject this, bool block,
+                                                           jobject fdo, jbyteArray group,
+                                                           jint index, jbyteArray source) {
 #ifdef AF_INET6
   #ifdef MACOSX
     /* no IPv6 exclude-mode filtering for now */
