@@ -107,7 +107,7 @@ void ThrowUnknownHostExceptionWithGaiError(JNIEnv *env,
 
 /* do we have address translation support */
 
-extern jboolean NET_addrtransAvailable(void);
+extern bool NET_addrtransAvailable(void);
 
 #define NET_WAIT_READ   0x01
 #define NET_WAIT_WRITE  0x02
