@@ -165,18 +165,17 @@ ZFILE_read(ZFILE zfd, char *buf, jint nbytes, jlong offset) {
 static jint
 InitializeZip()
 {
-    static jboolean inited = JNI_FALSE;
+  static bool inited = JNI_FALSE;
 
-    // Initialize errno to 0.  It may be set later (e.g. during memory
-    // allocation) but we can disregard previous values.
-    errno = 0;
+  // Initialize errno to 0.  It may be set later (e.g. during memory
+  // allocation) but we can disregard previous values.
+  errno = 0;
 
-    if (inited)
-        return 0;
-    zfiles_lock = MCREATE();
-    if (zfiles_lock == 0) {
-        return -1;
-    }
+  if (inited) return 0;
+  zfiles_lock = MCREATE();
+  if (zfiles_lock == 0) {
+    return -1;
+  }
     inited = JNI_TRUE;
 
     return 0;
@@ -265,22 +264,19 @@ static const jlong END_MAXLEN = 0xFFFF + ZIP_ENDHDR;
 
 #define READBLOCKSZ 128
 
-static jboolean verifyEND(jzfile *zip, jlong endpos, char *endbuf) {
-    /* ENDSIG matched, however the size of file comment in it does not
-       match the real size. One "common" cause for this problem is some
-       "extra" bytes are padded at the end of the zipfile.
-       Let's do some extra verification, we don't care about the performance
-       in this situation.
-     */
-    jlong cenpos = endpos - ZIP_ENDSIZ(endbuf);
-    jlong locpos = cenpos - ZIP_ENDOFF(endbuf);
-    char buf[4];
-    return (cenpos >= 0 &&
-            locpos >= 0 &&
-            readFullyAt(zip->zfd, buf, sizeof(buf), cenpos) != -1 &&
-            GETSIG(buf) == ZIP_CENSIG &&
-            readFullyAt(zip->zfd, buf, sizeof(buf), locpos) != -1 &&
-            GETSIG(buf) == ZIP_LOCSIG);
+static bool verifyEND(jzfile *zip, jlong endpos, char *endbuf) {
+  /* ENDSIG matched, however the size of file comment in it does not
+     match the real size. One "common" cause for this problem is some
+     "extra" bytes are padded at the end of the zipfile.
+     Let's do some extra verification, we don't care about the performance
+     in this situation.
+   */
+  jlong cenpos = endpos - ZIP_ENDSIZ(endbuf);
+  jlong locpos = cenpos - ZIP_ENDOFF(endbuf);
+  char buf[4];
+  return (cenpos >= 0 && locpos >= 0 && readFullyAt(zip->zfd, buf, sizeof(buf), cenpos) != -1 &&
+          GETSIG(buf) == ZIP_CENSIG && readFullyAt(zip->zfd, buf, sizeof(buf), locpos) != -1 &&
+          GETSIG(buf) == ZIP_LOCSIG);
 }
 
 /*
@@ -840,17 +836,15 @@ ZIP_Put_In_Cache(const char *name, ZFILE zfd, char **pmsg, jlong lastModified)
     return ZIP_Put_In_Cache0(name, zfd, pmsg, lastModified, JNI_TRUE);
 }
 
-jzfile *
-ZIP_Put_In_Cache0(const char *name, ZFILE zfd, char **pmsg, jlong lastModified,
-                 jboolean usemmap)
-{
-    char errbuf[256];
-    jlong len;
-    jzfile *zip;
+jzfile *ZIP_Put_In_Cache0(const char *name, ZFILE zfd, char **pmsg, jlong lastModified,
+                          bool usemmap) {
+  char errbuf[256];
+  jlong len;
+  jzfile *zip;
 
-    if ((zip = allocZip(name)) == NULL) {
-        return NULL;
-    }
+  if ((zip = allocZip(name)) == NULL) {
+    return NULL;
+  }
 
 #ifdef USE_MMAP
     zip->usemmap = usemmap;
@@ -1370,64 +1364,62 @@ ZIP_Read(jzfile *zip, jzentry *entry, jlong pos, void *buf, jint len)
  * The current implementation does not support reading an entry that
  * has the size bigger than 2**32 bytes in ONE invocation.
  */
-jboolean
-InflateFully(jzfile *zip, jzentry *entry, void *buf, char **msg)
-{
-    z_stream strm;
-    char tmp[BUF_SIZE];
-    jlong pos = 0;
-    jlong count = entry->csize;
+bool InflateFully(jzfile *zip, jzentry *entry, void *buf, char **msg) {
+  z_stream strm;
+  char tmp[BUF_SIZE];
+  jlong pos = 0;
+  jlong count = entry->csize;
 
-    *msg = 0; /* Reset error message */
+  *msg = 0; /* Reset error message */
 
-    if (count == 0) {
-        *msg = "inflateFully: entry not compressed";
-        return JNI_FALSE;
+  if (count == 0) {
+    *msg = "inflateFully: entry not compressed";
+    return JNI_FALSE;
+  }
+
+  memset(&strm, 0, sizeof(z_stream));
+  if (inflateInit2(&strm, -MAX_WBITS) != Z_OK) {
+    *msg = strm.msg;
+    return JNI_FALSE;
+  }
+
+  strm.next_out = buf;
+  strm.avail_out = (uInt)entry->size;
+
+  while (count > 0) {
+    jint n = count > (jlong)sizeof(tmp) ? (jint)sizeof(tmp) : (jint)count;
+    ZIP_Lock(zip);
+    n = ZIP_Read(zip, entry, pos, tmp, n);
+    ZIP_Unlock(zip);
+    if (n <= 0) {
+      if (n == 0) {
+        *msg = "inflateFully: Unexpected end of file";
+      }
+      inflateEnd(&strm);
+      return JNI_FALSE;
     }
-
-    memset(&strm, 0, sizeof(z_stream));
-    if (inflateInit2(&strm, -MAX_WBITS) != Z_OK) {
-        *msg = strm.msg;
-        return JNI_FALSE;
-    }
-
-    strm.next_out = buf;
-    strm.avail_out = (uInt)entry->size;
-
-    while (count > 0) {
-        jint n = count > (jlong)sizeof(tmp) ? (jint)sizeof(tmp) : (jint)count;
-        ZIP_Lock(zip);
-        n = ZIP_Read(zip, entry, pos, tmp, n);
-        ZIP_Unlock(zip);
-        if (n <= 0) {
-            if (n == 0) {
-                *msg = "inflateFully: Unexpected end of file";
-            }
+    pos += n;
+    count -= n;
+    strm.next_in = (Bytef *)tmp;
+    strm.avail_in = n;
+    do {
+      switch (inflate(&strm, Z_PARTIAL_FLUSH)) {
+        case Z_OK:
+          break;
+        case Z_STREAM_END:
+          if (count != 0 || entry->size < 0 || strm.total_out != (uint64_t)entry->size) {
+            *msg = "inflateFully: Unexpected end of stream";
             inflateEnd(&strm);
             return JNI_FALSE;
-        }
-        pos += n;
-        count -= n;
-        strm.next_in = (Bytef *)tmp;
-        strm.avail_in = n;
-        do {
-            switch (inflate(&strm, Z_PARTIAL_FLUSH)) {
-            case Z_OK:
-                break;
-            case Z_STREAM_END:
-                if (count != 0 || entry->size < 0 || strm.total_out != (uint64_t)entry->size) {
-                    *msg = "inflateFully: Unexpected end of stream";
-                    inflateEnd(&strm);
-                    return JNI_FALSE;
-                }
-                break;
-            default:
-                break;
-            }
-        } while (strm.avail_in > 0);
-    }
-    inflateEnd(&strm);
-    return JNI_TRUE;
+          }
+          break;
+        default:
+          break;
+      }
+    } while (strm.avail_in > 0);
+  }
+  inflateEnd(&strm);
+  return JNI_TRUE;
 }
 
 /*
@@ -1451,49 +1443,45 @@ ZIP_FindEntry(jzfile *zip, char *name, jint *sizeP, jint *nameLenP)
  * Note: this is called from the separately delivered VM (hotspot/classic)
  * so we have to be careful to maintain the expected behaviour.
  */
-jboolean JNICALL
-ZIP_ReadEntry(jzfile *zip, jzentry *entry, unsigned char *buf, char *entryname)
-{
-    char *msg;
+bool JNICALL ZIP_ReadEntry(jzfile *zip, jzentry *entry, unsigned char *buf, char *entryname) {
+  char *msg;
 
-    strcpy(entryname, entry->name);
-    if (entry->csize == 0) {
-        /* Entry is stored */
-        jlong pos = 0;
-        jlong size = entry->size;
-        while (pos < size) {
-            jint n;
-            jlong limit = ((((jlong) 1) << 31) - 1);
-            jint count = (size - pos < limit) ?
-                /* These casts suppress a VC++ Internal Compiler Error */
-                (jint) (size - pos) :
-                (jint) limit;
-            ZIP_Lock(zip);
-            n = ZIP_Read(zip, entry, pos, buf, count);
-            msg = zip->msg;
-            ZIP_Unlock(zip);
-            if (n == -1) {
-                jio_fprintf(stderr, "%s: %s\n", zip->name,
-                            msg != 0 ? msg : strerror(errno));
-                return JNI_FALSE;
-            }
-            buf += n;
-            pos += n;
-        }
-    } else {
-        /* Entry is compressed */
-        int ok = InflateFully(zip, entry, buf, &msg);
-        if (!ok) {
-            if ((msg == NULL) || (*msg == 0)) {
-                msg = zip->msg;
-            }
-            jio_fprintf(stderr, "%s: %s\n", zip->name,
-                        msg != 0 ? msg : strerror(errno));
-            return JNI_FALSE;
-        }
+  strcpy(entryname, entry->name);
+  if (entry->csize == 0) {
+    /* Entry is stored */
+    jlong pos = 0;
+    jlong size = entry->size;
+    while (pos < size) {
+      jint n;
+      jlong limit = ((((jlong)1) << 31) - 1);
+      jint count = (size - pos < limit) ?
+                                        /* These casts suppress a VC++ Internal Compiler Error */
+                       (jint)(size - pos)
+                                        : (jint)limit;
+      ZIP_Lock(zip);
+      n = ZIP_Read(zip, entry, pos, buf, count);
+      msg = zip->msg;
+      ZIP_Unlock(zip);
+      if (n == -1) {
+        jio_fprintf(stderr, "%s: %s\n", zip->name, msg != 0 ? msg : strerror(errno));
+        return JNI_FALSE;
+      }
+      buf += n;
+      pos += n;
     }
+  } else {
+    /* Entry is compressed */
+    int ok = InflateFully(zip, entry, buf, &msg);
+    if (!ok) {
+      if ((msg == NULL) || (*msg == 0)) {
+        msg = zip->msg;
+      }
+      jio_fprintf(stderr, "%s: %s\n", zip->name, msg != 0 ? msg : strerror(errno));
+      return JNI_FALSE;
+    }
+  }
 
-    ZIP_FreeEntry(zip, entry);
+  ZIP_FreeEntry(zip, entry);
 
-    return JNI_TRUE;
+  return JNI_TRUE;
 }
