@@ -171,7 +171,37 @@ void EnumGenerator::GenerateHeader(io::Printer* printer) {
       "+ ($classname$ *)valueOfWithNSString:(NSString *)name;\n"
       "+ ($classname$ *)valueOfWithInt:($valuepreprocessorname$)value;\n"
       "+ ($classname$ *)forNumberWithInt:($valuepreprocessorname$)value;\n"
-      "- ($valuepreprocessorname$)getNumber;\n"
+      "- ($valuepreprocessorname$)getNumber;\n",
+      "classname", ClassName(descriptor_), "valueenumname",
+      CValueEnumName(descriptor_), "valuepreprocessorname",
+      CValuePreprocessorName(descriptor_));
+
+  if (IsGenerateProperties(descriptor_->file())) {
+    printer->Print("- ($valuepreprocessorname$)number;\n",
+                   "valuepreprocessorname",
+                   CValuePreprocessorName(descriptor_));
+  }
+
+  // We add an explicit Swift name to prevent weird implicit conversions
+  // ("my_ENUM_LITERAL"). If we want to change this to "myEnumLiteral", we'll
+  // need to revert cl/738525802
+  if (IsGenerateProperties(descriptor_->file())) {
+    for (int i = 0; i < canonical_values_.size(); i++) {
+      printer->Print(
+          "@property(class, readonly, retain) $classname$ *$name$ "
+          "NS_SWIFT_NAME($name$);\n",
+          "classname", ClassName(descriptor_), "name",
+          PropertyName(canonical_values_[i]));
+    }
+  }
+  if (!descriptor_->is_closed()) {
+    printer->Print(
+        "@property(class, readonly, retain) $classname$ *UNRECOGNIZED "
+        "NS_SWIFT_NAME(UNRECOGNIZED);\n",
+        "classname", ClassName(descriptor_));
+  }
+
+  printer->Print(
       "\n"
       "@end\n"
       "\n"
@@ -207,6 +237,11 @@ void EnumGenerator::GenerateHeader(io::Printer* printer) {
         "FOUNDATION_EXPORT $classname$ *$classname$_get_UNRECOGNIZED(void);\n",
         "classname", ClassName(descriptor_));
   }
+
+  if (IsGenerateProperties(descriptor_->file())) {
+    printer->Print("@compatibility_alias KNP$classname$ $classname$;\n",
+                   "classname", ClassName(descriptor_));
+  }
 }
 
 const int kMaxRowChars = 80;
@@ -232,7 +267,7 @@ void EnumGenerator::GenerateSource(io::Printer* printer) {
   // Count characters and only add line breaks when the line exceeds the max.
   int row_chars = kMaxRowChars + 1;
   for (int i = 0; i < canonical_values_.size(); i++) {
-    std::string name = canonical_values_[i]->name();
+    absl::string_view name = canonical_values_[i]->name();
     size_t added_chars = name.length() + 5;
     if (row_chars + added_chars > kMaxRowChars) {
       printer->Print("\n     ");
@@ -292,6 +327,35 @@ void EnumGenerator::GenerateSource(io::Printer* printer) {
       "valuepreprocessorname", CValuePreprocessorName(descriptor_),
       "is_closed", SimpleItoa(descriptor_->is_closed()));
 
+  if (IsGenerateProperties(descriptor_->file())) {
+    for (int i = 0; i < canonical_values_.size(); i++) {
+      printer->Print(
+          "+ ($classname$ *) $name$ {\n"
+          "  return $classname$_get_$original_name$();\n"
+          "}\n",
+          "classname", ClassName(descriptor_),         //
+          "name", PropertyName(canonical_values_[i]),  //
+          "original_name", canonical_values_[i]->name());
+    }
+  }
+
+  if (!descriptor_->is_closed()) {
+    printer->Print(
+        "+ ($classname$ *) UNRECOGNIZED {\n"
+        "  return $classname$_get_UNRECOGNIZED();\n"
+        "}\n",
+        "classname", ClassName(descriptor_));
+  }
+
+  if (IsGenerateProperties(descriptor_->file())) {
+    printer->Print(
+        "- ($valuepreprocessorname$)number {\n"
+        "  return [self getNumber];\n"
+        "}\n",
+        "classname", ClassName(descriptor_), "valuepreprocessorname",
+        CValuePreprocessorName(descriptor_));
+  }
+
   printer->Print(
       "- ($valuepreprocessorname$)getNumber {\n",
       "valuepreprocessorname", CValuePreprocessorName(descriptor_));
