@@ -163,11 +163,12 @@ void FileGenerator::GenerateHeader(GeneratorContext* context) {
 
   if (!GenerateMultipleFiles()) {
     for (int i = 0; i < file_->message_type_count(); i++) {
-      MessageGenerator* generator;
+      std::unique_ptr<MessageGenerator> generator;
       if (enforce_lite_) {
-        generator = new MessageLiteGenerator(file_->message_type(i));
+        generator =
+            std::make_unique<MessageLiteGenerator>(file_->message_type(i));
       } else {
-        generator = new MessageGenerator(file_->message_type(i));
+        generator = std::make_unique<MessageGenerator>(file_->message_type(i));
       }
       generator->CollectMessageOrBuilderImports(&headers);
       generator->CollectHeaderImports(&headers);
@@ -184,6 +185,8 @@ void FileGenerator::GenerateHeader(GeneratorContext* context) {
       "#pragma clang diagnostic push\n"
       "#pragma clang diagnostic ignored \"-Wnullability-completeness\""
       "\n"
+      "#pragma clang diagnostic ignored \"-Wobjc-protocol-property-synthesis\""
+      "\n"
       "@interface $classname$ : NSObject\n"
       "\n",
       "classname", ClassName(file_));
@@ -245,11 +248,12 @@ void FileGenerator::GenerateHeader(GeneratorContext* context) {
     }
 
     for (int i = 0; i < file_->message_type_count(); i++) {
-      MessageGenerator* generator;
+      std::unique_ptr<MessageGenerator> generator;
       if (enforce_lite_) {
-        generator = new MessageLiteGenerator(file_->message_type(i));
+        generator =
+            std::make_unique<MessageLiteGenerator>(file_->message_type(i));
       } else {
-        generator = new MessageGenerator(file_->message_type(i));
+        generator = std::make_unique<MessageGenerator>(file_->message_type(i));
       }
       generator->GenerateMessageOrBuilder(&printer);
       generator->GenerateHeader(&printer);
@@ -427,7 +431,7 @@ void FileGenerator::Generate(GeneratorContext* context) {
 
 void FileGenerator::GenerateEnumHeader(GeneratorContext* context,
                                        const EnumDescriptor* descriptor) {
-  std::string filename = output_dir_ + descriptor->name() + ".h";
+  std::string filename = absl::StrCat(output_dir_, descriptor->name(), ".h");
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -442,7 +446,7 @@ void FileGenerator::GenerateEnumHeader(GeneratorContext* context,
 
 void FileGenerator::GenerateEnumSource(GeneratorContext* context,
                                        const EnumDescriptor* descriptor) {
-  std::string filename = output_dir_ + descriptor->name() + ".m";
+  std::string filename = absl::StrCat(output_dir_, descriptor->name(), ".m");
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -450,7 +454,7 @@ void FileGenerator::GenerateEnumSource(GeneratorContext* context,
 
   EnumGenerator generator(descriptor);
   std::set<std::string> headers;
-  headers.insert(output_dir_ + descriptor->name() + ".h");
+  headers.insert(absl::StrCat(output_dir_, descriptor->name(), ".h"));
   AddSourceImports(headers);
   generator.CollectSourceImports(&headers);
   PrintImports(&headers, &printer);
@@ -460,7 +464,7 @@ void FileGenerator::GenerateEnumSource(GeneratorContext* context,
 
 void FileGenerator::GenerateMessageHeader(GeneratorContext* context,
                                           const Descriptor* descriptor) {
-  std::string filename = output_dir_ + descriptor->name() + ".h";
+  std::string filename = absl::StrCat(output_dir_, descriptor->name(), ".h");
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -468,7 +472,7 @@ void FileGenerator::GenerateMessageHeader(GeneratorContext* context,
 
   MessageGenerator generator(descriptor);
   std::set<std::string> headers;
-  headers.insert(output_dir_ + descriptor->name() + "OrBuilder.h");
+  headers.insert(absl::StrCat(output_dir_, descriptor->name(), "OrBuilder.h"));
   AddHeaderImports(headers);
   generator.CollectHeaderImports(&headers);
   PrintImports(&headers, &printer);
@@ -481,7 +485,7 @@ void FileGenerator::GenerateMessageHeader(GeneratorContext* context,
 
 void FileGenerator::GenerateMessageSource(GeneratorContext* context,
                                           const Descriptor* descriptor) {
-  std::string filename = output_dir_ + descriptor->name() + ".m";
+  std::string filename = absl::StrCat(output_dir_, descriptor->name(), ".m");
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
@@ -489,7 +493,7 @@ void FileGenerator::GenerateMessageSource(GeneratorContext* context,
 
   MessageGenerator generator(descriptor);
   std::set<std::string> headers;
-  headers.insert(output_dir_ + descriptor->name() + ".h");
+  headers.insert(absl::StrCat(output_dir_, descriptor->name(), ".h"));
   generator.CollectSourceImports(&headers);
   AddSourceImports(headers);
   PrintImports(&headers, &printer);
@@ -500,16 +504,17 @@ void FileGenerator::GenerateMessageSource(GeneratorContext* context,
 void FileGenerator::GenerateMessageOrBuilder(
     GeneratorContext* context,
     const Descriptor* descriptor) {
-  std::string filename = output_dir_ + descriptor->name() + "OrBuilder.h";
+  std::string filename = absl::StrCat(output_dir_, descriptor->name(),
+                                      "OrBuilder.h");
   std::unique_ptr<io::ZeroCopyOutputStream> output(context->Open(filename));
   io::Printer printer(output.get(), '$');
 
   GenerateBoilerplate(&printer);
-  MessageGenerator* generator;
+  std::unique_ptr<MessageGenerator> generator;
   if (enforce_lite_) {
-    generator = new MessageLiteGenerator(descriptor);
+    generator = std::make_unique<MessageLiteGenerator>(descriptor);
   } else {
-    generator = new MessageGenerator(descriptor);
+    generator = std::make_unique<MessageGenerator>(descriptor);
   }
 
   std::set<std::string> headers;
