@@ -34,10 +34,12 @@
 
 #include <google/protobuf/compiler/j2objc/j2objc_helpers.h>
 
+#include <cctype>
+#include <cstdint>
 #include <fstream>
 #include <map>
 #include <set>
-#include <sstream>
+#include <string>
 
 namespace google {
 namespace protobuf {
@@ -55,6 +57,7 @@ const char *kOuterClassNameSuffix = "OuterClass";
 // The field number of the "j2objc_package_prefix" file option defined in
 // j2objc-descriptor.proto.
 const int kPackagePrefixFieldNumber = 102687446;
+const int kGeneratePropertiesFieldNumber = 102778330;
 
 // NOLINTBEGIN(runtime/string) - Existing code design requires globals.
 static std::string globalPrefix;
@@ -66,38 +69,86 @@ static std::map<std::string, std::string> wildcardPrefixes;
 
 static bool generateFileDirMapping = false;
 
-const char* const kKeywordList[] = {
-  "TYPE_BOOL",
-  "TRUE",
-  "FALSE",
-  "YES",
-  "NO",
-  "NULL",
-  "FILE"
+// When enabled, this flag will strip out pieces of gencode that's
+// "non-functional" and likely to change in an edition bump.
+static bool stripNonfunctionalCodegen = false;
+
+// A set of names where we don't generate a Swift-friendly property. These are
+// mostly reserved words, but also including some cases where properties
+// collide with pre-existing code (which should be cleaned up on-demand as
+// needed)
+const std::set<absl::string_view> kKeywords = {
+    "DEBUG",
+    "FALSE",
+    "FILE",
+    "NO",
+    "NO_DATA",
+    "NULL",
+    "OVERFLOW",
+    "TRUE",
+    "TYPE_BOOL",
+    "YES",
+    "and",
+    "auto",
+    "break",
+    "case",
+    "class",
+    "const",
+    "continue",
+    "default",
+    "delete",
+    "description",
+    "do",
+    "double",
+    "else",
+    "enum",
+    "for",
+    "float",
+    "hash",
+    "id",
+    "if",
+    "inline",
+    "int",
+    "long",
+    "namespace",
+    "new",
+    "not",
+    "operator",
+    "or",
+    "private",
+    "protected",
+    "public",
+    "return",
+    "signed",
+    "specialCalendars",
+    "static",
+    "struct",
+    "switch",
+    "template",
+    "text",
+    "this",
+    "throw",
+    "try",
+    "union",
+    "unsigned",
+    "virtual",
+    "void",
+    "volatile",
+    "while",
 };
 
-std::set<std::string> MakeKeywordsMap() {
-  std::set<std::string> result;
-  for (int i = 0; i < GOOGLE_ARRAYSIZE(kKeywordList); i++) {
-    result.insert(kKeywordList[i]);
-  }
-  return result;
-}
-
-std::set<std::string> kKeywords = MakeKeywordsMap();
-
-const std::string &FieldName(const FieldDescriptor *field) {
+absl::string_view FieldName(const FieldDescriptor *field) {
   // Groups are hacky:  The name of the field is just the lower-cased name
   // of the group type.  In Java, though, we would like to retain the original
   // capitalization of the type name.
-  if (field->type() == FieldDescriptor::TYPE_GROUP) {
+  if (internal::cpp::IsGroupLike(*field)) {
     return field->message_type()->name();
   } else {
     return field->name();
   }
 }
 
-std::string StripProto(const std::string &filename) {
+std::string StripProto(absl::string_view filename) {
   if (absl::EndsWith(filename, ".protodevel")) {
     return std::string(absl::StripSuffix(filename, ".protodevel"));
   } else {
@@ -123,7 +174,8 @@ std::string CapitalizeJavaPackage(const std::string input) {
   return result;
 }
 
-const UnknownField *FindUnknownField(const FileDescriptor *file, int field_num) {
+const UnknownField *FindUnknownField(const FileDescriptor *file,
+                                     int field_num) {
   const Reflection *reflection = file->options().GetReflection();
   const UnknownFieldSet& unknown_fields =
       reflection->GetUnknownFields(file->options());
@@ -144,7 +196,7 @@ std::string GetPackagePrefix(const FileDescriptor *file) {
   const UnknownField *package_prefix_field =
       FindUnknownField(file, kPackagePrefixFieldNumber);
   if (package_prefix_field) {
-    return globalPrefix + package_prefix_field->length_delimited();
+    return absl::StrCat(globalPrefix, package_prefix_field->length_delimited());
   }
 
   // Look for a matching prefix from the prefixes file.
@@ -199,7 +251,7 @@ std::string GetClassPrefix(const FileDescriptor *file,
   }
 }
 
-} // namespace
+}  // namespace
 
 std::string SafeName(const std::string &name) {
   std::string result = name;
@@ -209,7 +261,7 @@ std::string SafeName(const std::string &name) {
   return result;
 }
 
-std::string UnderscoresToCamelCase(const std::string &input,
+std::string UnderscoresToCamelCase(absl::string_view input,
                                    bool cap_next_letter) {
   std::string result;
   // Note:  I distrust ctype.h due to locales.
@@ -284,13 +336,13 @@ bool HasConflictingClassName(const FileDescriptor *file,
 
 std::string FileParentDir(const FileDescriptor *file) {
   std::string::size_type last_slash = file->name().find_last_of('/');
-  return file->name().substr(0, last_slash + 1);
+  return std::string(file->name().substr(0, last_slash + 1));
 }
 
 std::string FileBaseName(const FileDescriptor *file) {
   std::string::size_type last_slash = file->name().find_last_of('/');
-  return last_slash == std::string::npos ? file->name()
-                                         : file->name().substr(last_slash + 1);
+  return std::string(last_slash == std::string::npos ? file->name()
+                                         : file->name().substr(last_slash + 1));
 }
 
 std::string FileJavaPackage(const FileDescriptor *file) {
@@ -316,13 +368,15 @@ std::string JavaPackageToDir(std::string package_name) {
 }
 
 std::string ClassName(const Descriptor *descriptor) {
-  return GetClassPrefix(descriptor->file(), descriptor->containing_type()) +
-         descriptor->name() + globalPostfix;
+  return absl::StrCat(GetClassPrefix(descriptor->file(),
+                                     descriptor->containing_type()),
+                                     descriptor->name(), globalPostfix);
 }
 
 std::string ClassName(const EnumDescriptor *descriptor) {
-  return GetClassPrefix(descriptor->file(), descriptor->containing_type()) +
-         descriptor->name() + globalPostfix;
+  return absl::StrCat(GetClassPrefix(descriptor->file(),
+                                     descriptor->containing_type()),
+                                     descriptor->name(), globalPostfix);
 }
 
 std::string COrdinalEnumName(const EnumDescriptor *descriptor) {
@@ -347,27 +401,31 @@ std::string ClassName(const FileDescriptor *descriptor) {
 }
 
 std::string EnumOrdinalName(const EnumValueDescriptor *descriptor) {
-  return COrdinalEnumName(descriptor->type()) + "_" + descriptor->name();
+  return absl::StrCat(COrdinalEnumName(descriptor->type()), "_",
+                      descriptor->name());
 }
 
 std::string EnumValueName(const EnumValueDescriptor *descriptor) {
-  return CValueEnumName(descriptor->type()) + "_" + descriptor->name();
+  return absl::StrCat(CValueEnumName(descriptor->type()), "_",
+                      descriptor->name());
 }
 
 std::string FieldConstantName(const FieldDescriptor *field) {
-  std::string name = field->name() + "_FIELD_NUMBER";
+  std::string name = absl::StrCat(field->name(), "_FIELD_NUMBER");
   UpperString(&name);
   return name;
 }
 
 std::string JavaClassName(const Descriptor *descriptor) {
-  return GetJavaClassPrefix(descriptor->file(), descriptor->containing_type())
-      + "." + descriptor->name();
+  return absl::StrCat(GetJavaClassPrefix(descriptor->file(),
+                                         descriptor->containing_type()),
+                                         ".", descriptor->name());
 }
 
 std::string JavaClassName(const EnumDescriptor *descriptor) {
-  return GetJavaClassPrefix(descriptor->file(), descriptor->containing_type())
-      + "." + descriptor->name();
+  return absl::StrCat(GetJavaClassPrefix(descriptor->file(),
+                                         descriptor->containing_type()),
+                                         ".", descriptor->name());
 }
 
 std::string JavaClassName(const FileDescriptor *descriptor) {
@@ -393,8 +451,8 @@ std::string GetHeader(const Descriptor *descriptor) {
       if (IsGenerateFileDirMapping()) {
         return StaticOutputFileName(file, ".h");
       } else {
-        return JavaPackageToDir(FileJavaPackage(file))
-            + descriptor->name() + ".h";
+        return absl::StrCat(JavaPackageToDir(FileJavaPackage(file)),
+            descriptor->name(), ".h");
       }
     }
   } else {
@@ -412,8 +470,8 @@ std::string GetHeader(const EnumDescriptor *descriptor) {
       if (IsGenerateFileDirMapping()) {
         return StaticOutputFileName(file, ".h");
       } else {
-        return JavaPackageToDir(FileJavaPackage(file))
-            + descriptor->name() + ".h";
+        return absl::StrCat(JavaPackageToDir(FileJavaPackage(file)),
+            descriptor->name(), ".h");
       }
     }
   } else {
@@ -455,7 +513,7 @@ std::string GetFieldFlags(const FieldDescriptor *field) {
   return JoinFlags(flags);
 }
 
-static std::string DefaultValueInt(int32 i) {
+static std::string DefaultValueInt(int32_t i) {
   // gcc and llvm reject the decimal form of kint32min and kint64min.
   if (i == INT_MIN) {
     return "-0x80000000";
@@ -463,7 +521,7 @@ static std::string DefaultValueInt(int32 i) {
   return SimpleItoa(i);
 }
 
-static std::string DefaultValueLong(int64 l) {
+static std::string DefaultValueLong(int64_t l) {
   // gcc and llvm reject the decimal form of kint32min and kint64min.
   if (l == LLONG_MIN) {
     return "-0x8000000000000000LL";
@@ -565,7 +623,7 @@ std::string DefaultValue(const FieldDescriptor *field) {
     case FieldDescriptor::CPPTYPE_BOOL:
       return field->default_value_bool() ? "YES" : "NO";
     case FieldDescriptor::CPPTYPE_STRING: {
-      const std::string &default_string = field->default_value_string();
+      const absl::string_view default_string = field->default_value_string();
       if (field->type() == FieldDescriptor::TYPE_BYTES) {
         const bool has_default_value = field->has_default_value();
         if (!has_default_value || default_string.length() == 0) {
@@ -644,6 +702,8 @@ std::string GetDefaultValueTypeName(const FieldDescriptor *descriptor) {
 }
 
 std::string GetFieldOptionsData(const FieldDescriptor *descriptor) {
+  if (stripNonfunctionalCodegen) return "NULL";
+
   std::string field_options = descriptor->options().SerializeAsString();
   // Must convert to a standard byte order for packing length into
   // a cstring.
@@ -656,6 +716,35 @@ std::string GetFieldOptionsData(const FieldDescriptor *descriptor) {
   return "NULL";
 }
 
+std::string PropertyName(const FieldDescriptor *descriptor) {
+  // Use this when -std=c++20 is the minimum version:
+  // !kKeywords.contains(UnderscoresToCamelCase(descriptor));
+  return absl::StrCat(
+      UnderscoresToCamelCase(descriptor),
+      kKeywords.find(UnderscoresToCamelCase(descriptor)) == kKeywords.end()
+          ? ""
+          : "_");
+}
+
+std::string PropertyName(const EnumValueDescriptor *descriptor) {
+  std::string lowercase_name = std::string(descriptor->name());
+  absl::AsciiStrToLower(&lowercase_name);
+  std::string camel_name = UnderscoresToCamelCase(lowercase_name, true);
+  std::string type_name = std::string(descriptor->type()->name());
+  if (absl::StartsWith(camel_name, type_name) &&
+      camel_name.size() > type_name.size()) {
+    camel_name = camel_name.substr(type_name.size());
+  }
+  if (std::isdigit(camel_name[0])) {
+    camel_name = absl::StrCat("_", camel_name);
+  } else {
+    camel_name[0] = absl::ascii_tolower(camel_name[0]);
+  }
+  return absl::StrCat(
+      camel_name,
+      kKeywords.find(camel_name) == kKeywords.end() ? "" : "_");
+}
+
 void ParsePrefixLine(std::string line) {
   std::string::size_type equals = line.find('=');
   if (equals != std::string::npos) {
@@ -683,6 +772,8 @@ void SetFileSubExtension(std::string fileSubExtension) {
   globalFileSubExtension = std::move(fileSubExtension);
 }
 
+void SetStripNonfunctionalCodegen() { stripNonfunctionalCodegen = true; }
+
 void ParsePrefixFile(std::string prefix_file) {
   std::ifstream in(prefix_file.c_str());
   if (in.is_open()) {
@@ -718,6 +809,14 @@ bool IsGenerateFileDirMapping() {
   return generateFileDirMapping;
 }
 
+bool IsGenerateProperties(const FileDescriptor *file) {
+  // Check for the "j2objc_package_prefix" option using unknown fields so we
+  // don't have to pre-build j2objc-descriptor.pb.[h|cc].
+  const UnknownField *generate_properties_field =
+      FindUnknownField(file, kGeneratePropertiesFieldNumber);
+  return generate_properties_field && generate_properties_field->varint();
+}
+
 }  // namespace j2objc
 }  // namespace compiler
 }  // namespace protobuf
