@@ -46,7 +46,7 @@ namespace j2objc {
 
 std::string SafeName(const std::string &name);
 
-std::string UnderscoresToCamelCase(const std::string &input,
+std::string UnderscoresToCamelCase(absl::string_view input,
                                    bool cap_next_letter);
 
 // Converts the field's name to camel-case, e.g. "foo_bar_baz" becomes
@@ -134,9 +134,13 @@ std::string GetFieldTypeEnumValue(const FieldDescriptor *descriptor);
 std::string GetDefaultValueTypeName(const FieldDescriptor *descriptor);
 std::string GetFieldOptionsData(const FieldDescriptor *descriptor);
 
+std::string PropertyName(const FieldDescriptor *descriptor);
+std::string PropertyName(const EnumValueDescriptor *descriptor);
+
 void SetGlobalPrefix(std::string prefix);
 void SetGlobalPostfix(std::string postfix);
 void SetFileSubExtension(std::string fileSubExtension);
+void SetStripNonfunctionalCodegen();
 void ParsePrefixFile(std::string prefix_file);
 
 std::string MappedInputName(const FileDescriptor *file);
@@ -145,6 +149,7 @@ std::string StaticOutputFileName(const FileDescriptor *file,
 std::string FileDirMappingOutputName(const FileDescriptor *file);
 void GenerateFileDirMapping();
 bool IsGenerateFileDirMapping();
+bool IsGenerateProperties(const FileDescriptor *file);
 
 }  // namespace j2objc
 }  // namespace compiler
