@@ -207,10 +207,33 @@ void MessageGenerator::GenerateHeader(io::Printer* printer) {
     superclassName = "ComGoogleProtobufGeneratedMessage_ExtendableMessage";
   }
 
+  if (IsGenerateProperties(descriptor_->file())) {
+    printer->Print(
+        "\n"
+        "@class $classname$;"
+        "\n"
+        "@protocol $classname$_Companion\n"
+        "- (nonnull $classname$_Builder *)newBuilder OBJC_METHOD_FAMILY_NONE;\n"
+        "- (nonnull $classname$ *)getDefaultInstance;\n"
+        "- (nonnull $classname$ *)parseFromWithComGoogleProtobufByteString:"
+        "(nonnull ComGoogleProtobufByteString *)data"
+        " withComGoogleProtobufExtensionRegistryLite:(nullable"
+        " ComGoogleProtobufExtensionRegistryLite *)registry;\n"
+        "@end\n\n"
+        "\n"
+        "// in j2objc_message.cc \n"
+        "@interface $classname$ : $superclassname$<$classname$OrBuilder>\n\n"
+        "@property (nonnull, readonly, class)"
+        " id<$classname$_Companion> companion;\n",
+        "classname", ClassName(descriptor_), "superclassname", superclassName);
+  } else {
+    printer->Print(
+        "\n"
+        "// in j2objc_message.cc \n"
+        "@interface $classname$ : $superclassname$<$classname$OrBuilder>\n\n",
+        "classname", ClassName(descriptor_), "superclassname", superclassName);
+  }
   printer->Print(
-      "\n"
-      "// in j2objc_message.cc \n"
-      "@interface $classname$ : $superclassname$<$classname$OrBuilder>\n\n"
       "+ (nonnull $classname$ *)getDefaultInstance;\n"
       "- (nonnull $classname$ *)getDefaultInstanceForType;\n"
       "+ (nonnull $classname$_Builder *)newBuilder OBJC_METHOD_FAMILY_NONE;\n"
@@ -233,9 +256,10 @@ void MessageGenerator::GenerateHeader(io::Printer* printer) {
       "(JavaIoInputStream *)bytes "
       "withComGoogleProtobufExtensionRegistryLite:"
       "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
-      "+ ($classname$ *)parseFromWithByteString:"
+      "+ ($classname$ *)parseFromWithComGoogleProtobufByteString:"
       "(ComGoogleProtobufByteString *)byteString;\n"
-      "+ (id)parseFromWithByteString:(ComGoogleProtobufByteString *)byteString "
+      "+ (id)parseFromWithComGoogleProtobufByteString:("
+      "ComGoogleProtobufByteString *)byteString "
       "withComGoogleProtobufExtensionRegistryLite:"
       "(ComGoogleProtobufExtensionRegistryLite *)registry;\n"
       "+ ($classname$ *)parseDelimitedFromWithJavaIoInputStream:"
@@ -331,6 +355,15 @@ void MessageGenerator::GenerateHeader(io::Printer* printer) {
     generator.GenerateHeader(printer);
   }
 
+  if (IsGenerateProperties(descriptor_->file())) {
+    printer->Print(
+        "\n"
+        "@compatibility_alias KNP$classname$ $classname$;\n"
+        "@compatibility_alias KNP$classname$_Builder $classname$_Builder;\n"
+        "\n",
+        "classname", ClassName(descriptor_));
+  }
+
   GenerateBuilderHeader(printer);
 }
 
@@ -372,11 +405,11 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
 
   printer->Indent();
   for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
-    OneofGenerator(descriptor_->oneof_decl(i))
+    OneofGenerator(descriptor_->real_oneof_decl(i))
         .GenerateStorageDeclaration(printer);
   }
-  for (int i = 0; i < descriptor_->oneof_decl_count(); i++) {
-    const OneofDescriptor* oneof = descriptor_->oneof_decl(i);
+  for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
+    const OneofDescriptor* oneof = descriptor_->real_oneof_decl(i);
     if (oneof->field_count() == 1) {
       field_generators_.get(oneof->field(0)).GenerateDeclaration(printer);
     } else {
@@ -391,7 +424,7 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
   }
   for (int i = 0; i < descriptor_->field_count(); i++) {
     const FieldDescriptor* field = descriptor_->field(i);
-    if (field->containing_oneof() == nullptr) {
+    if (field->real_containing_oneof() == nullptr) {
       field_generators_.get(field).GenerateDeclaration(printer);
     }
   }
@@ -405,6 +438,19 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
       "}\n",
       "classname", ClassName(descriptor_));
 
+  if (IsGenerateProperties(descriptor_->file())) {
+    printer->Print(
+        "+ (nonnull id<$classname$_Companion>)companion {\n"
+        "  return (id<$classname$_Companion>) [self class];\n"
+        "}\n",
+        "classname", ClassName(descriptor_));
+    for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
+      printer->Print(
+          "@dynamic $camelcase_name$Case;\n", "camelcase_name",
+          UnderscoresToCamelCase(descriptor_->oneof_decl(i)->name(), false));
+    }
+  }
+
   printer->Print(
       "\n"
       "// Minimal metadata for runtime access to Java class name.\n"
@@ -486,6 +532,10 @@ void MessageGenerator::GenerateSource(io::Printer* printer) {
   printer->Outdent();
   printer->Print("  }\n}\n");
 
+  for (int i = 0; i < descriptor_->field_count(); i++) {
+    field_generators_.get(descriptor_->field(i)).GenerateFieldSource(printer);
+  }
+
   printer->Print(
       "\n"
       "@end\n"
@@ -583,9 +633,9 @@ void MessageGenerator::GenerateBuilderHeader(io::Printer* printer) {
       "\n"
       "- (nonnull $classname$ *)getDefaultInstanceForType;\n"
       "- (nonnull $classname$_Builder *)mergeFromWith$classname$:"
-      "($classname$ *)message;\n"
+      "(nonnull $classname$ *)message;\n"
       "- (nonnull $classname$_Builder *)mergeFromWithComGoogleProtobufMessage:"
-      "(id<ComGoogleProtobufMessage>)message;\n"
+      "(nonnull id<ComGoogleProtobufMessage>)message;\n"
       "- (nonnull $classname$ *)build;\n"
       "- (nonnull $classname$ *)buildPartial;\n"
       "+ (nonnull ComGoogleProtobufDescriptors_Descriptor *)getDescriptor;\n",
@@ -618,7 +668,21 @@ void MessageGenerator::GenerateBuilderSource(io::Printer* printer) {
       "@implementation $classname$_Builder\n\n"
       "+ (ComGoogleProtobufDescriptors_Descriptor *)getDescriptor {\n"
       "  return [$classname$ getDescriptor];\n"
-      "}\n"
+      "}\n",
+      "classname", ClassName(descriptor_));
+
+  for (int i = 0; i < descriptor_->field_count(); i++) {
+    field_generators_.get(descriptor_->field(i))
+        .GenerateFieldBuilderSource(printer);
+  }
+  if (IsGenerateProperties(descriptor_->file())) {
+    for (int i = 0; i < descriptor_->real_oneof_decl_count(); i++) {
+      printer->Print(
+          "@dynamic $camelcase_name$Case;\n", "camelcase_name",
+          UnderscoresToCamelCase(descriptor_->oneof_decl(i)->name(), false));
+    }
+  }
+  printer->Print(
       "\n"
       "@end\n"
       "\n"
