@@ -55,8 +55,8 @@ typedef ComGoogleProtobufGeneratedMessage_GeneratedExtension CGPGeneratedExtensi
 @interface ComGoogleProtobufGeneratedMessage : ComGoogleProtobufAbstractMessage
 
 + (id)getDescriptor;
-+ (id)parseFromWithByteString:(ComGoogleProtobufByteString *)byteString;
-+ (id)parseFromWithByteString:(ComGoogleProtobufByteString *)byteString
++ (id)parseFromWithComGoogleProtobufByteString:(ComGoogleProtobufByteString *)byteString;
++ (id)parseFromWithComGoogleProtobufByteString:(ComGoogleProtobufByteString *)byteString
     withComGoogleProtobufExtensionRegistryLite:(ComGoogleProtobufExtensionRegistryLite *)registry;
 + (id)parseFromNSData:(NSData *)data;
 + (id)parseFromNSData:(NSData *)data registry:(ComGoogleProtobufExtensionRegistryLite *)registry;
@@ -91,10 +91,9 @@ J2OBJC_TYPE_LITERAL_HEADER(ComGoogleProtobufGeneratedMessage_Builder)
 @protocol ComGoogleProtobufGeneratedMessage_ExtendableMessageOrBuilder
     <ComGoogleProtobufMessageOrBuilder, JavaObject>
 
-- (jboolean)hasExtensionWithComGoogleProtobufExtensionLite:
-    (ComGoogleProtobufExtensionLite *)extension;
-- (jboolean)hasExtensionWithComGoogleProtobufExtension:(CGPExtension *)extension;
-- (jboolean)hasExtensionWithComGoogleProtobufGeneratedMessage_GeneratedExtension:
+- (bool)hasExtensionWithComGoogleProtobufExtensionLite:(ComGoogleProtobufExtensionLite *)extension;
+- (bool)hasExtensionWithComGoogleProtobufExtension:(CGPExtension *)extension;
+- (bool)hasExtensionWithComGoogleProtobufGeneratedMessage_GeneratedExtension:
     (CGPGeneratedExtension *)extension;
 
 - (id)getExtensionWithComGoogleProtobufExtensionLite:
