@@ -30,6 +30,8 @@ import com.google.protobuf.InvalidProtocolBufferException;
 import com.google.protobuf.Message;
 import com.google.protobuf.MessageLite;
 import com.google.protobuf.ProtocolMessageEnum;
+import com.google.protobuf.ProtocolStringList;
+import com.google.protobuf.TextFormat;
 import foo.bar.baz.PrefixDummy2;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -802,100 +804,115 @@ public class CompatibilityTest extends ProtobufTest {
     }
   }
 
+  // TODO: b/346997541 - Consider moving this test so that it no longer is
+  //                     invoked in a J2ObjC context. Proto builder nullability
+  //                     improvements associated with b/330337638 result in
+  //                     compiler errors if nil values are passed as arguments
+  //                     to proto builder methods in generated Objective-C.
   public void testAddingNullValues() throws Exception {
+    @SuppressWarnings("ModifiedButNotUsed")
     TypicalData.Builder dataBuilder = TypicalData.newBuilder();
+
+    // `nullObject` is used in the below tests along with type casting to ensure
+    // generated Objective-C will successfully compile. Passing in `null`
+    // directly results in the following error: `null passed to a callee that
+    // requires a non-null argument [-Werror,-Wnonnull]`. This error is expected
+    // since proto builders require `nonnull` method arguments. This test is
+    // used in Java and J2ObjC libraries: `test_j2objc_lib` and `test_java_lib`,
+    // these changes ensure it can still run in both contexts.
+    Object nullObject = null;
     try {
-      dataBuilder.setMyMessage((TypicalDataMessage) null);
+      dataBuilder.setMyMessage((TypicalDataMessage) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setMyMessage((TypicalDataMessage.Builder) null);
+      dataBuilder.setMyMessage((TypicalDataMessage.Builder) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setMyString(null);
+      dataBuilder.setMyString((String) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setMyBytes(null);
+      dataBuilder.setMyBytes((ByteString) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setMyEnumType(null);
+      dataBuilder.setMyEnumType((TypicalData.EnumType) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.addRepeatedMessage((TypicalDataMessage) null);
+      dataBuilder.addRepeatedMessage((TypicalDataMessage) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.addRepeatedMessage((TypicalDataMessage.Builder) null);
+      dataBuilder.addRepeatedMessage((TypicalDataMessage.Builder) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setRepeatedMessage(0, (TypicalDataMessage) null);
+      dataBuilder.setRepeatedMessage(0, (TypicalDataMessage) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.addRepeatedString(null);
+      dataBuilder.addRepeatedString((String) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setRepeatedString(0, null);
+      dataBuilder.setRepeatedString(0, (String) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.addRepeatedBytes(null);
+      dataBuilder.addRepeatedBytes((ByteString) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setRepeatedBytes(0, null);
+      dataBuilder.setRepeatedBytes(0, (ByteString) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.addRepeatedEnum(null);
+      dataBuilder.addRepeatedEnum((TypicalData.EnumType) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setRepeatedEnum(0, null);
+      dataBuilder.setRepeatedEnum(0, (TypicalData.EnumType) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.setExtension(Typical.myExtension, null);
+      dataBuilder.setExtension(Typical.myExtension, (TypicalDataMessage) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
     }
     try {
-      dataBuilder.addExtension(Typical.myRepeatedExtension, null);
+      dataBuilder.addExtension(Typical.myRepeatedExtension, (TypicalDataMessage) nullObject);
       fail("Expected NullPointerException");
     } catch (NullPointerException e) {
       // Expected.
@@ -942,7 +959,7 @@ public class CompatibilityTest extends ProtobufTest {
     // Using the Message type to ensure translation of toString compiles on the
     // interface type.
     Message data = TypicalData.newBuilder().setMyInt(31).build();
-    String result = data.toString();
+    String result = TextFormat.printer().printToString(data);
     assertTrue("Unexpected toString result: " + result,
                // Java and ObjC results are not identical.
                result.contains("my_int: 31") || result.contains("myInt: 31"));
@@ -1397,4 +1414,14 @@ public class CompatibilityTest extends ProtobufTest {
     Descriptor descriptor = TypicalData.Builder.getDescriptor();
     assertTrue(descriptor.getFullName().endsWith("protos.TypicalData"));
   }
+
+  public void testProtocolStringListType() throws Exception {
+    TypicalData data = TypicalData.newBuilder().addRepeatedString("foobar").build();
+    ProtocolStringList list = data.getRepeatedStringList();
+    assertEquals(1, list.size());
+    assertTrue(list instanceof ProtocolStringList);
+    assertEquals("foobar", list.get(0));
+    List<ByteString> bsList = list.asByteStringList();
+    assertEquals("foobar", new String(bsList.get(0).toByteArray()));
+  }
 }
