@@ -34,6 +34,7 @@ import javax.lang.model.type.TypeMirror;
  *
  * @author Tom Ball
  */
+@SuppressWarnings("UngroupedOverloads")
 public class DebugASTPrinter extends TreeVisitor {
   protected SourceBuilder sb = new SourceBuilder(false);
   private boolean inIfStatement = false;
@@ -548,6 +549,11 @@ public class DebugASTPrinter extends TreeVisitor {
     node.getLeftOperand().accept(this);
     sb.print(" instanceof ");
     node.getRightOperand().accept(this);
+    Pattern pattern = node.getPattern();
+    if (pattern != null && pattern.getKind() == TreeNode.Kind.BINDING_PATTERN) {
+      sb.print(" ");
+      var unused = visit(((Pattern.BindingPattern) pattern).getVariable());
+    }
     return false;
   }
 
@@ -795,6 +801,28 @@ public class DebugASTPrinter extends TreeVisitor {
     return false;
   }
 
+  @Override
+  public boolean visit(RecordDeclaration node) {
+    sb.printIndent();
+    printAnnotations(node.getAnnotations());
+    printModifiers(node.getModifiers());
+    sb.print("record ");
+    node.getName().accept(this);
+    sb.print(' ');
+    sb.println('{');
+    sb.indent();
+    List<BodyDeclaration> bodyDeclarations = new ArrayList<>(node.getBodyDeclarations());
+    sort(bodyDeclarations);
+    for (BodyDeclaration bodyDecl : bodyDeclarations) {
+      bodyDecl.accept(this);
+    }
+    printStaticBlock(node);
+    sb.unindent();
+    sb.printIndent();
+    sb.println('}');
+    return false;
+  }
+
   @Override
  public boolean visit(ReturnStatement node) {
     sb.printIndent();
@@ -918,16 +946,53 @@ public class DebugASTPrinter extends TreeVisitor {
 
   @Override
   public boolean visit(SwitchCase node) {
-    sb.unindent();
     sb.printIndent();
     if (node.isDefault()) {
-      sb.println("default :");
+      sb.println("default:");
     } else {
       sb.print("case ");
       node.getExpression().accept(this);
       sb.println(":");
     }
-    sb.indent();
+    return false;
+  }
+
+  @Override
+  public boolean visit(SwitchExpressionCase node) {
+    sb.printIndent();
+    if (node.isDefault()) {
+      sb.print("default: ");
+    } else {
+      sb.print("case ");
+      for (Iterator<Expression> it = node.getExpressions().iterator();
+           it.hasNext(); ) {
+        it.next().accept(this);
+        if (it.hasNext()) {
+          sb.print(", ");
+        }
+      }
+      if (node.getPattern() != null) {
+        if (node.getPattern() instanceof Pattern.BindingPattern) {
+          SingleVariableDeclaration varDecl =
+              ((Pattern.BindingPattern) node.getPattern()).getVariable();
+          VariableElement variableElement = varDecl.getVariableElement();
+          sb.print(variableElement.asType().toString());
+          sb.print(" ");
+          sb.print(variableElement.getSimpleName().toString());
+        }
+      }
+      if (node.getGuard() != null) {
+        sb.print(" when ");
+        node.getGuard().accept(this);
+      }
+      sb.print(": ");
+    }
+    TreeNode body = node.getBody();
+    if (body == null) {
+      sb.newline();
+    } else {
+      sb.print(body.toString());
+    }
     return false;
   }
 
@@ -939,14 +1004,28 @@ public class DebugASTPrinter extends TreeVisitor {
     sb.print(") ");
     sb.println("{");
     sb.indent();
-    sb.indent();
     for (Iterator<Statement> it = node.getStatements().iterator(); it.hasNext(); ) {
       it.next().accept(this);
     }
     sb.unindent();
+    sb.printIndent();
+    sb.println("};");
+    return false;
+  }
+
+  @Override
+  public boolean visit(SwitchExpression node) {
+    sb.print("switch (");
+    node.getExpression().accept(this);
+    sb.print(") ");
+    sb.println("{");
+    sb.indent();
+    for (Statement element : node.getStatements()) {
+      element.accept(this);
+    }
     sb.unindent();
     sb.printIndent();
-    sb.println("}");
+    sb.print("}");
     return false;
   }
 
@@ -1021,39 +1100,42 @@ public class DebugASTPrinter extends TreeVisitor {
     printAnnotations(node.getAnnotations());
     printModifiers(node.getModifiers());
     sb.print(node.isInterface() ? "interface " : "class ");
-    node.getName().accept(this);
-    printTypeParameters(node.getTypeElement().getTypeParameters());
-    sb.print(' ');
-    TypeMirror superclassTypeMirror = node.getSuperclassTypeMirror();
-    if (!(TypeUtil.isNone(superclassTypeMirror)
-        || TypeUtil.isJavaObject(superclassTypeMirror))) {
-      sb.print("extends ");
-      sb.print(superclassTypeMirror.toString());
+    if (node.getName() != null) {
+      node.getName().accept(this);
+      printTypeParameters(node.getTypeElement().getTypeParameters());
       sb.print(' ');
-    }
-    List<? extends TypeMirror> superInterfaceTypeMirrors = node.getSuperInterfaceTypeMirrors();
-    if (!superInterfaceTypeMirrors.isEmpty()) {
-      sb.print(node.isInterface() ? "extends " : "implements "); //$NON-NLS-2$
-      for (Iterator<? extends TypeMirror> it = node.getSuperInterfaceTypeMirrors().iterator();
-          it.hasNext(); ) {
-        sb.print(it.next().toString());
-        if (it.hasNext()) {
-          sb.print(", ");
-        }
+      TypeMirror superclassTypeMirror = node.getSuperclassTypeMirror();
+      if (!(TypeUtil.isNone(superclassTypeMirror) || TypeUtil.isJavaObject(superclassTypeMirror))) {
+        sb.print("extends ");
+        sb.print(superclassTypeMirror.toString());
+        sb.print(' ');
       }
-      sb.print(' ');
+      List<? extends TypeMirror> superInterfaceTypeMirrors = node.getSuperInterfaceTypeMirrors();
+      if (!superInterfaceTypeMirrors.isEmpty()) {
+        sb.print(node.isInterface() ? "extends " : "implements "); // $NON-NLS-2$
+        for (Iterator<? extends TypeMirror> it = node.getSuperInterfaceTypeMirrors().iterator();
+            it.hasNext(); ) {
+          sb.print(it.next().toString());
+          if (it.hasNext()) {
+            sb.print(", ");
+          }
+        }
+        sb.print(' ');
+      }
+      sb.println('{');
+      sb.indent();
+      List<BodyDeclaration> bodyDeclarations = new ArrayList<>(node.getBodyDeclarations());
+      sort(bodyDeclarations);
+      for (BodyDeclaration bodyDecl : bodyDeclarations) {
+        bodyDecl.accept(this);
+      }
+      printStaticBlock(node);
+      sb.unindent();
+      sb.printIndent();
+      sb.println('}');
+    } else {
+      sb.println("<uninitialized> {}");
     }
-    sb.println('{');
-    sb.indent();
-    List<BodyDeclaration> bodyDeclarations = new ArrayList<>(node.getBodyDeclarations());
-    sort(bodyDeclarations);
-    for (BodyDeclaration bodyDecl : bodyDeclarations) {
-      bodyDecl.accept(this);
-    }
-    printStaticBlock(node);
-    sb.unindent();
-    sb.printIndent();
-    sb.println('}');
     return false;
   }
 
@@ -1150,6 +1232,15 @@ public class DebugASTPrinter extends TreeVisitor {
     return false;
   }
 
+  @Override
+  public boolean visit(YieldStatement node) {
+    sb.printIndent();
+    sb.print("yield ");
+    node.getExpression().accept(this);
+    sb.println(";");
+    return false;
+  }
+
   protected void printAnnotations(List<Annotation> annotations) {
     Iterator<Annotation> iterator = annotations.iterator();
     while (iterator.hasNext()) {
