@@ -78,6 +78,8 @@ import com.google.devtools.j2objc.ast.SuperFieldAccess;
 import com.google.devtools.j2objc.ast.SuperMethodInvocation;
 import com.google.devtools.j2objc.ast.SuperMethodReference;
 import com.google.devtools.j2objc.ast.SwitchCase;
+import com.google.devtools.j2objc.ast.SwitchExpression;
+import com.google.devtools.j2objc.ast.SwitchExpressionCase;
 import com.google.devtools.j2objc.ast.SwitchStatement;
 import com.google.devtools.j2objc.ast.SynchronizedStatement;
 import com.google.devtools.j2objc.ast.ThisExpression;
@@ -750,6 +752,52 @@ public class StatementGenerator extends UnitTreeVisitor {
     return false;
   }
 
+  @Override
+  @SuppressWarnings("UngroupedOverloads")
+  public boolean visit(SwitchExpression node) {
+    Expression expr = node.getExpression();
+    buffer.append("switch (");
+    expr.accept(this);
+    buffer.append(") ");
+    buffer.append("{\n");
+    buffer.indent();
+    for (Statement stmt : node.getStatements()) {
+      buffer.printIndent();
+      stmt.accept(this);
+    }
+    buffer.unindent();
+    buffer.printIndent();
+    buffer.append("}");
+    return false;
+  }
+
+  @Override
+  @SuppressWarnings("UngroupedOverloads")
+  public boolean visit(SwitchExpressionCase node) {
+    if (node.isDefault()) {
+      buffer.append("default");
+    } else {
+      buffer.append("case ");
+      Iterator<Expression> caseIter = node.getExpressions().iterator();
+      while (caseIter.hasNext()) {
+        caseIter.next().accept(this);
+        if (caseIter.hasNext()) {
+          buffer.println(':');
+          buffer.append("  case ");
+        }
+      }
+    }
+    TreeNode body = node.getBody();
+    if (body != null) {
+      buffer.println(':');
+      buffer.indent();
+      buffer.printIndent();
+      body.accept(this);
+      buffer.unindent();
+    }
+    return false;
+  }
+
   @Override
   public boolean visit(SwitchStatement node) {
     Expression expr = node.getExpression();
