@@ -42,7 +42,6 @@ import java.util.Collections;
 import java.util.Comparator;
 import java.util.Iterator;
 import java.util.List;
-import javax.lang.model.element.Element;
 import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
@@ -129,11 +128,8 @@ public class TypeDeclarationGenerator extends TypeGenerator {
 
     printTypeDocumentation();
     printNonnullAuditedRegion(AuditedRegion.BEGIN);
-    if (typeElement.getKind().isInterface()) {
-      printf("@protocol %s", typeName);
-    } else {
-      printInterfaceType();
-    }
+
+    printInterfaceType();
     printImplementedProtocols();
     if (!typeElement.getKind().isInterface()) {
       printInstanceVariables();
@@ -173,51 +169,66 @@ public class TypeDeclarationGenerator extends TypeGenerator {
     }
 
     List<EnumConstantDeclaration> constants = ((EnumDeclaration) typeNode).getEnumConstants();
-    String nativeName = NameTable.getNativeEnumName(typeName);
+    String nativeName = nameTable.getNativeEnumName(typeElement);
     String ordinalName = NameTable.getNativeOrdinalPreprocessorName(typeName);
 
     // C doesn't allow empty enum declarations.  Java does, so we skip the
     // C enum declaration and generate the type declaration.
     if (!constants.isEmpty()) {
       newline();
-      printf("typedef NS_ENUM(jint, %s) {\n", nativeName);
+      printf("typedef NS_ENUM(int32_t, %s) {\n", nativeName);
 
       // Print C enum typedef.
       indent();
       int ordinal = 0;
       for (EnumConstantDeclaration constant : constants) {
         printIndent();
-        String caseName = nameTable.getVariableBaseName(constant.getVariableElement());
+        String nativeConstantName = nameTable.getNativeEnumConstantName(typeElement, constant);
         if (options.swiftEnums()) {
           printf(
-              "%s_%s NS_SWIFT_NAME(%s) = %d,\n",
-              nativeName, caseName, NameTable.getSwiftEnumName(caseName), ordinal++);
+              "%s NS_SWIFT_NAME(%s) = %d,\n",
+              nativeConstantName, nameTable.getNativeEnumSwiftConstantName(constant), ordinal++);
         } else {
-          printf("%s_%s = %d,\n", nativeName, caseName, ordinal++);
+          printf("%s = %d,\n", nativeConstantName, ordinal++);
         }
       }
       unindent();
-      print("};\n");
-      // Use different types for transpiled Java ordinals (which expects ordinals to be jint) and
+      print("}");
+      String swiftName = nameTable.getSwiftClassNameFromAnnotation(typeElement, true);
+      if (swiftName != null) {
+        printf(" NS_SWIFT_NAME(%s)", swiftName);
+      }
+      print(";\n");
+
+      // For compatibility with JreEnum(), J2OBJC_ENUM_CONSTANT() and similar macros, for each
+      // native renamed enum constant provide an alias if needed.
+      for (EnumConstantDeclaration constant : constants) {
+        String nativeConstantName = nameTable.getNativeEnumConstantName(typeElement, constant);
+        String caseName = nameTable.getVariableBaseName(constant.getVariableElement());
+        String compatibilityConstantName = UnicodeUtils.format("%s_Enum_%s", typeName, caseName);
+        if (!nativeConstantName.equals(compatibilityConstantName)) {
+          printf("#define %s %s\n", compatibilityConstantName, nativeConstantName);
+        }
+      }
+      print("\n");
+
+      // Use different types for transpiled Java ordinals (which expects ordinals to be int32_t) and
       // native code using the enum (where stricter ordinal types help clang warnings).
       printf(
           "#if J2OBJC_IMPORTED_BY_JAVA_IMPLEMENTATION\n"
-              + "#define %s jint\n"
+              + "#define %s int32_t\n"
               + "#else\n"
               + "#define %s %s\n"
               + "#endif\n\n",
           ordinalName, ordinalName, nativeName);
     } else {
-      printf("#define %s jint\n", ordinalName);
+      printf("#define %s int32_t\n", ordinalName);
     }
   }
 
   private void printTypeDocumentation() {
     newline();
     JavadocGenerator.printDocComment(getBuilder(), typeNode.getJavadoc());
-    if (needsDeprecatedAttribute(typeNode.getAnnotations())) {
-      println(DEPRECATED_ATTRIBUTE);
-    }
   }
 
   private String getSuperTypeName() {
@@ -255,9 +266,20 @@ public class TypeDeclarationGenerator extends TypeGenerator {
   }
 
   private void printInterfaceType() {
-    printf("@interface %s", typeName);
-    printInterfaceGenerics();
-    printf(" : %s", getSuperTypeName());
+    if (!typeNode.hasPrivateDeclaration()) {
+      // Swift Names only need to be on public types because private declarations are private
+      printSwiftName();
+    }
+    if (needsDeprecatedAttribute(typeNode.getAnnotations())) {
+      println(DEPRECATED_ATTRIBUTE);
+    }
+    if (typeElement.getKind().isInterface()) {
+      printf("@protocol %s", typeName);
+    } else {
+      printf("@interface %s", typeName);
+      printInterfaceGenerics();
+      printf(" : %s", getSuperTypeName());
+    }
   }
 
   private void printImplementedProtocols() {
@@ -276,6 +298,17 @@ public class TypeDeclarationGenerator extends TypeGenerator {
     }
   }
 
+  private void printSwiftName() {
+    String swiftName = nameTable.getSwiftClassNameFromAnnotation(typeElement, true);
+
+    if (swiftName != null) {
+      if ((typeNode instanceof EnumDeclaration)) {
+        swiftName = swiftName + "Class";
+      }
+      printf(" NS_SWIFT_NAME(%s)\n", swiftName);
+    }
+  }
+
   protected void printStaticInterfaceMethods() {
     for (BodyDeclaration declaration : getInnerDeclarations()) {
       if (declaration.getKind().equals(TreeNode.Kind.METHOD_DECLARATION)) {
@@ -397,7 +430,7 @@ public class TypeDeclarationGenerator extends TypeGenerator {
         }
         String swiftName = accessorName;
         if (options.swiftEnums()) {
-          swiftName = NameTable.getSwiftEnumName(accessorName);
+          swiftName = NameTable.camelCaseName(accessorName, false);
         }
         printf(
             ") %s *%s NS_SWIFT_NAME(%s);",
@@ -636,7 +669,7 @@ public class TypeDeclarationGenerator extends TypeGenerator {
     if (NameTable.isReservedName(propertyName)) {
       return false;
     }
-    // Check if there is a exisitng property with the same name
+    // Check if there is a existing property with the same name
     if (ElementUtil.findField(declaringClass, propertyName) != null) {
       return false;
     }
@@ -653,7 +686,7 @@ public class TypeDeclarationGenerator extends TypeGenerator {
     if (NameTable.isReservedName(propertyName)) {
       return;
     }
-    // Check if there is a exisitng property with the same name
+    // Check if there is a existing property with the same name
     if (ElementUtil.findField(declaringClass, propertyName) != null) {
       return;
     }
@@ -683,6 +716,7 @@ public class TypeDeclarationGenerator extends TypeGenerator {
   private void printMethodDeclaration(MethodDeclaration m, boolean isCompanionClass) {
     ExecutableElement methodElement = m.getExecutableElement();
     TypeElement typeElement = ElementUtil.getDeclaringClass(methodElement);
+    boolean allowGenerics = !typeUtil.isProtoClass(typeElement.asType());
 
     if (typeElement.getKind().isInterface()) {
       // isCompanion and isStatic must be both false (i.e. this prints a non-static method decl
@@ -702,7 +736,7 @@ public class TypeDeclarationGenerator extends TypeGenerator {
     }
 
     // Method declarations allow generics.
-    String methodSignature = getMethodSignature(m, true);
+    String methodSignature = getMethodSignature(m, allowGenerics);
 
     // In order to properly map the method name from the entire signature, we must isolate it from
     // associated type and parameter declarations.  The method name is guaranteed to be between the
@@ -731,12 +765,20 @@ public class TypeDeclarationGenerator extends TypeGenerator {
       print(" OBJC_METHOD_FAMILY_NONE");
     }
 
+    String swiftName = nameTable.getSwiftMethodNameFromAnnotation(m);
+    if (swiftName != null) {
+      print(" NS_SWIFT_NAME(" + swiftName + ")");
+    }
+
     if (needsDeprecatedAttribute(m.getAnnotations())) {
       print(" " + DEPRECATED_ATTRIBUTE);
     }
     if (m.isUnavailable()) {
       print(" NS_UNAVAILABLE");
     }
+    if (options.addTextSegmentAttribute()) {
+      print(" J2OBJC_TEXT_SEGMENT");
+    }
     println(";");
   }
 
@@ -773,10 +815,13 @@ public class TypeDeclarationGenerator extends TypeGenerator {
 
   @Override
   protected void printFunctionDeclaration(FunctionDeclaration function) {
-    print("\nFOUNDATION_EXPORT " + getFunctionSignature(function, true));
+    print("\nFOUNDATION_EXPORT " + getFunctionSignature(function, true, false));
     if (function.returnsRetained()) {
       print(" NS_RETURNS_RETAINED");
     }
+    if (options.addTextSegmentAttribute()) {
+      print(" J2OBJC_TEXT_SEGMENT");
+    }
     println(";");
   }
 
@@ -858,23 +903,6 @@ public class TypeDeclarationGenerator extends TypeGenerator {
     }
   }
 
-  /**
-   * Returns an Objective-C nullability attribute string if there is a matching JSR305 annotation,
-   * or an empty string.
-   */
-  @Override
-  protected String nullability(Element element) {
-    if (options.nullability()) {
-      if (ElementUtil.hasNullableAnnotation(element)) {
-        return " _Nullable";
-      }
-      if (ElementUtil.isNonnull(element, parametersNonnullByDefault)) {
-        return " _Nonnull";
-      }
-    }
-    return "";
-  }
-
   /**
    * Method comparator, suitable for documentation and code-completion lists.
    *
