@@ -47,6 +47,7 @@ import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeMirror;
+import javax.lang.model.type.TypeVariable;
 
 /**
  * The base class for TypeDeclarationGenerator and TypeImplementationGenerator,
@@ -295,9 +296,11 @@ public abstract class TypeGenerator extends AbstractSourceGenerator {
 
   protected String getReturnType(MethodDeclaration m, boolean generatorAllowsGenerics) {
     ExecutableElement element = m.getExecutableElement();
+    boolean allowGenerics =
+        !typeUtil.isProtoClass(element.getReturnType()) && generatorAllowsGenerics;
     return nameTable.getObjCTypeDeclaration(
         element.getReturnType(),
-        generatorAllowsGenerics
+        allowGenerics
             && (generateObjectiveCGenerics(element.getReturnType())
                 || generateObjectiveCGenerics(typeElement.asType())),
         typeElement);
@@ -332,7 +335,19 @@ public abstract class TypeGenerator extends AbstractSourceGenerator {
       // Explicitly test hashCode() because of NSObject's hash return value.
       returnType = "NSUInteger";
     }
-    sb.append(UnicodeUtils.format("%c (%s%s)", prefix, returnType, nullability(element)));
+
+    String returnTypeNullability = "";
+    TypeMirror returnTypeMirror = m.getReturnTypeMirror();
+    if (TypeUtil.isTypeVariable(returnTypeMirror)) {
+      returnTypeNullability = nullability(((TypeVariable) returnTypeMirror).asElement());
+    }
+    String methodReturnNullability = nullability(element);
+    String returnNullability =
+        returnTypeNullability.isEmpty() ? methodReturnNullability : returnTypeNullability;
+    sb.append(
+        UnicodeUtils.format(
+            "%c (%s%s%s)",
+            prefix, returnType, returnNullability.isEmpty() ? "" : " ", returnNullability));
 
     List<SingleVariableDeclaration> params = m.getParameters();
     String[] selParts = selector.split(":");
@@ -349,32 +364,55 @@ public abstract class TypeGenerator extends AbstractSourceGenerator {
           sb.append(pad(baseLength - selParts[i].length()));
         }
         VariableElement var = params.get(i).getVariableElement();
+        boolean allowGenerics = !typeUtil.isProtoClass(var.asType()) && generatorAllowsGenerics;
         String typeName =
             nameTable.getObjCTypeDeclaration(
                 var.asType(),
-                generatorAllowsGenerics
+                allowGenerics
                     && (generateObjectiveCGenerics(var.asType())
                         || generateObjectiveCGenerics(typeElement.asType())),
                 typeElement);
+        String varNullability = nullability(var);
         sb.append(
             UnicodeUtils.format(
-                "%s:(%s%s)%s",
-                selParts[i], typeName, nullability(var), nameTable.getVariableShortName(var)));
+                "%s:(%s%s%s)%s",
+                selParts[i],
+                typeName,
+                varNullability.isEmpty() ? "" : " ",
+                varNullability,
+                nameTable.getVariableShortName(var)));
       }
     }
 
     return sb.toString();
   }
 
-  /** Returns an Objective-C nullability attribute string if needed. */
-  protected abstract String nullability(Element element);
+  /**
+   * Returns an Objective-C nullability attribute string without padding if there is a matching
+   * JSR305 annotation, or an empty string.
+   */
+  protected String nullability(Element element) {
+    if (element != null) {
+      if (options.nullability()) {
+        if (ElementUtil.hasNullableAnnotation(element)) {
+          return "_Nullable";
+        }
+        if (ElementUtil.isNonnull(element, parametersNonnullByDefault) && !nullMarked) {
+          return "_Nonnull";
+        }
+      }
+    }
+    return "";
+  }
 
-  protected String getFunctionSignature(FunctionDeclaration function, boolean isPrototype) {
+  protected String getFunctionSignature(
+      FunctionDeclaration function, boolean isPrototype, boolean isPrivate) {
     StringBuilder sb = new StringBuilder();
     TypeMirror returnTypeMirror = function.getReturnType().getTypeMirror();
     String returnType =
         paddedType(nameTable.getObjCType(returnTypeMirror), function.getExecutableElement());
-    sb.append(returnType).append(function.getName()).append('(');
+    String functionName = function.getName();
+    sb.append(returnType).append(functionName).append('(');
     if (isPrototype && function.getParameters().isEmpty()) {
       sb.append("void");
     } else {
@@ -389,6 +427,14 @@ public abstract class TypeGenerator extends AbstractSourceGenerator {
       }
     }
     sb.append(')');
+
+    if (!isPrivate) {
+      String swiftName = nameTable.getSwiftFunctionNameFromAnnotation(function);
+      if (swiftName != null) {
+        sb.append(swiftName);
+      }
+    }
+
     return sb.toString();
   }
 
@@ -407,7 +453,18 @@ public abstract class TypeGenerator extends AbstractSourceGenerator {
   protected String paddedType(String type, Element element) {
     String suffix = " ";
     if (type.endsWith("*")) {
-      suffix = shouldAddNullableAnnotation(element) ? "_Nullable " : "";
+      // shouldAddNullableAnnotation() is partially redundant to checks inside nullability()
+      String nullabilitySuffix = shouldAddNullableAnnotation(element) ? nullability(element) : "";
+      if (nullabilitySuffix.isEmpty()) {
+        suffix = "";
+      } else {
+        suffix = nullabilitySuffix + " ";
+      }
+    } else if (type.contentEquals("id") || type.startsWith("id<")) {
+      // shouldAddNullableAnnotation() is partially redundant to checks inside nullability()
+      if (shouldAddNullableAnnotation(element)) {
+        suffix = " " + nullability(element) + " ";
+      }
     }
     return type + suffix;
   }
