@@ -127,13 +127,19 @@ public class TypePrivateDeclarationGenerator extends TypeDeclarationGenerator {
     if (!Modifier.isNative(function.getModifiers())) {
       print("__attribute__((unused)) static ");
     }
-    print(getFunctionSignature(function, true));
+    print(getFunctionSignature(function, true, true));
     if (function.returnsRetained()) {
       print(" NS_RETURNS_RETAINED");
     }
     println(";");
   }
 
+  @Override
+  protected void printNonnullAuditedRegion(AuditedRegion state) {
+    // Private declarations do not include nullability for the same reasons
+    // implementation files do not.
+  }
+
   @Override
   protected String nullability(Element element) {
     return "";
