@@ -15,11 +15,13 @@
 package com.google.devtools.j2objc.pipeline;
 
 import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Joiner;
 import com.google.devtools.j2objc.ast.CompilationUnit;
 import com.google.devtools.j2objc.ast.DebugASTDump;
 import com.google.devtools.j2objc.gen.GenerationUnit;
 import com.google.devtools.j2objc.gen.ObjectiveCHeaderGenerator;
 import com.google.devtools.j2objc.gen.ObjectiveCImplementationGenerator;
+import com.google.devtools.j2objc.gen.ObjectiveCMultiHeaderGenerator;
 import com.google.devtools.j2objc.gen.ObjectiveCSegmentedHeaderGenerator;
 import com.google.devtools.j2objc.translate.AbstractMethodRewriter;
 import com.google.devtools.j2objc.translate.AnnotationRewriter;
@@ -47,16 +49,20 @@ import com.google.devtools.j2objc.translate.LogSiteInjector;
 import com.google.devtools.j2objc.translate.MetadataWriter;
 import com.google.devtools.j2objc.translate.NilCheckResolver;
 import com.google.devtools.j2objc.translate.NumberMethodRewriter;
+import com.google.devtools.j2objc.translate.ObjectiveCAdapterMethodAnnotation;
+import com.google.devtools.j2objc.translate.ObjectiveCNativeProtocolAnnotation;
 import com.google.devtools.j2objc.translate.OcniExtractor;
 import com.google.devtools.j2objc.translate.OperatorRewriter;
 import com.google.devtools.j2objc.translate.OuterReferenceResolver;
 import com.google.devtools.j2objc.translate.PackageInfoRewriter;
 import com.google.devtools.j2objc.translate.PrivateDeclarationResolver;
+import com.google.devtools.j2objc.translate.RecordExpander;
 import com.google.devtools.j2objc.translate.ReflectionCodeDetector;
 import com.google.devtools.j2objc.translate.Rewriter;
 import com.google.devtools.j2objc.translate.SerializationStripper;
 import com.google.devtools.j2objc.translate.StaticVarRewriter;
 import com.google.devtools.j2objc.translate.SuperMethodInvocationRewriter;
+import com.google.devtools.j2objc.translate.SwitchCaseRewriter;
 import com.google.devtools.j2objc.translate.SwitchRewriter;
 import com.google.devtools.j2objc.translate.UnsequencedExpressionRewriter;
 import com.google.devtools.j2objc.translate.VarargsRewriter;
@@ -70,7 +76,10 @@ import com.google.devtools.j2objc.util.ExternalAnnotations;
 import com.google.devtools.j2objc.util.Parser;
 import com.google.devtools.j2objc.util.TimeTracker;
 import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 import java.util.logging.Level;
 import java.util.logging.Logger;
@@ -123,9 +132,11 @@ public class TranslationProcessor extends FileProcessor {
 
   @Override
   protected void processOutputs(Iterable<ProcessingContext> outputs) {
+    HashMap<String, Set<String>> headerIncludesMap = new HashMap<>();
     for (ProcessingContext output : outputs) {
-      generateObjectiveCSource(output.getGenerationUnit());
+      generateObjectiveCSource(output.getGenerationUnit(), headerIncludesMap);
     }
+    checkNoIncludeCycles(headerIncludesMap);
   }
 
   /**
@@ -169,6 +180,9 @@ public class TranslationProcessor extends FileProcessor {
     new OuterReferenceResolver(unit).run();
     ticker.tick("OuterReferenceResolver");
 
+    new RecordExpander(unit).run();
+    ticker.tick("RecordExpander");
+
     // Update code that has GWT references.
     new GwtConverter(unit).run();
     ticker.tick("GwtConverter");
@@ -318,11 +332,27 @@ public class TranslationProcessor extends FileProcessor {
     new CastResolver(unit).run();
     ticker.tick("CastResolver");
 
+    // After: SwitchRewriter to handle Java 21 patterns and guards in switches,
+    // and CastResolver to avoid duplicate cast checks.
+    new SwitchCaseRewriter(unit).run();
+    ticker.tick("SwitchCaseRewriter");
+
     // After: InnerClassExtractor, Functionizer - Expects all types to be
     //   top-level and functionizing to have occurred.
     new PrivateDeclarationResolver(unit).run();
     ticker.tick("PrivateDeclarationResolver");
 
+    // Add native protocols after all prior translation. Occurs before
+    // adapter methods that may reference those protocols.
+    new ObjectiveCNativeProtocolAnnotation(unit).run();
+    ticker.tick("ObjectiveCNativeProtocolAnnotation");
+
+    // After all methods are resolved and functionized, add adapter methods to
+    // use their native types as annotated. Done last as the generated methods
+    // do not need other processing above.
+    new ObjectiveCAdapterMethodAnnotation(unit).run();
+    ticker.tick("ObjectiveCAdapterMethodAnnotation");
+
     if (deadCodeMap != null) {
       deadCodeEliminator.removeDeadClasses();
       ticker.tick("removeDeadClasses");
@@ -335,7 +365,8 @@ public class TranslationProcessor extends FileProcessor {
   }
 
   @VisibleForTesting
-  public static void generateObjectiveCSource(GenerationUnit unit) {
+  public static void generateObjectiveCSource(
+      GenerationUnit unit, Map<String, Set<String>> headerIncludesMap) {
     assert unit.getOutputPath() != null;
     assert unit.isFullyParsed();
     TimeTracker ticker = TimeTracker.getTicker(unit.getSourceName(), unit.options().timingLevel());
@@ -346,11 +377,15 @@ public class TranslationProcessor extends FileProcessor {
         + unit.options().fileUtil().getHeaderOutputDirectory().getAbsolutePath());
     ticker.push();
 
-    // write header
-    if (unit.options().generateSegmentedHeaders()) {
+    // write header(s)
+    if (unit.options().generateSeparateHeaders()) {
+      ObjectiveCMultiHeaderGenerator.generate(unit);
+    } else if (unit.options().generateSegmentedHeaders()) {
       ObjectiveCSegmentedHeaderGenerator.generate(unit);
     } else {
-      ObjectiveCHeaderGenerator.generate(unit);
+      // Only need to populate headerIncludesMap in this case, since segmented or separate headers
+      // cannot produce include cycles.
+      ObjectiveCHeaderGenerator.generate(unit, headerIncludesMap);
     }
     ticker.tick("Header generation");
 
@@ -396,4 +431,63 @@ public class TranslationProcessor extends FileProcessor {
       }
     }
   }
+
+  @VisibleForTesting
+  static void checkNoIncludeCycles(Map<String, Set<String>> headerIncludesMap) {
+    List<String> cycle = findIncludeCycle(headerIncludesMap);
+    if (!cycle.isEmpty()) {
+      ErrorUtil.error(
+          "This target contains an include cycle, but segmented headers are disabled. Enable"
+              + " segmented headers and try again. Cycle:\n"
+              + Joiner.on("\n").join(cycle));
+    }
+  }
+
+  /**
+   * Looks for an include cycle in the map. If a cycle is found, the path argument is populated with
+   * the path that found the cycle (in reverse order).
+   */
+  private static List<String> findIncludeCycle(Map<String, Set<String>> headerIncludesMap) {
+    HashSet<String> finished = new HashSet<>();
+    HashSet<String> visited = new HashSet<>();
+    ArrayList<String> path = new ArrayList<>();
+    for (String header : headerIncludesMap.keySet()) {
+      if (findIncludeCycle(headerIncludesMap, header, finished, visited, path)) {
+        String cycleHead = path.get(0);
+        return path.subList(0, path.lastIndexOf(cycleHead) + 1);
+      }
+    }
+    return new ArrayList<>();
+  }
+
+  /**
+   * Looks for a cycle in the map, starting at currentPath. If a cycle is found, the path argument
+   * is populated with the path that found the cycle (in reverse order).
+   */
+  private static boolean findIncludeCycle(
+      Map<String, Set<String>> headerIncludesMap,
+      String currentPath,
+      Set<String> finished,
+      Set<String> visited,
+      List<String> path) {
+    if (finished.contains(currentPath)) {
+      return false;
+    }
+    if (visited.contains(currentPath)) {
+      path.add(currentPath);
+      return true;
+    }
+    visited.add(currentPath);
+    Set<String> includes = headerIncludesMap.get(currentPath);
+    if (includes != null) {
+      for (String include : includes) {
+        if (findIncludeCycle(headerIncludesMap, include, finished, visited, path)) {
+          path.add(currentPath);
+          return true;
+        }
+      }
+    }
+    finished.add(currentPath);
+    return false;
+  }
 }
