@@ -40,6 +40,7 @@ import com.google.devtools.j2objc.ast.ReturnStatement;
 import com.google.devtools.j2objc.ast.SimpleName;
 import com.google.devtools.j2objc.ast.SuperConstructorInvocation;
 import com.google.devtools.j2objc.ast.SuperMethodInvocation;
+import com.google.devtools.j2objc.ast.SwitchExpression;
 import com.google.devtools.j2objc.ast.SwitchStatement;
 import com.google.devtools.j2objc.ast.TreeNode;
 import com.google.devtools.j2objc.ast.TreeUtil;
@@ -131,6 +132,11 @@ public class Autoboxer extends UnitTreeVisitor {
     }
     ExecutableElement valueMethod = ElementUtil.findMethod(
         boxedClass, TypeUtil.getName(primitiveType) + VALUE_METHOD);
+    if (valueMethod == null) {
+      primitiveType = typeUtil.unboxedType(boxedClass.asType());
+      valueMethod = ElementUtil.findMethod(
+          boxedClass, TypeUtil.getName(primitiveType) + VALUE_METHOD);
+    }
     assert valueMethod != null : "could not find value method for " + boxedClass;
     MethodInvocation invocation = new MethodInvocation(new ExecutablePair(valueMethod), null);
     expr.replaceWith(invocation);
@@ -463,6 +469,15 @@ public class Autoboxer extends UnitTreeVisitor {
     }
   }
 
+  @Override
+  @SuppressWarnings("UngroupedOverloads")
+  public void endVisit(SwitchExpression node) {
+    Expression expression = node.getExpression();
+    if (!expression.getTypeMirror().getKind().isPrimitive()) {
+      unbox(expression);
+    }
+  }
+
   @Override
   public void endVisit(SwitchStatement node) {
     Expression expression = node.getExpression();
