@@ -23,6 +23,7 @@ import com.google.devtools.j2objc.ast.FieldDeclaration;
 import com.google.devtools.j2objc.ast.MethodDeclaration;
 import com.google.devtools.j2objc.ast.NormalAnnotation;
 import com.google.devtools.j2objc.ast.PackageDeclaration;
+import com.google.devtools.j2objc.ast.RecordDeclaration;
 import com.google.devtools.j2objc.ast.SimpleName;
 import com.google.devtools.j2objc.ast.TypeDeclaration;
 import com.google.devtools.j2objc.ast.UnitTreeVisitor;
@@ -105,6 +106,11 @@ public final class ExternalAnnotationInjector extends UnitTreeVisitor {
     return visitAbstractTypeDeclaration(node);
   }
 
+  @Override
+  public boolean visit(RecordDeclaration node) {
+    return visitAbstractTypeDeclaration(node);
+  }
+
   @Override
   public boolean visit(MethodDeclaration node) {
     if (!annotatedElementStack.peekLast().isPresent()) {
@@ -154,6 +160,11 @@ public final class ExternalAnnotationInjector extends UnitTreeVisitor {
     endVisitAbstractTypeDeclaration();
   }
 
+  @Override
+  public void endVisit(RecordDeclaration node) {
+    endVisitAbstractTypeDeclaration();
+  }
+
   private boolean visitAbstractTypeDeclaration(AbstractTypeDeclaration node) {
     String elementName = elementUtil.getBinaryName(node.getTypeElement());
     AClass annotatedElement = annotatedAst.classes.get(elementName);
