@@ -44,11 +44,13 @@ import com.google.devtools.j2objc.ast.MethodDeclaration;
 import com.google.devtools.j2objc.ast.MethodInvocation;
 import com.google.devtools.j2objc.ast.NullLiteral;
 import com.google.devtools.j2objc.ast.ParenthesizedExpression;
+import com.google.devtools.j2objc.ast.RecordDeclaration;
 import com.google.devtools.j2objc.ast.ReturnStatement;
 import com.google.devtools.j2objc.ast.Statement;
 import com.google.devtools.j2objc.ast.SuperConstructorInvocation;
 import com.google.devtools.j2objc.ast.SuperMethodInvocation;
 import com.google.devtools.j2objc.ast.SwitchCase;
+import com.google.devtools.j2objc.ast.SwitchExpression;
 import com.google.devtools.j2objc.ast.SwitchStatement;
 import com.google.devtools.j2objc.ast.ThrowStatement;
 import com.google.devtools.j2objc.ast.TreeNode;
@@ -535,6 +537,11 @@ public class NilCheckResolver extends UnitTreeVisitor {
     return visitType(node);
   }
 
+  @Override
+  public boolean visit(RecordDeclaration node) {
+    return visitType(node);
+  }
+
   @Override
   public boolean visit(TypeDeclaration node) {
     return visitType(node);
@@ -708,6 +715,20 @@ public class NilCheckResolver extends UnitTreeVisitor {
     return false;
   }
 
+  @Override
+  @SuppressWarnings("UngroupedOverloads")
+  public boolean visit(SwitchExpression node) {
+    // No need to test for no matched value like SwitchStatement,
+    // as switch expressions are exhaustive.
+    node.getExpression().accept(this);
+    pushLoopOrSwitchScope(null);
+    for (Statement stmt : node.getStatements()) {
+      stmt.accept(this);
+    }
+    popAndMerge();
+    return false;
+  }
+
   @Override
   public boolean visit(SwitchStatement node) {
     node.getExpression().accept(this);
