@@ -79,6 +79,7 @@ public class OperatorRewriter extends UnitTreeVisitor {
   private final LinkedList<Set<VariableElement>> retainedLocalCandidateStack = new LinkedList<>();
   private Set<VariableElement> retainedLocalCandidates = new HashSet<>();
   private boolean maybeRetainMethodReturn = false;
+  private boolean methodIsConstructorOrDestructor = false;
 
   public OperatorRewriter(CompilationUnit unit) {
     super(unit);
@@ -146,6 +147,7 @@ public class OperatorRewriter extends UnitTreeVisitor {
   @Override
   public boolean visit(MethodDeclaration node) {
     ExecutableElement method = node.getExecutableElement();
+    methodIsConstructorOrDestructor = isConstructorOrDestructor(method);
     maybeRetainMethodReturn = Modifier.isSynchronized(node.getModifiers())
         || maybeRetainLambdaOrAnonymousClassMethod(method);
     retainedLocalCandidates.addAll(
@@ -156,16 +158,20 @@ public class OperatorRewriter extends UnitTreeVisitor {
     return true;
   }
 
-  // Returns true if the method is part of a lambda or anonymous class, isn't a constructor
-  // or destructor, and returns a retainable type.
+  // Returns true if the method is a constructor or destructor
+  private boolean isConstructorOrDestructor(ExecutableElement method) {
+    String methodName = ElementUtil.getName(method);
+    return ElementUtil.isConstructor(method) || methodName.equals("dealloc");
+  }
+
+  // Returns true if the method isn't a constructor or destructor, and returns a retainable type.
   private boolean maybeRetainLambdaOrAnonymousClassMethod(ExecutableElement method) {
     TypeElement declaringClass = ElementUtil.getDeclaringClass(method);
     if (!ElementUtil.isAnonymous(declaringClass) && !ElementUtil.isLambda(declaringClass)) {
       return false;
     }
     String methodName = ElementUtil.getName(method);
-    if (ElementUtil.isConstructor(method)
-        || methodName.equals("dealloc")
+    if (isConstructorOrDestructor(method)
         || methodName.startsWith("__")) { // True for translator-generated internal methods.
       return false;
     }
@@ -177,6 +183,7 @@ public class OperatorRewriter extends UnitTreeVisitor {
     retainedLocalCandidateStack.clear();
     retainedLocalCandidates.clear();
     maybeRetainMethodReturn = false;
+    methodIsConstructorOrDestructor = false;
   }
 
   @Override
@@ -194,9 +201,11 @@ public class OperatorRewriter extends UnitTreeVisitor {
   @Override
   public void endVisit(ReturnStatement node) {
     Expression expr = node.getExpression();
-    if ((maybeRetainMethodReturn || !retainedLocalCandidateStack.isEmpty())
-        && expr != null
-        && !expr.getTypeMirror().getKind().isPrimitive()) {
+    boolean isRetainableReturnType = expr != null && TypeUtil.isReferenceType(expr.getTypeMirror());
+    if (options.useRetainAutoreleaseReturns() && isRetainableReturnType) {
+      rewriteRetainedAutoreleaseReturnValue(expr);
+    } else if ((maybeRetainMethodReturn || !retainedLocalCandidateStack.isEmpty())
+        && isRetainableReturnType) {
       rewriteRetainedLocal(expr);
     }
   }
@@ -204,6 +213,7 @@ public class OperatorRewriter extends UnitTreeVisitor {
   @Override
   public boolean visit(FieldAccess node) {
     rewriteVolatileLoad(node);
+    rewriteStrictFieldLoad(node);
     node.getExpression().accept(this);
     return false;
   }
@@ -211,18 +221,21 @@ public class OperatorRewriter extends UnitTreeVisitor {
   @Override
   public boolean visit(SuperFieldAccess node) {
     rewriteVolatileLoad(node);
+    rewriteStrictFieldLoad(node);
     return false;
   }
 
   @Override
   public boolean visit(QualifiedName node) {
     rewriteVolatileLoad(node);
+    rewriteStrictFieldLoad(node);
     return false;
   }
 
   @Override
   public boolean visit(SimpleName node) {
     rewriteVolatileLoad(node);
+    rewriteStrictFieldLoad(node);
     return false;
   }
 
@@ -287,7 +300,8 @@ public class OperatorRewriter extends UnitTreeVisitor {
   }
 
   private void rewriteRetainedLocal(Expression expr) {
-    if (expr.getKind() == TreeNode.Kind.STRING_LITERAL
+    if (options.useARC()  // ARC generates an equivalent function call.
+        || expr.getKind() == TreeNode.Kind.STRING_LITERAL
         || expr.getKind() == TreeNode.Kind.FUNCTION_INVOCATION) {
       return;
     }
@@ -304,11 +318,34 @@ public class OperatorRewriter extends UnitTreeVisitor {
     }
   }
 
+  private void rewriteRetainedAutoreleaseReturnValue(Expression expr) {
+    if (options.useARC() // ARC generates equivalent code.
+        // Literals do not need retain.
+        || expr.getKind() == TreeNode.Kind.STRING_LITERAL
+        // Methods and functions allow innermost call to handle retain/autorelease.
+        || expr.getKind() == TreeNode.Kind.FUNCTION_INVOCATION
+        || expr.getKind() == TreeNode.Kind.METHOD_INVOCATION
+        // Constructor methods do not need additional retain/autorelease.
+        || methodIsConstructorOrDestructor) {
+      return;
+    }
+    String funcName =
+        options.useARCAutoreleaseReturns()
+            ? "objc_retainAutoreleaseReturnValue"
+            : "JreRetainedAutoreleasedReturnValue";
+    FunctionElement element = new FunctionElement(funcName, TypeUtil.ID_TYPE, null);
+    FunctionInvocation invocation = new FunctionInvocation(element, expr.getTypeMirror());
+    expr.replaceWith(invocation);
+    invocation.addArgument(expr);
+  }
+
   private void rewriteVolatileLoad(Expression node) {
     VariableElement var = TreeUtil.getVariableElement(node);
     if (var != null && ElementUtil.isVolatile(var) && !TranslationUtil.isAssigned(node)) {
       TypeMirror type = node.getTypeMirror();
-      TypeMirror declaredType = type.getKind().isPrimitive() ? type : TypeUtil.ID_TYPE;
+      TypeMirror declaredType = type.getKind().isPrimitive()
+          ? typeUtil.getPrimitiveType(type.getKind())
+          : TypeUtil.ID_TYPE;
       String funcName = "JreLoadVolatile" + NameTable.capitalize(declaredType.toString());
       FunctionElement element = new FunctionElement(funcName, declaredType, null)
           .addParameters(TypeUtil.ID_PTR_TYPE);
@@ -319,6 +356,36 @@ public class OperatorRewriter extends UnitTreeVisitor {
     }
   }
 
+  private void rewriteStrictFieldLoad(Expression node) {
+    if (!options.useStrictFieldLoad()) {
+      return;
+    }
+    VariableElement var = TreeUtil.getVariableElement(node);
+    if (var != null && ElementUtil.isField(var) && !TranslationUtil.isAssigned(node)) {
+      TypeMirror type = node.getTypeMirror();
+      if (!type.getKind().isPrimitive() && !ElementUtil.isWeakReference(var)) {
+
+        // Ignore direct access to IOSObjectArray buffers generated by EnhancedForRewriter.
+        if (node.getKind() == TreeNode.Kind.FIELD_ACCESS) {
+          Expression fieldParent = ((FieldAccess) node).getExpression();
+          TypeMirror parentType = fieldParent.getTypeMirror();
+          if (TypeUtil.isArray(parentType) && var.getSimpleName().toString().equals("buffer")) {
+            return;
+          }
+        }
+
+        FunctionElement element =
+            new FunctionElement("JreStrictFieldStrongLoad", TypeUtil.ID_TYPE, null)
+                .addParameters(TypeUtil.ID_PTR_TYPE);
+        FunctionInvocation invocation = new FunctionInvocation(element, type);
+        node.replaceWith(invocation);
+        invocation.addArgument(
+            new PrefixExpression(
+                new PointerType(type), PrefixExpression.Operator.ADDRESS_OF, node));
+      }
+    }
+  }
+
   private String getAssignmentFunctionName(
       Assignment node, VariableElement var, boolean isRetainedWith) {
     if (!ElementUtil.isField(var)) {
@@ -341,6 +408,15 @@ public class OperatorRewriter extends UnitTreeVisitor {
           + (isPrimitive ? NameTable.capitalize(TypeUtil.getName(type)) : "Id");
     }
 
+    // Prefer strict fields over ARC (strict fields are compatible with both ARC and manual
+    // reference counting).
+    if (isStrong && options.useStrictFieldAssign()) {
+      // For reasons similar to volatile check above, there is no "AndConsume" optimization
+      // available here. We need ensure the assigned object remains valid in the assigning
+      // thread.
+      return "JreStrictFieldStrongAssign";
+    }
+
     if (isStrong && options.useReferenceCounting()) {
       String funcName = "JreStrongAssign";
       Expression retainedRhs = TranslationUtil.retainResult(node.getRightHandSide());
