@@ -39,6 +39,7 @@ import com.google.devtools.j2objc.ast.PackageDeclaration;
 import com.google.devtools.j2objc.ast.ParenthesizedExpression;
 import com.google.devtools.j2objc.ast.PropertyAnnotation;
 import com.google.devtools.j2objc.ast.QualifiedName;
+import com.google.devtools.j2objc.ast.RecordDeclaration;
 import com.google.devtools.j2objc.ast.SimpleName;
 import com.google.devtools.j2objc.ast.SingleVariableDeclaration;
 import com.google.devtools.j2objc.ast.Statement;
@@ -271,6 +272,7 @@ public class Rewriter extends UnitTreeVisitor {
         MethodDeclaration method = (MethodDeclaration) body;
         if (method.getParameters().isEmpty()
             && !method.isConstructor()
+            && !TypeUtil.isVoid(method.getReturnTypeMirror())
             && !ElementUtil.hasAnnotation(method.getExecutableElement(), Property.Suppress.class)) {
           method.setIsPseudoProperty(true);
         }
@@ -307,6 +309,11 @@ public class Rewriter extends UnitTreeVisitor {
     checkForNullabilityAnnotation(node);
   }
 
+  @Override
+  public void endVisit(RecordDeclaration node) {
+    checkForNullabilityAnnotation(node);
+  }
+
   private void checkForNullabilityAnnotation(AbstractTypeDeclaration node) {
     if (ElementUtil.hasAnnotation(node.getTypeElement(), ParametersAreNonnullByDefault.class)) {
       unit.setHasNullabilityAnnotations();
