@@ -28,14 +28,16 @@ import com.google.devtools.j2objc.ast.NumberLiteral;
 import com.google.devtools.j2objc.ast.SimpleName;
 import com.google.devtools.j2objc.ast.Statement;
 import com.google.devtools.j2objc.ast.SwitchCase;
+import com.google.devtools.j2objc.ast.SwitchExpression;
+import com.google.devtools.j2objc.ast.SwitchExpressionCase;
 import com.google.devtools.j2objc.ast.SwitchStatement;
+import com.google.devtools.j2objc.ast.TreeNode;
 import com.google.devtools.j2objc.ast.TreeUtil;
 import com.google.devtools.j2objc.ast.UnitTreeVisitor;
 import com.google.devtools.j2objc.ast.VariableDeclarationFragment;
 import com.google.devtools.j2objc.ast.VariableDeclarationStatement;
 import com.google.devtools.j2objc.types.ExecutablePair;
 import com.google.devtools.j2objc.types.FunctionElement;
-import com.google.devtools.j2objc.util.NameTable;
 import com.google.devtools.j2objc.util.TypeUtil;
 import java.util.List;
 import javax.lang.model.element.ElementKind;
@@ -43,10 +45,15 @@ import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.ArrayType;
 import javax.lang.model.type.DeclaredType;
 import javax.lang.model.type.TypeMirror;
+import org.jspecify.annotations.Nullable;
 
 /**
  * Rewrites switch statements to be more compatible with Objective-C code.
  *
+ * This stage doesn't include support for Java 21 patterns or guards in
+ * switches. That is handled by the SwitchCaseRewriter, which is run after
+ * this rewriter.
+ *
  * @author Keith Stanger
  */
 public class SwitchRewriter extends UnitTreeVisitor {
@@ -73,6 +80,12 @@ public class SwitchRewriter extends UnitTreeVisitor {
       }
     }
   }
+  
+  @Override
+  public void endVisit(SwitchExpression node) {
+    fixStringValue(node);
+    fixEnumValue(node);
+  }
 
   @Override
   public void endVisit(SwitchCase node) {
@@ -83,9 +96,7 @@ public class SwitchRewriter extends UnitTreeVisitor {
     }
     TypeMirror type = var.asType();
     if (TypeUtil.isEnum(type)) {
-      String enumValue =
-          NameTable.getNativeEnumName(nameTable.getFullName(TypeUtil.asTypeElement(type))) + "_"
-          + nameTable.getVariableBaseName(var);
+      String enumValue = nameTable.getNativeEnumConstantName(TypeUtil.asTypeElement(type), var);
       node.setExpression(new NativeExpression(enumValue, typeUtil.getInt()));
     } else if (type.getKind().isPrimitive() && var.getKind() == ElementKind.LOCAL_VARIABLE) {
       Object value = var.getConstantValue();
@@ -95,6 +106,62 @@ public class SwitchRewriter extends UnitTreeVisitor {
     }
   }
 
+  @Override
+  public void endVisit(SwitchExpressionCase node) {
+    List<Expression> exprs = node.getExpressions();
+    for (int i = 0; i < exprs.size(); i++) {
+      Expression expr = exprs.get(i);
+      VariableElement var = expr != null ? TreeUtil.getVariableElement(expr) : null;
+      if (var != null) {
+        TypeMirror type = var.asType();
+        if (TypeUtil.isEnum(type)) {
+          String enumValue = nameTable.getNativeEnumConstantName(TypeUtil.asTypeElement(type), var);
+          exprs.set(i, new NativeExpression(enumValue, typeUtil.getInt()));
+        } else if (type.getKind().isPrimitive() && var.getKind() == ElementKind.LOCAL_VARIABLE) {
+          Object value = var.getConstantValue();
+          if (value != null) {
+            exprs.set(i, TreeUtil.newLiteral(value, typeUtil));
+          }
+        }
+      }
+    }
+  }
+
+  @Override
+  public void endVisit(Block node) {
+    List<Statement> statements = node.getStatements();
+    Block block = new Block();
+    List<Statement> blockStmts = block.getStatements();
+    boolean blockChanged = false;
+    for (Statement stmt : statements) {
+      // If statement declares a local variable and its initializer is a switch
+      // expression, move the initializer into a separate expression statement.
+      if (stmt instanceof VariableDeclarationStatement) {
+        VariableDeclarationStatement declStmt = (VariableDeclarationStatement) stmt;
+        if (!declStmt.getFragments().isEmpty()) {
+          VariableDeclarationFragment fragment = declStmt.getFragments().get(0);
+          SwitchExpression switchExpression = null;
+          Expression initializer = fragment.getInitializer();
+          if (initializer instanceof SwitchExpression) {
+            switchExpression = ((SwitchExpression) initializer).copy();
+            VariableDeclarationFragment newDecl =
+                new VariableDeclarationFragment(fragment.getVariableElement(), null);
+            blockStmts.add(new VariableDeclarationStatement(newDecl));
+            blockStmts.add(new ExpressionStatement(switchExpression));
+            blockChanged = true;
+          } else {
+            blockStmts.add(declStmt.copy());
+          }
+        }
+      } else {
+        blockStmts.add(stmt.copy());
+      }
+    }
+    if (blockChanged) {
+      node.replaceWith(block);
+    }
+  }
+
   /**
    * Moves all variable declarations above the first case statement.
    */
@@ -127,22 +194,30 @@ public class SwitchRewriter extends UnitTreeVisitor {
     }
   }
 
-  private void fixStringValue(SwitchStatement node) {
-    Expression expr = node.getExpression();
+  private @Nullable Expression fixStringValue(Expression expr, List<Statement> statements) {
     TypeMirror type = expr.getTypeMirror();
     if (!typeUtil.isString(type)) {
-      return;
+      return null;
     }
     ArrayType arrayType = typeUtil.getArrayType(type);
     ArrayInitializer arrayInit = new ArrayInitializer(arrayType);
     int idx = 0;
-    for (Statement stmt : node.getStatements()) {
-      if (stmt instanceof SwitchCase) {
+    for (Statement stmt : statements) {
+      if (stmt.getKind() == TreeNode.Kind.SWITCH_CASE) {
         SwitchCase caseStmt = (SwitchCase) stmt;
         if (!caseStmt.isDefault()) {
           arrayInit.addExpression(TreeUtil.remove(caseStmt.getExpression()));
           caseStmt.setExpression(NumberLiteral.newIntLiteral(idx++, typeUtil));
         }
+      } else if (stmt.getKind() == TreeNode.Kind.SWITCH_EXPRESSION_CASE) {
+        SwitchExpressionCase caseStmt = (SwitchExpressionCase) stmt;
+        if (!caseStmt.isDefault()) {
+          List<Expression> caseExprs = caseStmt.getExpressions();
+          for (int i = 0; i < caseExprs.size(); i++) {
+            arrayInit.addExpression(caseExprs.get(i).copy());
+            caseExprs.set(i, NumberLiteral.newIntLiteral(idx++, typeUtil));
+          }
+        }
       }
     }
     TypeMirror intType = typeUtil.getInt();
@@ -152,7 +227,21 @@ public class SwitchRewriter extends UnitTreeVisitor {
     invocation.addArgument(TreeUtil.remove(expr))
         .addArgument(arrayInit)
         .addArgument(NumberLiteral.newIntLiteral(idx, typeUtil));
-    node.setExpression(invocation);
+    return invocation;
+  }
+
+  private void fixStringValue(SwitchStatement node) {
+    Expression expr = fixStringValue(node.getExpression(), node.getStatements());
+    if (expr != null) {
+      SwitchStatement unused = node.setExpression(expr);
+    }
+  }
+
+  private void fixStringValue(SwitchExpression node) {
+    Expression expr = fixStringValue(node.getExpression(), node.getStatements());
+    if (expr != null) {
+      SwitchExpression unused = node.setExpression(expr);
+    }
   }
 
   private void fixEnumValue(SwitchStatement node) {
@@ -166,4 +255,16 @@ public class SwitchRewriter extends UnitTreeVisitor {
     MethodInvocation invocation = new MethodInvocation(ordinalMethod, TreeUtil.remove(expr));
     node.setExpression(invocation);
   }
+
+  private void fixEnumValue(SwitchExpression node) {
+    Expression expr = node.getExpression();
+    TypeMirror type = expr.getTypeMirror();
+    if (!TypeUtil.isEnum(type)) {
+      return;
+    }
+    DeclaredType enumType = typeUtil.getSuperclass(type);
+    ExecutablePair ordinalMethod = typeUtil.findMethod(enumType, "ordinal");
+    MethodInvocation invocation = new MethodInvocation(ordinalMethod, TreeUtil.remove(expr));
+    var unused = node.setExpression(invocation);
+  }
 }
