@@ -16,13 +16,16 @@ package com.google.devtools.j2objc.types;
 
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 import com.google.devtools.j2objc.ast.DebugASTPrinter;
 import com.google.devtools.j2objc.util.ElementUtil;
 import com.google.devtools.j2objc.util.NameTable;
 import com.google.devtools.j2objc.util.TypeUtil;
+import com.google.errorprone.annotations.CanIgnoreReturnValue;
 import java.lang.annotation.Annotation;
 import java.util.Collection;
 import java.util.List;
+import java.util.Set;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.AnnotationValue;
 import javax.lang.model.element.Element;
@@ -52,6 +55,7 @@ public class GeneratedExecutableElement extends GeneratedElement implements Exec
   private final List<TypeParameterElement> typeParameters = Lists.newArrayList();
   private final TypeMirror returnType;
   private final boolean varargs;
+  private ExecutableElement originalElement;
 
   private GeneratedExecutableElement(
       String name, String selector, ElementKind kind, TypeMirror returnType,
@@ -91,6 +95,30 @@ public class GeneratedExecutableElement extends GeneratedElement implements Exec
         method.isVarArgs(), ElementUtil.isSynthetic(method));
   }
 
+  public static GeneratedExecutableElement newAdapterMethod(
+      String selector,
+      TypeMirror adapterReturnType,
+      List<VariableElement> adapterParameters,
+      ExecutableElement method) {
+    GeneratedExecutableElement generatedMethod =
+        new GeneratedExecutableElement(
+            selector,
+            selector,
+            method.getKind(),
+            adapterReturnType,
+            method.getEnclosingElement(),
+            method.isVarArgs(),
+            ElementUtil.isSynthetic(method));
+    generatedMethod.addAnnotationMirrors(method.getAnnotationMirrors());
+    Set<Modifier> modifiers = Sets.newHashSet(method.getModifiers());
+    modifiers.remove(Modifier.ABSTRACT); // Adapter methods are always concrete.
+    generatedMethod.addModifiers(modifiers);
+    generatedMethod.parameters.addAll(adapterParameters);
+    generatedMethod.thrownTypes.addAll(method.getThrownTypes());
+    generatedMethod.typeParameters.addAll(method.getTypeParameters());
+    return generatedMethod;
+  }
+
   public static GeneratedExecutableElement mutableCopy(String selector, ExecutableElement method) {
     GeneratedExecutableElement generatedMethod =
         new GeneratedExecutableElement(
@@ -208,6 +236,16 @@ public class GeneratedExecutableElement extends GeneratedElement implements Exec
     throw new AssertionError("not implemented");
   }
 
+  public ExecutableElement getOriginalElement() {
+    return originalElement;
+  }
+
+  @CanIgnoreReturnValue
+  public GeneratedExecutableElement setOriginalElement(ExecutableElement element) {
+    originalElement = element;
+    return this;
+  }
+
   /** The associated ExecutableType. */
   private class Mirror extends AbstractTypeMirror implements ExecutableType {
 
