@@ -194,7 +194,6 @@ public class PackageInfoLookup {
 
     // @NullMarked
     if (pkgInfo.contains("@NullMarked")
-        || pkgInfo.contains("@org.jspecify.nullness.NullMarked")
         || pkgInfo.contains("@org.jspecify.annotations.NullMarked")) {
       builder.setNullMarked();
     }
@@ -231,7 +230,7 @@ public class PackageInfoLookup {
         }
       } else if (signature.equals("Ljavax/annotation/ParametersAreNonnullByDefault;")) {
         builder.setParametersAreNonnullByDefault();
-      } else if (signature.equals("Lorg/jspecify/nullness/NullMarked;")) {
+      } else if (signature.equals("Lorg/jspecify/annotations/NullMarked;")) {
         builder.setNullMarked();
       } else if (signature.equals("Lcom/google/j2objc/annotations/ReflectionSupport;")) {
         for (Expression expr : annotation.getArguments()) {
