@@ -15,8 +15,8 @@
 package com.google.devtools.j2objc.util;
 
 import com.google.common.base.Joiner;
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
-import com.google.common.collect.ImmutableSet;
 import com.google.devtools.j2objc.types.AbstractTypeMirror;
 import com.google.devtools.j2objc.types.ExecutablePair;
 import com.google.devtools.j2objc.types.GeneratedArrayType;
@@ -34,6 +34,7 @@ import java.util.List;
 import java.util.Map;
 import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.ExecutableElement;
+import javax.lang.model.element.PackageElement;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.TypeParameterElement;
 import javax.lang.model.element.VariableElement;
@@ -59,12 +60,15 @@ import javax.lang.model.util.Types;
  */
 public final class TypeUtil {
 
+  public static final TypeMirror BOOL_TYPE = new NativeType("bool");
   public static final TypeMirror ID_TYPE = new NativeType("id");
   public static final TypeMirror ID_PTR_TYPE = new PointerType(ID_TYPE);
   public static final TypeElement NS_OBJECT =
       GeneratedTypeElement.newIosClass("NSObject", null, "");
   public static final TypeElement NS_STRING =
       GeneratedTypeElement.newIosClass("NSString", NS_OBJECT, "");
+  public static final TypeElement NS_ERROR =
+      GeneratedTypeElement.newIosClass("NSError", NS_OBJECT, "");
   public static final TypeElement NS_EXCEPTION =
       GeneratedTypeElement.newIosClass("NSException", NS_OBJECT, "");
   public static final TypeElement NS_NUMBER =
@@ -211,6 +215,10 @@ public final class TypeUtil {
     return t.getKind() == TypeKind.TYPEVAR;
   }
 
+  public static boolean isNativeType(TypeMirror t) {
+    return t instanceof NativeType;
+  }
+
   public static boolean hasGenerateObjectiveCGenerics(TypeElement t) {
     return ElementUtil.hasAnnotation(t, GenerateObjectiveCGenerics.class);
   }
@@ -231,6 +239,16 @@ public final class TypeUtil {
     return null;
   }
 
+  public static List<? extends TypeMirror> getTypeArguments(TypeMirror t) {
+    if (isDeclaredType(t)) {
+      return ((DeclaredType) t).getTypeArguments();
+    } else if (isNativeType(t)) {
+      return ((NativeType) t).getTypeArguments();
+    } else {
+      return ImmutableList.of();
+    }
+  }
+
   public static boolean isJavaObject(TypeMirror t) {
     TypeElement typeElement = asTypeElement(t);
     return typeElement != null && !isInterface(t) && isNone(typeElement.getSuperclass());
@@ -603,7 +621,7 @@ public final class TypeUtil {
   }
 
   public static boolean isId(TypeMirror t) {
-    return t instanceof NativeType && ((NativeType) t).getName().equals("id");
+    return isNativeType(t) && ((NativeType) t).getName().equals("id");
   }
 
   public PrimitiveType getPrimitiveType(TypeKind kind) {
@@ -710,18 +728,14 @@ public final class TypeUtil {
   }
 
   public boolean isProtoClass(TypeMirror type) {
-    ImmutableSet<String> protoClassNames =
-        ImmutableSet.of(
-            "com.google.protobuf.GeneratedMessage",
-            "com.google.protobuf.GeneratedMessageLite",
-            "com.google.protobuf.MessageOrBuilder",
-            "com.google.protobuf.MessageLiteOrBuilder",
-            "com.google.protobuf.Extension",
-            "com.google.protobuf.ExtensionLite");
-
     TypeElement element = asTypeElement(type);
-    if (element != null && protoClassNames.contains(element.getQualifiedName().toString())) {
-      return true;
+    if (element != null) {
+      PackageElement pkg = ElementUtil.getPackage(element);
+      if (pkg != null) {
+        if (pkg.toString().equals("com.google.protobuf")) {
+          return true;
+        }
+      }
     }
     for (TypeMirror t : directSupertypes(type)) {
       boolean result = isProtoClass(t);
@@ -797,6 +811,9 @@ public final class TypeUtil {
   }
 
   public static String getName(TypeMirror t) {
+    if (t instanceof NativeType) {
+      return ((NativeType) t).getName();
+    }
     switch (t.getKind()) {
       case ARRAY:
         return getName(((ArrayType) t).getComponentType()) + "[]";
