@@ -66,17 +66,22 @@ import com.google.devtools.j2objc.translate.LogSiteInjectorTest;
 import com.google.devtools.j2objc.translate.MetadataWriterTest;
 import com.google.devtools.j2objc.translate.NilCheckResolverTest;
 import com.google.devtools.j2objc.translate.NumberMethodRewriterTest;
+import com.google.devtools.j2objc.translate.ObjectiveCAdapterMethodAnnotationTest;
+import com.google.devtools.j2objc.translate.ObjectiveCNativeEnumNameTest;
+import com.google.devtools.j2objc.translate.ObjectiveCNativeProtocolAnnotationTest;
 import com.google.devtools.j2objc.translate.OcniExtractorTest;
 import com.google.devtools.j2objc.translate.OperatorRewriterTest;
 import com.google.devtools.j2objc.translate.OuterReferenceFixerTest;
 import com.google.devtools.j2objc.translate.OuterReferenceResolverTest;
 import com.google.devtools.j2objc.translate.PackageInfoRewriterTest;
 import com.google.devtools.j2objc.translate.PrivateDeclarationResolverTest;
+import com.google.devtools.j2objc.translate.RecordExpanderTest;
 import com.google.devtools.j2objc.translate.ReflectionCodeDetectorTest;
 import com.google.devtools.j2objc.translate.RewriterTest;
 import com.google.devtools.j2objc.translate.SerializationStripperTest;
 import com.google.devtools.j2objc.translate.StaticVarRewriterTest;
 import com.google.devtools.j2objc.translate.SuperMethodInvocationRewriterTest;
+import com.google.devtools.j2objc.translate.SwitchCaseRewriterTest;
 import com.google.devtools.j2objc.translate.SwitchRewriterTest;
 import com.google.devtools.j2objc.translate.TypeUseAnnotationTest;
 import com.google.devtools.j2objc.translate.UnsequencedExpressionRewriterTest;
@@ -154,8 +159,11 @@ public class SmallTests {
         NameTableTest.class,
         NilCheckResolverTest.class,
         NumberMethodRewriterTest.class,
+        ObjectiveCAdapterMethodAnnotationTest.class,
         ObjectiveCHeaderGeneratorTest.class,
         ObjectiveCImplementationGeneratorTest.class,
+        ObjectiveCNativeEnumNameTest.class,
+        ObjectiveCNativeProtocolAnnotationTest.class,
         ObjectiveCSegmentedHeaderGeneratorTest.class,
         ObjectiveCSourceFileGeneratorTest.class,
         OcniExtractorTest.class,
@@ -169,6 +177,7 @@ public class SmallTests {
         PrimitiveArrayTest.class,
         PrivateDeclarationResolverTest.class,
         ProGuardUsageParserTest.class,
+        RecordExpanderTest.class,
         ReflectionCodeDetectorTest.class,
         RewriterTest.class,
         SerializationStripperTest.class,
@@ -176,6 +185,7 @@ public class SmallTests {
         StatementGeneratorTest.class,
         StaticVarRewriterTest.class,
         SuperMethodInvocationRewriterTest.class,
+        SwitchCaseRewriterTest.class,
         SwitchRewriterTest.class,
         TypeDeclarationGeneratorTest.class,
         TypeImplementationGeneratorTest.class,
