@@ -38,7 +38,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
         + "class MyClass {}", "Example", "mypackage/Example.h");
     assertTranslation(translation, "@interface MypackageExample");
     // enum declaration
-    assertTranslation(translation, "typedef NS_ENUM(jint, MypackageAbcd_Enum) {");
+    assertTranslation(translation, "typedef NS_ENUM(int32_t, MypackageAbcd_Enum) {");
     assertTranslation(translation, "@interface MypackageAbcd");
     assertTranslation(translation, "@interface MypackageMyClass");
     assertTranslation(translation, "MypackageMyClass *myclass_;");
@@ -446,7 +446,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
       "public class Example { int one, two, three; }",
       "Example", "Example.h");
     assertTranslatedLines(translation,
-        "jint one_;", "jint two_;", "jint three_;");
+        "int32_t one_;", "int32_t two_;", "int32_t three_;");
   }
 
   public void testMultipleInterfaceDeclaration() throws IOException {
@@ -490,13 +490,14 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
       "Color", "Color.h");
     assertTranslatedLines(
         translation,
-        "typedef NS_ENUM(jint, Color_Enum) {",
+        "typedef NS_ENUM(int32_t, Color_Enum) {",
         "  Color_Enum_RED NS_SWIFT_NAME(red) = 0,",
         "  Color_Enum_WHITE NS_SWIFT_NAME(white) = 1,",
         "  Color_Enum_BLUE NS_SWIFT_NAME(blue) = 2,",
         "};");
     assertTranslation(translation, "@interface Color : JavaLangEnum");
     assertTranslation(translation, "+ (IOSObjectArray *)values;");
+    assertTranslation(translation, "@property(readonly) Color_Enum enumValue;");
     assertTranslation(translation, "+ (Color *)valueOfWithNSString:(NSString *)name;");
     assertTranslation(translation, "FOUNDATION_EXPORT Color *Color_values_[];");
     assertTranslatedLines(
@@ -513,7 +514,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
         translateSourceFile("public enum Color { RED, WHITE, BLUE }", "Color", "Color.h");
     assertTranslatedLines(
         translation,
-        "typedef NS_ENUM(jint, Color_Enum) {",
+        "typedef NS_ENUM(int32_t, Color_Enum) {",
         "  Color_Enum_RED = 0,",
         "  Color_Enum_WHITE = 1,",
         "  Color_Enum_BLUE = 2,",
@@ -542,7 +543,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
         translation, "NS_ASSUME_NONNULL_BEGIN", "@interface Color : JavaLangEnum");
     assertTranslatedLines(
         translation,
-        "typedef NS_ENUM(jint, Color_Enum) {",
+        "typedef NS_ENUM(int32_t, Color_Enum) {",
         "  Color_Enum_RED NS_SWIFT_NAME(red) = 0,",
         "  Color_Enum_WHITE NS_SWIFT_NAME(white) = 1,",
         "  Color_Enum_BLUE NS_SWIFT_NAME(blue) = 2,",
@@ -560,10 +561,10 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
         "Color", "Color.h");
     assertTranslation(translation, "@interface Color : JavaLangEnum");
     translation = getTranslatedFile("Color.m");
-    assertTranslation(translation, "int rgb_;");
+    assertTranslation(translation, "int32_t rgb_;");
     assertTranslatedLines(translation,
         "void Color_initWithInt_withNSString_withInt_("
-        + "Color *self, jint rgb, NSString *__name, jint __ordinal) {");
+        + "Color *self, int32_t rgb, NSString *__name, int32_t __ordinal) {");
   }
 
   public void testEnumWithMultipleConstructors() throws IOException {
@@ -577,28 +578,42 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
       "Color", "Color.h");
     assertTranslation(translation, "@interface Color : JavaLangEnum");
     translation = getTranslatedFile("Color.m");
-    assertTranslation(translation, "jboolean primary_;");
+    assertTranslation(translation, "bool primary_;");
     assertTranslation(translation,
         "Color_initWithInt_withBoolean_withNSString_withInt_("
         + "self, rgb, true, __name, __ordinal);");
-    assertTranslatedLines(translation,
+    assertTranslatedLines(
+        translation,
         "void Color_initWithInt_withBoolean_withNSString_withInt_("
-          + "Color *self, jint rgb, jboolean primary, NSString *__name, jint __ordinal) {",
+            + "Color *self, int32_t rgb, bool primary, NSString *__name, int32_t __ordinal) {",
         "  JavaLangEnum_initWithNSString_withInt_(self, __name, __ordinal);",
         "  self->rgb_ = rgb;",
         "  self->primary_ = primary;",
         "}");
     assertTranslatedLines(translation,
         "void Color_initWithInt_withNSString_withInt_("
-          + "Color *self, jint rgb, NSString *__name, jint __ordinal) {",
+          + "Color *self, int32_t rgb, NSString *__name, int32_t __ordinal) {",
         "  Color_initWithInt_withBoolean_withNSString_withInt_("
           + "self, rgb, true, __name, __ordinal);",
         "}");
     assertTranslation(translation,
         "Color_initWithInt_withBoolean_withNSString_withInt_("
-          + "e, (jint) 0xffffff, false, @\"WHITE\", 1);");
+          + "e, (int32_t) 0xffffff, false, @\"WHITE\", 1);");
     assertTranslation(translation,
-        "Color_initWithInt_withNSString_withInt_(e, (jint) 0x0000ff, @\"BLUE\", 2);");
+        "Color_initWithInt_withNSString_withInt_(e, (int32_t) 0x0000ff, @\"BLUE\", 2);");
+  }
+
+  public void testEnumInitialization() throws IOException {
+    String sourceContent = "  package com.foo.bar;" + "" + "public enum Color { RED, WHITE, BLUE }";
+    String translation = translateSourceFile(sourceContent, "Color", "com/foo/bar/Color.h");
+    assertTranslatedLines(
+        translation, "- (nullable instancetype)initWithColor:(ComFooBarColor_Enum)value;");
+    translation = getTranslatedFile("com/foo/bar/Color.m");
+    assertTranslatedLines(
+        translation,
+        "- (nullable instancetype)initWithColor:(ComFooBarColor_Enum)value {",
+        "  return RETAIN_(ComFooBarColor_fromOrdinal((ComFooBarColor_ORDINAL)value));",
+        "}");
   }
 
   public void testArrayFieldDeclaration() throws IOException {
@@ -628,16 +643,16 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     assertTranslation(translation, "@interface FooCompatible : NSObject < FooCompatible >");
 
     // Verify that the value is defined as a property instead of a method.
-    assertTranslation(translation, "@public\n  jboolean fooable_;");
-    assertTranslation(translation, "@property (readonly) jboolean fooable;");
+    assertTranslation(translation, "@public\n  bool fooable_;");
+    assertTranslation(translation, "@property (readonly) bool fooable;");
 
     // Check that constructor was created with the property as parameter.
-    assertTranslation(translation,
-        "FOUNDATION_EXPORT id<FooCompatible> create_FooCompatible(jboolean fooable);");
+    assertTranslation(
+        translation, "FOUNDATION_EXPORT id<FooCompatible> create_FooCompatible(bool fooable);");
 
     translation = getTranslatedFile("foo/Compatible.m");
     // Verify default value accessor is generated for property.
-    assertTranslation(translation, "+ (jboolean)fooableDefault {");
+    assertTranslation(translation, "+ (bool)fooableDefault {");
   }
 
   public void testCharacterEdgeValues() throws IOException {
@@ -654,7 +669,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     String translation = translateSourceFile(
         "public enum MyEnum { ONE, TWO, THREE }",
         "MyEnum", "MyEnum.h");
-    assertTranslation(translation, "typedef NS_ENUM(jint, MyEnum_Enum) {");
+    assertTranslation(translation, "typedef NS_ENUM(int32_t, MyEnum_Enum) {");
     assertTranslation(translation, "@interface MyEnum : JavaLangEnum");
     assertTranslation(translation, "FOUNDATION_EXPORT MyEnum *MyEnum_values_[];");
     assertTranslation(translation, "inline MyEnum *MyEnum_get_ONE(void);");
@@ -846,7 +861,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     options.setNullMarked(true);
     options.setNullability(true);
     addSourceFile(
-        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "@NullMarked package foo.bar;" + "import org.jspecify.annotations.NullMarked;",
         "foo/bar/package-info.java");
 
     String sourceContent =
@@ -873,6 +888,18 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
             + " NSString * fooField;");
   }
 
+  public void testPropertyAnnotationIgnoresVoidMethod() throws IOException {
+    String sourceContent =
+        "  import com.google.j2objc.annotations.Property;"
+            + "@Property "
+            + "public class FooBar {"
+            + "  public void doFoo() {}"
+            + "  "
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "FooBar.h");
+    assertNotInTranslation(translation, "@property");
+  }
+
   public void testPropertiesOfClassTypes() throws IOException {
     String sourceContent =
         "  import com.google.j2objc.annotations.Property;"
@@ -940,7 +967,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     options.setNullMarked(true);
     options.setNullability(true);
     addSourceFile(
-        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "@NullMarked package foo.bar;" + "import org.jspecify.annotations.NullMarked;",
         "foo/bar/package-info.java");
     String translation =
         translateSourceFile(
@@ -950,7 +977,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
             "foo/bar/Test.h");
     assertTranslation(translation, "J2OBJC_FIELD_SETTER(FooBarTest, a_, NSString *_Nullable)");
     assertTranslation(translation, "J2OBJC_FIELD_SETTER(FooBarTest, b_, NSString *)");
-    assertTranslation(translation, "J2OBJC_FIELD_SETTER(FooBarTest, c_, id)");
+    assertTranslation(translation, "J2OBJC_FIELD_SETTER(FooBarTest, c_, id _Nullable)");
   }
 
   public void testStaticFieldObject() throws IOException {
@@ -964,7 +991,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     options.setNullMarked(true);
     options.setNullability(true);
     addSourceFile(
-        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "@NullMarked package foo.bar;" + "import org.jspecify.annotations.NullMarked;",
         "foo/bar/package-info.java");
     String translation =
         translateSourceFile(
@@ -993,10 +1020,10 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     assertTranslation(translation, "@interface Test : JavaLangEnum");
     translation = getTranslatedFile("Test.m");
     assertTranslation(translation, "NSString *name_Test_;");
-    assertTranslation(translation, "int ordinal_Test_;");
+    assertTranslation(translation, "int32_t ordinal_Test_;");
     assertTranslation(translation,
         "void Test_initWithNSString_withInt_withNSString_withInt_("
-        + "Test *self, NSString *name, jint ordinal, NSString *__name, jint __ordinal) {");
+        + "Test *self, NSString *name, int32_t ordinal, NSString *__name, int32_t __ordinal) {");
   }
 
   public void testDeprecatedEnumType() throws IOException {
@@ -1037,6 +1064,304 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     assertTranslation(translation, "+ (NSString *)namespace__Default {");
   }
 
+  public void testSwiftNamePackageAnnotation() throws IOException {
+    addSourceFile(
+        "@SwiftName "
+            + "package com.foo.bar;"
+            + ""
+            + "import com.google.j2objc.annotations.SwiftName;",
+        "com/foo/bar/package-info.java");
+
+    String sourceContent =
+        "  package com.foo.bar;"
+            + ""
+            + "public abstract class FooBar {"
+            + ""
+            + "  public void setFooField(String fooField) {"
+            + " "
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar)");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(set(fooField:))");
+  }
+
+  public void testSwiftNameClassAnnotation() throws IOException {
+    String sourceContent =
+        "  package com.foo.bar;"
+            + "import com.google.j2objc.annotations.SwiftName;"
+            + ""
+            + "@SwiftName "
+            + "public abstract class FooBar {"
+            + "  public void setFooField(String fooField) {}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar)");
+  }
+
+  public void testSwiftNameClassAnnotationOverride() throws IOException {
+    String sourceContent =
+        "  package com.foo.bar;"
+            + "import com.google.j2objc.annotations.SwiftName;"
+            + ""
+            + "@SwiftName(\"MyFooBar\") "
+            + "public abstract class FooBar {"
+            + "  public void setFooField(String fooField) {}"
+            + "  public static void builderWithExpectedSize(int expectedSize){}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(MyFooBar)");
+  }
+
+  public void testSwiftNameMethodAnnotationOverride() throws IOException {
+    String sourceContent =
+        "  package com.foo.bar;"
+            + "import com.google.j2objc.annotations.SwiftName;"
+            + ""
+            + "public abstract class FooBar {"
+            + "  @SwiftName(\"set(name:)\") "
+            + "  public void setFooField(String fooField) {}"
+            + "  @SwiftName(\"builder(size:)\") "
+            + "  public static void builderWithExpectedSize(int expectedSize){}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(set(name:))");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(builder(size:))");
+  }
+
+  public void testSwiftNameAnnotationWithNestedTypes() throws IOException {
+    addSourceFile(
+        "@SwiftName "
+            + "package com.foo.bar;"
+            + ""
+            + "import com.google.j2objc.annotations.SwiftName;",
+        "com/foo/bar/package-info.java");
+
+    String sourceContent =
+        "  package com.foo.bar;"
+            + ""
+            + "public abstract class FooBar {"
+            + ""
+            + "  public static final class NestedBar {}"
+            + "  public void setFooField(String fooField) {"
+            + " "
+            + "  }"
+            + "   public static void builderWithExpectedSize(int expectedSize){}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar.NestedBar)");
+  }
+
+  public void testSwiftNamePackageAnnotationEnumSubtype() throws IOException {
+    addSourceFile(
+        "@SwiftName "
+            + "package com.foo.bar;"
+            + ""
+            + "import com.google.j2objc.annotations.SwiftName;",
+        "com/foo/bar/package-info.java");
+
+    String sourceContent =
+        "  package com.foo.bar;"
+            + ""
+            + "public abstract class FooBar {"
+            + "  public enum MyEnum { ONE, TWO, THREE } "
+            + "  public void setFooField(String fooField) {"
+            + " "
+            + "  }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar.MyEnumClass)");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar.MyEnum)");
+  }
+
+  public void testSwiftNameInheritedInterface() throws IOException {
+    options.setStripReflection(true);
+    options.setEmitWrapperMethods(false);
+    options.setAsObjCGenericDecl(true);
+    addSourceFile(
+        "@SwiftName " + "package foo;" + "" + "import com.google.j2objc.annotations.SwiftName;",
+        "foo/package-info.java");
+
+    addSourceFile(
+        "@SwiftName " + "package bar;" + "" + "import com.google.j2objc.annotations.SwiftName;",
+        "bar/package-info.java");
+
+    addSourceFile(
+        "  package foo;"
+            + ""
+            + "public interface FooBar<E> {"
+            + "  public E getElementByIndex(int index); "
+            + "}",
+        "foo/FooBar.java");
+
+    addSourceFile(
+        "  package bar;"
+            + "import foo.FooBar; "
+            + ""
+            + "public interface BarFoo extends FooBar<String> {"
+            + " "
+            + "}",
+        "bar/BarFoo.java");
+
+    String fooHeader = translateCombinedFiles("com/Foo", ".h", "foo/FooBar.java");
+    String barHeader = translateCombinedFiles("com/Bar", ".h", "bar/BarFoo.java");
+    assertTranslatedLines(
+        barHeader,
+        "- (NSString *)getElementByIndexWithInt:(int32_t)index"
+            + " NS_SWIFT_NAME(getElementByInt(index:));");
+    assertTranslatedLines(
+        fooHeader,
+        "- (id)getElementByIndexWithInt:(int32_t)index NS_SWIFT_NAME(getElementBy(index:));");
+  }
+
+  public void testSwiftNameAbnormalArgumentNames() throws IOException {
+    addSourceFile(
+        "@SwiftName " + "package foo;" + "" + "import com.google.j2objc.annotations.SwiftName;",
+        "foo/package-info.java");
+
+    addSourceFile(
+        "  package foo;"
+            + ""
+            + "public interface FooBar {"
+            + "  public String fooBar(String fooBar);"
+            + "  public String setSomeValue(String value);"
+            + "  public String setSomeArgument(String arg0);"
+            + "}",
+        "foo/FooBar.java");
+
+    String fooHeader = translateCombinedFiles("com/Foo", ".h", "foo/FooBar.java");
+    assertTranslatedLines(
+        fooHeader,
+        "- (NSString *)fooBarWithNSString:(NSString *)fooBar NS_SWIFT_NAME(fooBar(_:));");
+    assertTranslatedLines(
+        fooHeader,
+        "- (NSString *)setSomeValueWithNSString:(NSString *)value NS_SWIFT_NAME(setSome(_:));");
+    assertTranslatedLines(
+        fooHeader,
+        "- (NSString *)setSomeArgumentWithNSString:(NSString *)arg0"
+            + " NS_SWIFT_NAME(setSomeArgument(_:));");
+  }
+
+  public void testSwiftNameAnnotationWithStaticFunctions() throws IOException {
+    options.setEmitWrapperMethods(false);
+
+    addSourceFile(
+        "@SwiftName "
+            + "package com.foo.bar;"
+            + ""
+            + "import com.google.j2objc.annotations.SwiftName;",
+        "com/foo/bar/package-info.java");
+
+    String sourceContent =
+        "  package com.foo.bar;"
+            + ""
+            + "public class FooBar {"
+            + "  FooBar() {}"
+            + "  public static final class NestedBar {}"
+            + "  public void setFooField(String fooField) {"
+            + " "
+            + "  }"
+            + "  public static String builderWithExpectedSize(int expectedSize){ return \"\"; }"
+            + "  public static String builderWithName(String name){ return \"\"; }"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar.init())");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar.builder(expectedSize:))");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(FooBar.builder(name:))");
+  }
+
+  public void testSwiftNameAnnotationWithStaticFunctionsWithWapperMethods() throws IOException {
+    options.setEmitWrapperMethods(true);
+
+    addSourceFile(
+        "@SwiftName "
+            + "package com.foo.bar;"
+            + ""
+            + "import com.google.j2objc.annotations.SwiftName;",
+        "com/foo/bar/package-info.java");
+
+    String sourceContent =
+        "  package com.foo.bar;"
+            + ""
+            + "public abstract class FooBar {"
+            + "  FooBar() {}"
+            + "  public static final class NestedBar {}"
+            + "  public void setFooField(String fooField) {"
+            + " "
+            + "  }"
+            + "   public static void builderWithExpectedSize(int expectedSize){}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(builder(expectedSize:))");
+  }
+
+  public void testSwiftNameAnnotationOverrideConstructor() throws IOException {
+    String sourceContent =
+        "  package com.foo.bar;"
+            + "import com.google.j2objc.annotations.SwiftName;"
+            + ""
+            + "public abstract class FooBar {"
+            + "  @SwiftName(\"init(size:)\") "
+            + "  FooBar(int expectedSize) {}"
+            + "  public void setFooField(String fooField) {}"
+            + "  public static void builderWithExpectedSize(int expectedSize){}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(init(size:))");
+  }
+
+  public void testSwiftNameAnnotationOverrideConstructorNoWrapperMethods() throws IOException {
+    options.setEmitWrapperMethods(false);
+    String sourceContent =
+        "  package com.foo.bar;"
+            + "import com.google.j2objc.annotations.SwiftName;"
+            + ""
+            + "public class FooBar {"
+            + "  @SwiftName(\"init(size:)\") "
+            + "  public FooBar(int expectedSize) {}"
+            + "  public void setFooField(String fooField) {}"
+            + "  public static void builderWithExpectedSize(int expectedSize){}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(ComFooBarFooBar.init(expectedSize:))");
+  }
+
+  public void testSwiftNameEnumAnnotation() throws IOException {
+    options.setSwiftNaming(true);
+
+    String sourceContent =
+        "  package com.foo.bar;"
+            + "import com.google.j2objc.annotations.SwiftName;"
+            + ""
+            + "@SwiftName "
+            + "public enum Color { RED, WHITE, BLUE }";
+    String translation = translateSourceFile(sourceContent, "Color", "com/foo/bar/Color.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(Color)");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(ColorClass)");
+  }
+
+  public void testSwiftNameInterfaceSubtype() throws IOException {
+    addSourceFile(
+        "@SwiftName "
+            + "package com.foo.bar;"
+            + ""
+            + "import com.google.j2objc.annotations.SwiftName;",
+        "com/foo/bar/package-info.java");
+
+    String sourceContent =
+        "  package com.foo.bar;"
+            + ""
+            + "public interface FooBar {"
+            + "  public enum MyEnum { ONE, TWO, THREE } "
+            + "  public void setFooField(String fooField);"
+            + "  public final class NestedBar {}"
+            + "}";
+    String translation = translateSourceFile(sourceContent, "FooBar", "com/foo/bar/FooBar.h");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(MyEnumClass)");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(MyEnum)");
+    assertTranslatedLines(translation, "NS_SWIFT_NAME(NestedBar)");
+  }
+
   public void testMethodSorting() throws IOException {
     String translation = translateSourceFile("class A {"
         + "protected void gnu(String s, int i, Runnable r) {}"
@@ -1047,12 +1372,12 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     assertTranslatedLines(translation,
         "#pragma mark Public",
         "",
-        "- (instancetype)initWithInt:(jint)i;",
+        "- (instancetype)initWithInt:(int32_t)i;",
         "",
         "#pragma mark Protected",
         "",
         "- (void)gnuWithNSString:(NSString *)s",
-                        "withInt:(jint)i",
+                        "withInt:(int32_t)i",
            "withJavaLangRunnable:(id<JavaLangRunnable>)r;",
         "",
         "#pragma mark Package-Private",
@@ -1087,7 +1412,7 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
   public void testDefaultConstructorDisallowed() throws IOException {
     options.setDisallowInheritedConstructors(true);
     String translation = translateSourceFile("class Test { Test(int i) {} }", "Test", "Test.h");
-    assertTranslation(translation, "- (instancetype)initWithInt:(jint)i;");
+    assertTranslation(translation, "- (instancetype)initWithInt:(int32_t)i;");
     assertTranslation(translation, "- (instancetype)init NS_UNAVAILABLE;");
   }
 
@@ -1104,11 +1429,12 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     assertTranslatedLines(translation,
         "- (instancetype)initWithNSString:(NSString *)arg0",
         "withJavaLangThrowable:(JavaLangThrowable *)arg1 NS_UNAVAILABLE;");
-    assertTranslatedLines(translation,
+    assertTranslatedLines(
+        translation,
         "- (instancetype)initWithNSString:(NSString *)arg0",
         "withJavaLangThrowable:(JavaLangThrowable *)arg1",
-        "withBoolean:(jboolean)arg2",
-        "withBoolean:(jboolean)arg3 NS_UNAVAILABLE;");
+        "withBoolean:(bool)arg2",
+        "withBoolean:(bool)arg3 NS_UNAVAILABLE;");
   }
 
   public void testStaticInterfaceMethodDeclaredInCompanionClass() throws IOException {
@@ -1164,4 +1490,111 @@ public class ObjectiveCHeaderGeneratorTest extends GenerationTest {
     // ... just the source file entry in that jar file.
     assertTranslation(kytheMetadata, "\"path\":\"foo/Test.java\"");
   }
+
+  public void testSeparateHeaderGeneration() throws IOException {
+    options.setSeparateHeaders(true);
+    String source =
+        String.join(
+            "\n",
+            "package foo;",
+            "public class Test implements Runnable {",
+            "  private int n;",
+            "  public void run() {}",
+            "  static class Bar implements AutoCloseable {",
+            "    public void close() throws Exception {}",
+            "  }",
+            "  static enum Mumble { A, B, C; }",
+            "  class Inner {",
+            "    int answerToEverything() { return n; }",
+            "  }",
+            "}");
+    String mainHeader = translateSourceFile(source, "Test", "foo/Test.h");
+    assertTranslation(mainHeader, "#import \"java/lang/Runnable.h\"");
+    assertTranslation(mainHeader, "- (instancetype)init;");
+    assertNotInTranslation(mainHeader, "Bar");
+    assertNotInTranslation(mainHeader, "Mumble");
+    assertNotInTranslation(mainHeader, "Inner");
+
+    String barHeader = getTranslatedFile("foo/Test_Bar.h");
+    assertTranslation(barHeader, "#import \"java/lang/AutoCloseable.h\"");
+    assertTranslation(barHeader, "- (void)close;");
+    assertNotInTranslation(barHeader, "Mumble");
+    assertNotInTranslation(barHeader, "Inner");
+
+    String mumbleHeader = getTranslatedFile("foo/Test_Mumble.h");
+    assertTranslation(mumbleHeader, "#import \"java/lang/Enum.h\"");
+    assertTranslation(mumbleHeader, "J2OBJC_ENUM_CONSTANT(FooTest_Mumble, A)");
+    assertNotInTranslation(mumbleHeader, "Bar");
+    assertNotInTranslation(mumbleHeader, "Inner");
+
+    String innerHeader = getTranslatedFile("foo/Test_Inner.h");
+    assertTranslation(innerHeader, "@class FooTest;");
+    assertTranslation(innerHeader, "- (int32_t)answerToEverything;");
+    assertNotInTranslation(innerHeader, "Bar");
+    assertNotInTranslation(innerHeader, "Mumble");
+
+    String implFile = getTranslatedFile("foo/Test.m");
+    assertTranslation(implFile, "#import \"foo/Test_Bar.h\"");
+    assertTranslation(implFile, "#import \"foo/Test_Mumble.h\"");
+    assertTranslation(implFile, "#import \"foo/Test_Inner.h\"");
+  }
+
+  // Verify circular includes example at j2objc.org translates correctly.
+  public void testCircularIncludes() throws IOException {
+    options.setSeparateHeaders(true);
+    addSourceFile("class Foo extends Bar {}", "Foo.java");
+    addSourceFile("class Bar {\n"
+            + "  static class Baz extends Foo {}\n"
+            + "}", "Bar.java");
+    runPipeline("Foo.java", "Bar.java");
+    String translation = getTranslatedFile("Foo.h");
+    assertTranslation(translation, "#import \"Bar.h\"");
+    assertTranslation(translation, "@interface Foo : Bar");
+    assertNotInTranslation(translation, "Bar_Baz.h");
+
+    translation = getTranslatedFile("Bar.h");
+    assertTranslation(translation, "@interface Bar : NSObject");
+    assertNotInTranslation(translation, "Foo");
+    assertNotInTranslation(translation, "Bar_Baz");
+
+    translation = getTranslatedFile("Bar_Baz.h");
+    assertTranslation(translation, "#import \"Foo.h\"");
+    assertTranslation(translation, "@interface Bar_Baz : Foo");
+
+    translation = getTranslatedFile("Foo.m");
+    assertTranslation(translation, "#import \"Bar.h\"");
+    assertTranslation(translation, "#import \"Foo.h\"");
+
+    translation = getTranslatedFile("Bar.m");
+    assertTranslation(translation, "#import \"Bar.h\"");
+    assertTranslation(translation, "#import \"Bar_Baz.h\"");
+    assertTranslation(translation, "#import \"Foo.h\"");
+  }
+
+  public void testSeparateHeaderGenerationOfAnnotationTypeWithInnerTypes() throws IOException {
+    options.setSeparateHeaders(true);
+    String source =
+        String.join(
+            "\n",
+            "package foo;",
+            "import java.lang.annotation.*;",
+            "@Retention(RetentionPolicy.CLASS)",
+            "public @interface ObjectiveCAdapterMethod {",
+            "    public enum Adaptation {",
+            "        EXCEPTIONS_AS_ERRORS, RETURN_NATIVE_BOOLS; }",
+            "    Adaptation[] adaptations() default {};",
+            "}");
+    String mainHeader =
+        translateSourceFile(source, "ObjectiveCAdapterMethod", "foo/ObjectiveCAdapterMethod.h");
+    assertTranslation(
+        mainHeader, "@protocol FooObjectiveCAdapterMethod < JavaLangAnnotationAnnotation >");
+    assertTranslation(
+        mainHeader,
+        "@interface FooObjectiveCAdapterMethod : NSObject < FooObjectiveCAdapterMethod >");
+    String enumHeader = getTranslatedFile("foo/ObjectiveCAdapterMethod_Adaptation.h");
+    assertTranslation(
+        enumHeader, "@interface FooObjectiveCAdapterMethod_Adaptation : JavaLangEnum");
+    String implFile = getTranslatedFile("foo/ObjectiveCAdapterMethod.m");
+    assertTranslation(implFile, "ObjectiveCAdapterMethod_Adaptation_Enum");
+  }
 }
