@@ -60,7 +60,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     String translation = translateSourceFile(
         "class Test { static final int FOO = 1; }", "Test", "Test.h");
     assertTranslation(translation, "#define Test_FOO 1");
-    assertTranslation(translation, "J2OBJC_STATIC_FIELD_CONSTANT(Test, FOO, jint)");
+    assertTranslation(translation, "J2OBJC_STATIC_FIELD_CONSTANT(Test, FOO, int32_t)");
   }
 
   // Verify that accessor methods for static vars and constants are generated on request.
@@ -75,10 +75,10 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     assertTranslation(translation, "+ (NSString *)ID;");
     assertTranslation(translation, "+ (void)setID:(NSString *)value;");
     assertTranslation(translation, "+ (Test *)DEFAULT;");
-    assertTranslation(translation, "+ (jboolean)DEBUG_;");
-    assertTranslation(translation, "+ (void)setDEBUG_:(jboolean)value");
-    assertNotInTranslation(translation, "+ (jint)i");
-    assertNotInTranslation(translation, "+ (void)setI:(jint)value");
+    assertTranslation(translation, "+ (bool)DEBUG_;");
+    assertTranslation(translation, "+ (void)setDEBUG_:(bool)value");
+    assertNotInTranslation(translation, "+ (int32_t)i");
+    assertNotInTranslation(translation, "+ (void)setI:(int32_t)value");
     assertNotInTranslation(translation, "+ (void)setDEFAULT:(Test *)value");
   }
 
@@ -86,7 +86,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
   // is annotated with @NullMarked.
   public void testStaticFieldAccessorMethodWithNullMarkedClass() throws IOException {
     String source =
-        "import javax.annotation.*; import org.jspecify.nullness.NullMarked;"
+        "import javax.annotation.*; import org.jspecify.annotations.NullMarked;"
             + "@NullMarked public class Test {"
             + "  @Nullable public static String nullableStr;"
             + "  public static String nonNullStr;"
@@ -107,7 +107,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
   // is annotated with @NullMarked.
   public void testStaticFieldAccessorMethodWithNullMarkedPackage() throws IOException {
     addSourceFile(
-        "@NullMarked package foo.bar; import org.jspecify.nullness.NullMarked;",
+        "@NullMarked package foo.bar; import org.jspecify.annotations.NullMarked;",
         "foo/bar/package-info.java");
     String source =
         "package foo.bar; "
@@ -138,8 +138,8 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     assertNotInTranslation(translation, "+ (NSString *)ID");
     assertNotInTranslation(translation, "+ (void)setID:(NSString *)value");
     assertNotInTranslation(translation, "+ (Test *)DEFAULT");
-    assertNotInTranslation(translation, "+ (jint)i");
-    assertNotInTranslation(translation, "+ (void)setI:(jint)value");
+    assertNotInTranslation(translation, "+ (int32_t)i");
+    assertNotInTranslation(translation, "+ (void)setI:(int32_t)value");
     assertNotInTranslation(translation, "+ (void)setDEFAULT:(Test *)value");
   }
 
@@ -284,7 +284,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         "",
         "@interface Test : NSObject",
         "",
-        "+ (jboolean)FOO;");
+        "+ (bool)FOO;");
   }
 
   public void testProperties() throws IOException {
@@ -305,23 +305,22 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
             + "  public void setAGenericProperty(T value) { }"
             + "}";
     String translation = translateSourceFile(source, "FooBar", "FooBar.h");
-    assertTranslation(translation, "@property (readonly, nonatomic) jint fieldBar;");
+    assertTranslation(translation, "@property (readonly, nonatomic) int32_t fieldBar;");
 
     // Should split out fieldBaz and include the declared getter.
     assertTranslation(translation,
-        "@property (readonly, nonatomic, getter=getFieldBaz) jint fieldBaz;");
+        "@property (readonly, nonatomic, getter=getFieldBaz) int32_t fieldBaz;");
 
     // Set copy for strings and drop readwrite.
     assertTranslation(translation,
         "@property (copy) NSString *fieldCopy;");
 
     // Test boolean getter.
-    assertTranslation(translation,
-        "@property (nonatomic, getter=isFieldBool) jboolean fieldBool;");
+    assertTranslation(translation, "@property (nonatomic, getter=isFieldBool) bool fieldBool;");
 
     // Reorder property attributes and pass setter through.
     assertTranslation(translation,
-        "@property (weak, readonly, nonatomic) jint fieldReorder;");
+        "@property (weak, readonly, nonatomic) int32_t fieldReorder;");
 
     // Test generic property.
     assertTranslation(
@@ -331,7 +330,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
             + "JavaLangThrowable *aGenericProperty;");
 
     // Test readonly property.
-    assertTranslation(translation, "@property (readonly) jint fieldFinal;");
+    assertTranslation(translation, "@property (readonly) int32_t fieldFinal;");
   }
 
   public void testSynchronizedPropertyGetter() throws IOException {
@@ -341,7 +340,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "  public synchronized int getFieldBar() { return fieldBar; }"
         + "}";
     String translation = translateSourceFile(source, "FooBar", "FooBar.h");
-    assertTranslation(translation, "@property (getter=getFieldBar) jint fieldBar;");
+    assertTranslation(translation, "@property (getter=getFieldBar) int32_t fieldBar;");
   }
 
   public void testBadPropertyAttribute() throws IOException {
@@ -442,10 +441,11 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "@Property static int test; "
         + "@Property(\"nonatomic\") static double d; "
         + "@Property static final boolean flag = true; }", "Test", "Test.h");
-    assertTranslatedLines(translation,
-        "@property (class) jint test;",
-        "@property (nonatomic, class) jdouble d;",
-        "@property (readonly, class) jboolean flag;");
+    assertTranslatedLines(
+        translation,
+        "@property (class) int32_t test;",
+        "@property (nonatomic, class) double d;",
+        "@property (readonly, class) bool flag;");
 
     // Verify class attributes aren't assigned to instance fields.
     translateSourceFile(
@@ -492,16 +492,16 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
             "Test.h");
     assertTranslatedLines(
         translation,
-        "@property (class) jint test NS_SWIFT_NAME(test);",
-        "@property (class) jdouble d NS_SWIFT_NAME(d);",
-        "@property (readonly, class) jboolean flag NS_SWIFT_NAME(flag);",
+        "@property (class) int32_t test NS_SWIFT_NAME(test);",
+        "@property (class) double d NS_SWIFT_NAME(d);",
+        "@property (readonly, class) bool flag NS_SWIFT_NAME(flag);",
         "@property (copy, class) NSString *TRUE_ NS_SWIFT_NAME(TRUE_);");
-    assertTranslation(translation, "- (jint)getTest;");
-    assertNotInTranslation(translation, "+ (jint)test;");
+    assertTranslation(translation, "- (int32_t)getTest;");
+    assertNotInTranslation(translation, "+ (int32_t)test;");
     assertNotInTranslation(translation, "@property (copy, class) NSString *s NS_SWIFT_NAME(s);");
 
     translation = getTranslatedFile("Test.m");
-    assertTranslation(translation, "+ (jint)test {");
+    assertTranslation(translation, "+ (int32_t)test {");
   }
 
   public void testPropertiesInInterfaceType() throws IOException {
@@ -517,7 +517,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         "@end",
         "",
         "@interface Test : NSObject",
-        "@property (readonly, class) jboolean FOO NS_SWIFT_NAME(FOO);");
+        "@property (readonly, class) bool FOO NS_SWIFT_NAME(FOO);");
   }
 
   public void testNullabilityAttributes() throws IOException {
@@ -564,7 +564,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         // Verify default nonnull is specified.
         "withId:(id _Nonnull)var",
         // Default should not apply to primitive type.
-        "withInt:(jint)count;");
+        "withInt:(int32_t)count;");
   }
 
   // Verify a ParametersAreNonnullByDefault package annotation sets unspecified
@@ -587,7 +587,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         // Verify default nonnull is specified.
         "withId:(id _Nonnull)var",
         // Default should not apply to primitive type.
-        "withInt:(jint)count;");
+        "withInt:(int32_t)count;");
   }
 
   public void testNullabilityPragmas() throws IOException {
@@ -685,8 +685,8 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
 
     // Verify primitive properties don't have nullability parameters.
     assertTranslatedLines(translation,
-        "@property jint test7;",
-        "@property (readonly, nonatomic) jdouble test8;");
+        "@property int32_t test7;",
+        "@property (readonly, nonatomic) double test8;");
   }
 
   // Verify that constructors are always nonnull (issue #960).
@@ -699,7 +699,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "  private Test(int i) { this.i = i; }}", "Test", "Test.h");
     assertTranslation(translation, "- (instancetype _Nonnull)init;");
     translation = getTranslatedFile("Test.m");
-    assertTranslation(translation, "- (instancetype)initWithInt:(jint)i;");
+    assertTranslation(translation, "- (instancetype)initWithInt:(int32_t)i;");
   }
 
   // Verify type nullability annotations.
@@ -751,7 +751,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
 
   public void testNullMarked() throws IOException {
     addSourceFile(
-        "@NullMarked package foo.bar; import org.jspecify.nullness.NullMarked;",
+        "@NullMarked package foo.bar; import org.jspecify.annotations.NullMarked;",
         "foo/bar/package-info.java");
     String source =
         "package foo.bar; import javax.annotation.*; "
@@ -776,20 +776,19 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         translation, "NS_ASSUME_NONNULL_BEGIN", "@interface FooBarTest : NSObject {");
     assertTranslatedLines(translation, "@public", "NSString *_Nullable identifier_;");
     assertTranslation(translation, "- (NSString * _Nullable)getToken;");
-    assertTranslation(
-        translation, "- (instancetype _Nonnull)initWithNSString:(NSString * _Nullable)token;");
+    assertTranslation(translation, "- (instancetype)initWithNSString:(NSString * _Nullable)token;");
     assertTranslatedLines(
         translation,
         "- (NSString * _Nullable)testNullableInstanceMethodWithNSString:(NSString *"
             + " _Nullable)msg",
         "withId:(id)var",
-        "withInt:(jint)count;");
+        "withInt:(int32_t)count;");
     assertTranslatedLines(
         translation,
         "+ (NSString * _Nullable)testNullableStaticMethodWithNSString:(NSString *"
             + " _Nullable)firstArg",
         "withId:(id)var",
-        "withInt:(jint)count;");
+        "withInt:(int32_t)count;");
     assertTranslatedLines(
         translation, "J2OBJC_TYPE_LITERAL_HEADER(FooBarTest)", "", "NS_ASSUME_NONNULL_END");
   }
@@ -797,7 +796,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
   public void testNullMarkedTypeAnnotation() throws IOException {
     String source =
         "import javax.annotation.*;"
-            + "import org.jspecify.nullness.NullMarked;"
+            + "import org.jspecify.annotations.NullMarked;"
             + "@NullMarked public class Test {"
             + "  private @Nullable String nullableStr;"
             + "  private String nonNullStr;"
@@ -812,6 +811,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     options.setNullability(true);
     String translation = translateSourceFile(source, "Test", "Test.h");
     assertTranslatedLines(translation, "NS_ASSUME_NONNULL_BEGIN", "@interface Test : NSObject");
+    assertTranslation(translation, "- (instancetype)init;");
     assertTranslatedLines(
         translation,
         "- (NSString * _Nullable)testWithNSString:(NSString *)a",
@@ -823,13 +823,12 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
 
   public void testNullMarkedPackageAnnotation() throws IOException {
     addSourceFile(
-        "@NullMarked package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "@NullMarked package foo.bar;" + "import org.jspecify.annotations.NullMarked;",
         "foo/bar/package-info.java");
     String source =
         "package foo.bar;"
             + "import javax.annotation.*;"
-            + "import org.jspecify.nullness.NullMarked;"
-            + "@NullMarked public class Test {"
+            + "public class Test {"
             + "  private @Nullable String nullableStr;"
             + "  private String nonNullStr;"
             + "  public @Nullable String test(String a,"
@@ -844,6 +843,7 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
     assertTranslatedLines(
         translation, "NS_ASSUME_NONNULL_BEGIN", "@interface FooBarTest : NSObject");
+    assertTranslation(translation, "- (instancetype)init;");
     assertTranslatedLines(
         translation,
         "- (NSString * _Nullable)testWithNSString:(NSString *)a",
@@ -853,6 +853,52 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
     assertTranslation(translation, "NS_ASSUME_NONNULL_END");
   }
 
+  public void testNullMarkedWithoutGenerics() throws IOException {
+    addSourceFile(
+        "@NullMarked package foo.bar;" + "import org.jspecify.annotations.NullMarked;",
+        "foo/bar/package-info.java");
+    String source =
+        "package foo.bar;"
+            + "import javax.annotation.*;"
+            + "import org.jspecify.annotations.Nullable;"
+            + "public class Test <T extends @Nullable String> {"
+            + "  public T getNullableStr() { return null; }"
+            + "  public void setNullableStr(T thing) { return; }"
+            + "}";
+    options.setNullMarked(true);
+    options.setNullability(true);
+    String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
+    assertTranslatedLines(
+        translation, "NS_ASSUME_NONNULL_BEGIN", "@interface FooBarTest : NSObject");
+    assertTranslation(translation, "- (NSString * _Nullable)getNullableStr;");
+    assertTranslation(
+        translation, "- (void)setNullableStrWithNSString:(NSString * _Nullable)thing;");
+    assertTranslation(translation, "NS_ASSUME_NONNULL_END");
+  }
+
+  public void testNullMarkedWithGenerics() throws IOException {
+    addSourceFile(
+        "@NullMarked package foo.bar;" + "import org.jspecify.annotations.NullMarked;",
+        "foo/bar/package-info.java");
+    String source =
+        "package foo.bar;"
+            + "import javax.annotation.*;"
+            + "import org.jspecify.annotations.Nullable;"
+            + "public class Test <@Nullable T> {"
+            + "  public T getNullableStr() { return null; }"
+            + "  public void setNullableStr(T thing) { return; }"
+            + "}";
+    options.setNullMarked(true);
+    options.setNullability(true);
+    options.setAsObjCGenericDecl(true);
+    String translation = translateSourceFile(source, "foo.bar.Test", "foo/bar/Test.h");
+    assertTranslatedLines(
+        translation, "NS_ASSUME_NONNULL_BEGIN", "@interface FooBarTest<T> : NSObject");
+    assertTranslation(translation, "- (T _Nullable)getNullableStr;");
+    assertTranslation(translation, "- (void)setNullableStrWithId:(T _Nullable)thing;");
+    assertTranslation(translation, "NS_ASSUME_NONNULL_END");
+  }
+
   public void testFieldWithIntersectionType() throws IOException {
     String translation = translateSourceFile(
         "class Test <T extends Comparable & Runnable> { T foo; }", "Test", "Test.h");
@@ -1012,8 +1058,38 @@ public class TypeDeclarationGeneratorTest extends GenerationTest {
         + "    public static final int anotherState=2;\n"
         + "}\n", "State", "State.h");
     // Should have trailing underscore, to avoid class with class initialize function name.
-    assertTranslation(translation, "J2OBJC_STATIC_FIELD_CONSTANT(State, initialize_, jint)");
-    assertTranslation(translation, "inline jint State_get_initialize_(void);");
+    assertTranslation(translation, "J2OBJC_STATIC_FIELD_CONSTANT(State, initialize_, int32_t)");
+    assertTranslation(translation, "inline int32_t State_get_initialize_(void);");
     assertTranslation(translation, "State_initialize_");
   }
+
+  public void testAddTextSegmentAttribute() throws IOException {
+    options.setAddTextSegmentAttribute(true);
+    String source =
+        "class Test { \n"
+            + "  void methodTest() { \n"
+            + "    System.out.println(\"test\"); \n"
+            + "  } \n"
+            + "  native void nativeTest() /*-[ \n"
+            + "     printf(\"hello\\n\");"
+            + "  ]-*/;"
+            + "  static enum TestEnum { FOO, BAR }"
+            + "  static @interface TestAnnotation {}"
+            + "}";
+    String translation = translateSourceFile(source, "Test", "Test.h");
+    assertTranslatedLines(translation, "- (void)methodTest J2OBJC_TEXT_SEGMENT;");
+    assertTranslatedLines(translation, "- (void)nativeTest J2OBJC_TEXT_SEGMENT;");
+    assertTranslatedLines(translation, "- (instancetype)init J2OBJC_TEXT_SEGMENT;");
+
+    // Automatically generatedn enum methods.
+    assertTranslatedLines(
+        translation,
+        "+ (Test_TestEnum *)valueOfWithNSString:(NSString *)name J2OBJC_TEXT_SEGMENT;");
+    assertTranslatedLines(translation, "+ (IOSObjectArray *)values J2OBJC_TEXT_SEGMENT;");
+    assertTranslatedLines(translation, "- (Test_TestEnum_Enum)toNSEnum J2OBJC_TEXT_SEGMENT;");
+
+    // Automatically generated annotation methods.
+    assertTranslatedLines(translation, "- (bool)isEqual:(id)obj J2OBJC_TEXT_SEGMENT;");
+    assertTranslatedLines(translation, "- (NSUInteger)hash J2OBJC_TEXT_SEGMENT;");
+  }
 }
