@@ -14,6 +14,8 @@
 
 package com.google.devtools.j2objc.pipeline;
 
+import com.google.common.collect.ImmutableMap;
+import com.google.common.collect.ImmutableSet;
 import com.google.devtools.j2objc.GenerationTest;
 import com.google.devtools.j2objc.J2ObjC;
 import com.google.devtools.j2objc.file.RegularInputFile;
@@ -88,4 +90,60 @@ public class TranslationProcessorTest extends GenerationTest {
     // Verify A.java wasn't compiled; it has a B reference, but B doesn't depend on it.
     assertFalse(new File(tempDir, "A.m").exists());
   }
+
+  public void testProcessIncludeCycleWithSegmentedHeaders() throws IOException {
+    options.load(new String[] {"--segmented-headers"});
+    doProcessIncludeCycle();
+
+    assertNoErrors();
+    assertNoWarnings();
+  }
+
+  public void testProcessIncludeCycleWithoutSegmentedHeaders() throws IOException {
+    options.load(new String[] {"--no-segmented-headers"});
+    doProcessIncludeCycle();
+
+    assertErrorRegex("This target contains an include cycle[\\s\\S]*");
+  }
+
+  private void doProcessIncludeCycle() throws IOException {
+    String fileA = addSourceFile("interface A {} class BI implements B {}", "A.java");
+    String fileB = addSourceFile("interface B {} class AI implements A {}", "B.java");
+
+    GenerationBatch batch = new GenerationBatch(options);
+    batch.addSource(new RegularInputFile(fileA, "A.java"));
+    batch.addSource(new RegularInputFile(fileB, "B.java"));
+
+    TranslationProcessor processor = new TranslationProcessor(J2ObjC.createParser(options), null);
+    processor.processInputs(batch.getInputs());
+  }
+
+  public void testCheckNoIncludeCyclesWithoutCycles() {
+    TranslationProcessor.checkNoIncludeCycles(
+        ImmutableMap.of(
+            "A.h",
+            ImmutableSet.of("B.h", "C.h", "D.h"),
+            "B.h",
+            ImmutableSet.of("C.h", "D.h"),
+            "C.h",
+            ImmutableSet.of("D.h")));
+
+    assertNoErrors();
+    assertNoWarnings();
+  }
+
+  public void testCheckNoIncludeCyclesWithCycles() {
+    TranslationProcessor.checkNoIncludeCycles(
+        ImmutableMap.of(
+            "A.h",
+            ImmutableSet.of("B.h", "E.h"),
+            "B.h",
+            ImmutableSet.of("C.h", "E.h"),
+            "C.h",
+            ImmutableSet.of("D.h"),
+            "D.h",
+            ImmutableSet.of("A.h")));
+
+    assertErrorRegex("This target contains an include cycle[\\s\\S]*");
+  }
 }
