@@ -179,9 +179,9 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     String translation = translateSourceFile(source, "Foo", "Foo.h");
     assertTranslation(translation, "#define Foo_y 0");
     translation = getTranslatedFile("Foo.m");
-    assertNotInTranslation(translation, "jint Foo_x_");
+    assertNotInTranslation(translation, "int32_t Foo_x_");
     assertNotInTranslation(translation, "Foo_x_ = Foo_f()");
-    assertNotInTranslation(translation, "+ (jint)f");
+    assertNotInTranslation(translation, "+ (int32_t)f");
     assertNotInTranslation(translation, "bar");
   }
 
@@ -263,7 +263,7 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertNotInTranslation(translation, "Foo *this$0_;");
     assertNotInTranslation(translation, "JreStrongAssign(&self->this$0_, outer$");
     assertNotInTranslation(translation, "self->z_ = x;");
-    assertNotInTranslation(translation, "- (jint)f");
+    assertNotInTranslation(translation, "- (int32_t)f");
   }
 
   public void testDeadClass_supertypeRemoval() throws IOException {
@@ -619,7 +619,7 @@ public class DeadCodeEliminatorTest extends GenerationTest {
 
   public void testDeadClass_stringConstants_nullMarked() throws IOException {
     addSourceFile(
-        "@NullMarked" + "package foo.bar;" + "import org.jspecify.nullness.NullMarked;",
+        "@NullMarked" + "package foo.bar;" + "import org.jspecify.annotations.NullMarked;",
         "foo/bar/package-info.java");
     String source =
         "package foo.bar;"
@@ -643,7 +643,7 @@ public class DeadCodeEliminatorTest extends GenerationTest {
     assertTranslation(translation, "FOUNDATION_EXPORT NSString *_Nullable FooBarA_B_FOO;");
 
     translation = getTranslatedFile("foo/bar/A.m");
-    assertTranslation(translation, "NSString *_Nullable FooBarA_B_FOO = @\"foo\";");
+    assertTranslation(translation, "NSString *FooBarA_B_FOO = @\"foo\";");
     assertTranslation(translation, "static NSString *FooBarA_B_BAR = @\"bar\";");
   }
 
