@@ -33,9 +33,9 @@ public class EnhancedForRewriterTest extends GenerationTest {
         + "void test() { for (char c : charArray) {} for (char c : charList) {} } }";
     String translation = translateSourceFile(source, "A", "A.m");
     assertTranslation(translation,
-        "jchar c = [((JavaLangCharacter *) nil_chk(*b__++)) charValue];");
+        "unichar c = [((JavaLangCharacter *) nil_chk(*b__++)) charValue];");
     assertTranslation(translation,
-        "jchar c = [((JavaLangCharacter *) nil_chk(boxed__)) charValue];");
+        "unichar c = [((JavaLangCharacter *) nil_chk(boxed__)) charValue];");
   }
 
   public void testEnhancedForLoopAnnotation() throws IOException {
