@@ -30,7 +30,7 @@ public class EnumRewriterTest extends GenerationTest {
         "enum Test { A(\"foo\"); private <T> Test(T t) {} }", "Test", "Test.m");
     assertTranslation(translation,
         "void Test_initWithId_withNSString_withInt_("
-          + "Test *self, id t, NSString *__name, jint __ordinal) {");
+          + "Test *self, id t, NSString *__name, int32_t __ordinal) {");
     assertTranslatedLines(translation,
         "((void) (JreEnum(Test, A) = e = "
           + "objc_constructInstance(self, (void *)ptr)), ptr += objSize);",
@@ -50,6 +50,15 @@ public class EnumRewriterTest extends GenerationTest {
         "- (Test_Enum)toNSEnum {",
         "  return (Test_Enum)[self ordinal];",
         "}");
+    assertTranslatedLines(
+        translation,
+        "+ (Test *)fromNSEnum:(Test_Enum)nativeValue {",
+        "  Test *javaEnum = Test_fromOrdinal(nativeValue);",
+        "  if (!javaEnum) @throw"
+            + " create_JavaLangIllegalArgumentException_initWithNSString_(@\"NSEnum Test_Enum out"
+            + " of range.\");",
+        "  return javaEnum;",
+        "}");
   }
 
   public void testEmptyEnum() throws Exception {
@@ -67,7 +76,7 @@ public class EnumRewriterTest extends GenerationTest {
         "size_t allocSize = 5 * objSize;",
         "uintptr_t ptr = (uintptr_t)calloc(allocSize, 1);",
         "id e;",
-        "for (jint i = 0; i < 5; i++) {",
+        "for (int32_t i = 0; i < 5; i++) {",
         "((void)(Test_values_[i] = e = "
           + "objc_constructInstance(self, (void *)ptr)), ptr += objSize);",
         "Test_initWithNSString_withInt_(e, JreEnumConstantName(Test_class_(), i), i);",
@@ -116,7 +125,7 @@ public class EnumRewriterTest extends GenerationTest {
         "id names[] = {",
         "@\"A\", @\"B\", @\"C\", @\"D\", @\"E\",",
         "};",
-        "for (jint i = 0; i < 5; i++) {",
+        "for (int32_t i = 0; i < 5; i++) {",
         "((void)(Test_values_[i] = e = "
           + "objc_constructInstance(self, (void *)ptr)), ptr += objSize);",
         "Test_initWithNSString_withInt_(e, names[i], i);",
