@@ -65,8 +65,9 @@ public class JavaToIOSMethodTranslatorTest extends GenerationTest {
     String translation = translateSourceFile(
         "class Test { public String toString(boolean value) { return String.valueOf(value); } }",
         "Test", "Test.m");
-    assertTranslatedLines(translation,
-        "- (NSString *)toStringWithBoolean:(jboolean)value {",
+    assertTranslatedLines(
+        translation,
+        "- (NSString *)toStringWithBoolean:(bool)value {",
         "return NSString_java_valueOfBool_(value);");
   }
 
@@ -91,7 +92,7 @@ public class JavaToIOSMethodTranslatorTest extends GenerationTest {
     List<Statement> stmts = translateStatements(source);
     assertEquals(1, stmts.size());
     String result = generateStatement(stmts.get(0));
-    assertEquals("jint test = ((jint) [@\"foo\" hash]);", result);
+    assertEquals("int32_t test = ((int32_t) [@\"foo\" hash]);", result);
   }
 
   public void testClassGetSuperclass() throws IOException {
@@ -142,7 +143,7 @@ public class JavaToIOSMethodTranslatorTest extends GenerationTest {
     List<Statement> stmts = translateStatements(source);
     assertEquals(9, stmts.size());
     String result = generateStatement(stmts.get(1));
-    assertEquals("jint idx = [s java_indexOf:'g'];", result);
+    assertEquals("int32_t idx = [s java_indexOf:'g'];", result);
     result = generateStatement(stmts.get(2));
     assertEquals("idx = [s java_indexOfString:@\"brillig\"];", result);
     result = generateStatement(stmts.get(3));
