@@ -32,7 +32,7 @@ public class NumberMethodRewriterTest extends GenerationTest {
       + "  public float floatValue() { return 0.0f; }"
       + "  public int intValue() { return 0; }"
       + "  public long longValue() { return 0L; }}", "A", "A.m");
-    assertTranslatedLines(translation, "- (jboolean)isEqual:(id)obj {", "return self == obj;");
+    assertTranslatedLines(translation, "- (bool)isEqual:(id)obj {", "return self == obj;");
     assertTranslatedLines(translation, "- (NSUInteger)hash {", "return (NSUInteger)self;");
   }
 
@@ -46,7 +46,7 @@ public class NumberMethodRewriterTest extends GenerationTest {
       + "  public long longValue() { return 0L; }"
       + "  public boolean equals(Object obj) { return this == obj; }"
       + "  public int hashCode() { return 0; }}", "A", "A.m");
-    assertOccurrences(translation, "- (jboolean)isEqual:(id)obj", 1);
+    assertOccurrences(translation, "- (bool)isEqual:(id)obj", 1);
     assertOccurrences(translation, "- (NSUInteger)hash", 1);
   }
 
@@ -65,10 +65,10 @@ public class NumberMethodRewriterTest extends GenerationTest {
       + "  public long longValue() { return 0L; }}", "A.java");
     runPipeline(testNumberTypeSource, aSource);
     String testNumberGen = getTranslatedFile("TestNumberType.m");
-    assertOccurrences(testNumberGen, "- (jboolean)isEqual:(id)obj", 1);
+    assertOccurrences(testNumberGen, "- (bool)isEqual:(id)obj", 1);
     assertOccurrences(testNumberGen, "- (NSUInteger)hash", 1);
     String subClassGen = getTranslatedFile("A.m");
-    assertNotInTranslation(subClassGen, "- (jboolean)isEqual:(id)obj");
+    assertNotInTranslation(subClassGen, "- (bool)isEqual:(id)obj");
     assertNotInTranslation(subClassGen, "- (NSUInteger)hash");
   }
 
@@ -83,13 +83,53 @@ public class NumberMethodRewriterTest extends GenerationTest {
             + "  public int intValue() { return 0; }"
             + "  public long longValue() { return 0L; }}", "A", "A.m");
 
-    assertTranslation(translation, "- (instancetype)initWithLongLong:(jlong)value {");
-    assertTranslation(translation, "void A_initWithLongLong_(A *self, jlong value) {");
-    assertTranslation(translation, "A *new_A_initWithLongLong_(jlong value) {");
-    assertTranslation(translation, "A *create_A_initWithLongLong_(jlong value) {");
+    assertTranslation(translation, "- (instancetype)initWithLongLong:(int64_t)value {");
+    assertTranslation(translation, "void A_initWithLongLong_(A *self, int64_t value) {");
+    assertTranslation(translation, "A *new_A_initWithLongLong_(int64_t value) {");
+    assertTranslation(translation, "A *create_A_initWithLongLong_(int64_t value) {");
     assertTranslatedLines(translation,
-        "- (A *)valueOfWithLong:(jlong)value {",
+        "- (A *)valueOfWithLong:(int64_t)value {",
         "return create_A_initWithLongLong_(value);",
         "}");
   }
+
+  public void testLongSuperConstructor() throws IOException {
+    String translation =
+        translateSourceFile(
+            "import java.math.*;"
+                + "class A extends BigDecimal { "
+                + "  public A(long value) { super(value); }"
+                + "  public A(long value, MathContext context) { super(value, context); }"
+                + "}",
+            "A",
+            "A.m");
+
+    // Verify that long constructor uses longlong type.
+    assertTranslatedLines(
+        translation,
+        "- (instancetype)initWithLongLong:(int64_t)value {",
+        "  A_initWithLongLong_(self, value);",
+        "  return self;",
+        "}");
+    assertTranslatedLines(
+        translation,
+        "void A_initWithLongLong_(A *self, int64_t value) {",
+        "  JavaMathBigDecimal_initWithLongLong_(self, value);",
+        "}");
+
+    // Verify that the other constructor is not affected (long used instead of longlong).
+    assertTranslatedLines(
+        translation,
+        "- (instancetype)initWithLong:(int64_t)value",
+        "    withJavaMathMathContext:(JavaMathMathContext *)context {",
+        "  A_initWithLong_withJavaMathMathContext_(self, value, context);",
+        "  return self;",
+        "}");
+    assertTranslatedLines(
+        translation,
+        "void A_initWithLong_withJavaMathMathContext_(A *self, int64_t value, JavaMathMathContext"
+            + " *context) {",
+        "  JavaMathBigDecimal_initWithLong_withJavaMathMathContext_(self, value, context);",
+        "}");
+  }
 }
