@@ -160,12 +160,12 @@ public class NameTableTest extends GenerationTest {
     String translation = translateSourceFile("A", "A.h");
     assertTranslatedLines(translation,
         "- (void)test:(NSString *)s",
-        "offset:(jint)n;");
+        "offset:(int32_t)n;");
     assertNotInTranslation(translation, "testWithNSString:");
     translation = getTranslatedFile("A.m");
     assertTranslatedLines(translation,
         "- (void)test:(NSString *)s",
-        "offset:(jint)n {");
+        "offset:(int32_t)n {");
     assertNotInTranslation(translation, "testWithNSString:");
 
     // Test invocation of renamed method.
@@ -180,16 +180,16 @@ public class NameTableTest extends GenerationTest {
         + "static void test(String s, int n) {}}", "Test", "Test.h");
     assertTranslatedLines(translation,
         "+ (void)fooWithNSString:(NSString *)s",
-        "                withInt:(jint)n;");
-    assertTranslation(translation, "FOUNDATION_EXPORT void Test_foo(NSString *s, jint n);");
+        "                withInt:(int32_t)n;");
+    assertTranslation(translation, "FOUNDATION_EXPORT void Test_foo(NSString *s, int32_t n);");
     translation = getTranslatedFile("Test.m");
     assertTranslatedLines(translation,
         "+ (void)fooWithNSString:(NSString *)s",
-        "                withInt:(jint)n {",
+        "                withInt:(int32_t)n {",
         "  Test_foo(s, n);",
         "}");
     assertTranslatedLines(translation,
-        "t_foo(NSString *s, jint n) {",
+        "t_foo(NSString *s, int32_t n) {",
         "  Test_initialize();",
         "}");
   }
@@ -202,12 +202,12 @@ public class NameTableTest extends GenerationTest {
     String translation = translateSourceFile("A", "A.h");
     assertTranslatedLines(translation,
         "- (instancetype)init:(NSString *)s",
-        "offset:(jint)n;");
+        "offset:(int32_t)n;");
     assertNotInTranslation(translation, "testWithNSString");
     translation = getTranslatedFile("A.m");
     assertTranslatedLines(translation,
         "- (instancetype)init:(NSString *)s",
-        "offset:(jint)n {");
+        "offset:(int32_t)n {");
     assertNotInTranslation(translation, "testWithNSString");
 
     // Test invocation of renamed constructor.
@@ -274,7 +274,7 @@ public class NameTableTest extends GenerationTest {
     String translation = translateSourceFile("foo.bar.Test", "foo/bar/Test.h");
     assertTranslatedLines(
         translation,
-        "typedef NS_ENUM(jint, FBTest_Enum) {",
+        "typedef NS_ENUM(int32_t, FBTest_Enum) {",
         "  FBTest_Enum_FOO NS_SWIFT_NAME(foo) = 0,",
         "  FBTest_Enum_BAR NS_SWIFT_NAME(bar) = 1,",
         "};");
@@ -327,19 +327,27 @@ public class NameTableTest extends GenerationTest {
     String translation = translateSourceFile(source, "Test", "Test.h");
     assertTranslation(translation, "- (void)aReservedMethodName__;");
     assertTranslatedLines(translation,
-        "- (void)testWithInt:(jint)noReserved",
-        "            withInt:(jint)stdin_",
-        "            withInt:(jint)aReservedParamName_;");
+        "- (void)testWithInt:(int32_t)noReserved",
+        "            withInt:(int32_t)stdin_",
+        "            withInt:(int32_t)aReservedParamName_;");
   }
 
-  public void testSwiftEnumName() {
-    assertEquals("banana", NameTable.getSwiftEnumName("BANANA"));
-    assertEquals("greenGrape", NameTable.getSwiftEnumName("GREEN_GRAPE"));
-    assertEquals("pearAppleSmoothie", NameTable.getSwiftEnumName("PEAR_APPLE_SMOOTHIE"));
-    assertEquals("pineapple", NameTable.getSwiftEnumName("pineapple"));
-    assertEquals("redGrape", NameTable.getSwiftEnumName("red_grape"));
-    assertEquals("appleGrapeJuice", NameTable.getSwiftEnumName("apple_grape_juice"));
-    assertEquals("false_", NameTable.getSwiftEnumName("FALSE"));
-    assertEquals("true_", NameTable.getSwiftEnumName("true"));
+  public void testNativeCamelCaseName() {
+    assertEquals("banana", NameTable.camelCaseName("BANANA", false));
+    assertEquals("Banana", NameTable.camelCaseName("BANANA", true));
+    assertEquals("greenGrape", NameTable.camelCaseName("GREEN_GRAPE", false));
+    assertEquals("GreenGrape", NameTable.camelCaseName("GREEN_GRAPE", true));
+    assertEquals("pearAppleSmoothie", NameTable.camelCaseName("PEAR_APPLE_SMOOTHIE", false));
+    assertEquals("PearAppleSmoothie", NameTable.camelCaseName("PEAR_APPLE_SMOOTHIE", true));
+    assertEquals("pineapple", NameTable.camelCaseName("pineapple", false));
+    assertEquals("Pineapple", NameTable.camelCaseName("pineapple", true));
+    assertEquals("redGrape", NameTable.camelCaseName("red_grape", false));
+    assertEquals("RedGrape", NameTable.camelCaseName("red_grape", true));
+    assertEquals("appleGrapeJuice", NameTable.camelCaseName("apple_grape_juice", false));
+    assertEquals("AppleGrapeJuice", NameTable.camelCaseName("apple_grape_juice", true));
+    assertEquals("false_", NameTable.camelCaseName("FALSE", false));
+    assertEquals("False", NameTable.camelCaseName("FALSE", true)); // Not reserved name
+    assertEquals("true_", NameTable.camelCaseName("true", false));
+    assertEquals("True", NameTable.camelCaseName("true", true)); // Not reserved name
   }
 }
